
#ifndef _acquire_H
#define _acquire_H

#include <apt-pkg/acquire.h>

/*
 * pkgAcquireBackend: interface to package acquisition and status
 * notification interfaces
 *
 * This is a pretty complex interface, partly because of the use of MI.
 * The idea is to make this almost self sufficient so that it can be
 * forked off and run in separate threads/processes if needed.
 *
 */
#include "../interface/edeityui.h"
#include "../interface/acquireui.h"

class EDeityUI;

/* hmm... multiple inheritence.... */
class pkgAcquireBackend: public pkgAcquire, public pkgAcquireStatus
{
protected:

	bool NoCacheInit;
	int signalfd;
	EDeityUI *Master;

	unsigned int ID;
	
	void _AcquireUpdate();
	void _AcquireRun();
	static void *DoAcquireUpdate(void *);
	static void *DoAcquireRun(void *);

	void AckErrors();

public:
	/* methods from pkgAcquireStatus */
	// Called to change media
	virtual bool MediaChange(string Media,string Drive);
   
	// Each of these is called by the workers when an event occures
	virtual void IMSHit(pkgAcquire::ItemDesc &/*Itm*/);
	virtual void Fetch(pkgAcquire::ItemDesc &/*Itm*/);
	virtual void Done(pkgAcquire::ItemDesc &/*Itm*/);
	virtual void Fail(pkgAcquire::ItemDesc &/*Itm*/);
	virtual bool Pulse(pkgAcquire *Owner); // returns false on user cancel
	virtual void Start();
	virtual void Stop();

	int AcquireUpdate();
	int AcquireRun();
	int AcquireUpgrade();

	pkgAcquireBackend(EDeityUI *);
	~pkgAcquireBackend();
};

#endif // _acquire_H
