/* $Id: backend.cc,v 1.10 2001/02/25 17:01:24 tausq Exp $ */
#ifdef USE_DEBCONF
#include <std.h>
#include "backend.h"
#include "itf.h"
#include "util.h"

//old #define _GNU_SOURCE
//old #include <stdlib.h>
//old #include <libio.h>
//old #include <stdio.h>
//old #include <fcntl.h>
//old #include <unistd.h>
//old#include <string.h>
//old #include <signal.h>
//old #include <sys/socket.h>
//old #include <sys/stat.h>
//old #include <sys/time.h>
//old #include <sys/types.h>
//old #innclude <sys/un.h>

#include <apt-pkg/fileutl.h>

DebConfBackend::Command DebConfBackend::Commands[] = {
	{ "CAPB", &DebConfBackend::DoCapb },
	{ "INPUT", &DebConfBackend::DoInput },
	{ "DATA", &DebConfBackend::DoData },
	{ "GO", &DebConfBackend::DoGo },
	{ "GET", &DebConfBackend::DoGet },
	{ "SET", &DebConfBackend::DoSet },
	{ "TITLE", &DebConfBackend::DoTitle },
	{ "STOP", &DebConfBackend::DoStop },
	{ 0, 0 }
};

DebConfBackend::DebConfBackend(DebConfItf &_itf) : itf(_itf)
{
	int fd;
	struct sockaddr_un addr;
	char buf[256];
	svr = client = -1;

	bufptr = buf;
	bufavail = sizeof(buf);

	status = false;
	connected = false;
	sawget = false;

	snprintf(socketfile, sizeof(socketfile), DEBCONFIPCDIR "/debconfipc.%d", getpid());
	unlink(socketfile);

	if ((fd = socket(PF_UNIX, SOCK_STREAM, 0)) < 0)
		return;
	memset(&addr, 0, sizeof(addr));
	addr.sun_family = AF_UNIX;
	snprintf(addr.sun_path, sizeof(addr.sun_path), socketfile);
	if (bind(fd, (struct sockaddr *)&addr, sizeof(addr)) < 0)
	{
		perror(_("bind"));		
		return;
	}
	if (chmod(socketfile, S_IRUSR | S_IWUSR) < 0)
	{
		perror(_("chmod"));
		return;
	}
	if (listen(fd, 1) < 0)
	{
		perror(_("listen"));
		return;
	}
	
	svr = fd;
	setenv("DEBCONF_PIPE", socketfile, 1);
	setenv("DEBIAN_FRONTEND", "Passthrough", 1);
	setenv("DEBCONF_DEBUG", ".*", 1);
	printf(_("Ready to listen on %s\n"), getenv("DEBCONF_PIPE"));
	status = true;
}

DebConfBackend::~DebConfBackend()
{
	itf.Shutdown();
	if (client > 0)
	{
		close(client);
		client = -1;
	}

	if (svr > 0)
	{
		close(svr);
		svr = -1;
	}
	unlink(socketfile);
	// putenv("DEBCONF_PIPE="); // removes it from the environment
}

void DebConfBackend::Dispatch(const char *line)
{
	int i = 0;
	char *params = 0;

	params = strchr(line, ' ');
	if (params) *params++ = 0;

	while (Commands[i].command != 0) {
		if (strcmp(line, Commands[i].command) == 0) {
			(this->*Commands[i].fn)(params);
			break;
		}
		i++;
	}
}

void DebConfBackend::SendToClient(char *fmt, ...)
{
	va_list ap;
	char buf[4096];

	va_start(ap, fmt);
	vsnprintf(buf, sizeof(buf), fmt, ap);
	va_end(ap);

	write(client, buf, strlen(buf));
}

void DebConfBackend::DoCapb(char *params)
{
	itf.SetCapbBackup(false);	
	if (params == 0) /* getting capabilities */
	{
		SendToClient("0 backup\n");
	}
	else  /* setting capabilities */
	{
		if (strcmp(params, "backup") == 0)
		{
			itf.SetCapbBackup(true);	
			SendToClient("0 OK\n");
		}
	}
}

void DebConfBackend::DoInput(char *params)
{
	char *tag = 0, *type = 0;

	if (sawget == true)
	{
		itf.questions.Clear();
		itf.answers.Clear();
		sawget = false;
	}
	
	tag = params;
	if (tag) type = strchr(tag, ' ');
	if (type) *type++ = 0;

	if (tag != 0 && type != 0) 
	{
		Question *q = itf.questions.Find(tag);
		if (q == 0) q = itf.questions.Add(tag);

		q->SetType(type);
		SendToClient("0 OK\n");
	}
	else
		SendToClient("100 no tag or type to INPUT\n");
}

void DebConfBackend::DoData(char *params)
{
	char *tag = 0, *type = 0, *value = 0;

	if (sawget == true)
	{
		itf.questions.Clear();
		itf.answers.Clear();
		sawget = false;
	}
	
	tag = params;
	if (tag) type = strchr(tag, ' ');
	if (type) 
	{
		*type++ = 0;
		
		value = strchr(type, ' ');
		if (value) *value++ = 0;
	}
	if (tag != 0 && type != 0 && value != 0) 
	{
		Question *q = itf.questions.Find(tag);
		if (q == 0) q = itf.questions.Add(tag);

		if (strcmp(type, "description") == 0)
		{
			q->desc = collapsenewline(value);
		} 
		else if (strcmp(type, "extended-description") == 0)
		{
			q->ext_desc = collapsenewline(value);
		}
		else if (strcmp(type, "choices") == 0)
		{
			q->choices = collapsenewline(value);
		}
		SendToClient("0 OK\n");
	}
	else
		SendToClient("100 no tag or type or data to DATA\n");
}


void DebConfBackend::DoSet(char *params)
{
	char *tag = 0, *value = 0;

	tag = params;
	if (tag) value = strchr(tag, ' ');
	if (value) *value++ = 0;

	if (tag != 0 && value != 0) 
	{
		Question *q = itf.questions.Find(tag);
		if (q == 0) q = itf.questions.Add(tag);
		q->defaultval = cppstrdup(value);
		SendToClient("0 OK\n");
	}
	else
		SendToClient("100 no tag or value to SET\n");
}

void DebConfBackend::DoGo(char *params)
{
	int ret = itf.AskQuestions();

	switch (ret) 
	{
		case 0: 
			SendToClient("0 OK\n"); 
			break;
		case DEBCONF_GOBACK: 
			itf.questions.Clear();
			itf.answers.Clear();
			SendToClient("30 GOBACK\n");
			break;
		default:
			SendToClient("100 Non-zero return code from GO\n");
	
	}
}

void DebConfBackend::DoGet(char *params)
{
	sawget = true;

	if (params == 0)
		SendToClient("100 no tag to GET\n");
	else
	{
		for (Answer *a = itf.answers.Head(); a != 0; a = a->next)
			if (strcmp(a->tag, params) == 0)
			{
				SendToClient("0 %s\n", a->value);
				return;
			}
		SendToClient("100 unknown tag %s\n", params);
	}
}

void DebConfBackend::DoTitle(char *params)
{
	itf.SetTitle(params);
	SendToClient("0 OK\n");
}

void DebConfBackend::DoStop(char *params)
{
	SendToClient("0 OK\n");
	itf.Shutdown();
}

int DebConfBackend::Input(int src)
{
	/* This is somewhat broken */
	char *line, *p;

	if (connected == false) 
	{
		ASSERT(src == svr);
		struct sockaddr_un peer;
		memset(&peer, 0, sizeof(peer));
		socklen_t addrlen = sizeof(peer);
		client = accept(src, (struct sockaddr *)&peer, &addrlen);
		if (client > 0)
		{
			connected = true;
			itf.ClientConnected(client);
			return 1;
		}
		else
			return -1;
	} 
	else 
	{

		int bytesread = read(src, bufptr, bufavail);
		if (bytesread > 0)
		{
			line = buf;
			while ((p = strchr(line, '\n')) != 0) {
				*p = 0;
				Dispatch(line);
				line = p + 1;
			}
			int len = bufptr + bytesread - line; 
			if (len > 0) {
				memmove(buf, line, len);
				bufptr = buf + len;
				bufavail = sizeof(buf) - len;
			} else {
				bufptr = buf;
				bufavail = sizeof(buf);
			}
		}
		return 0;
	}
}

bool DebConfPreconfigure(const char *deb)
{
	int pipes[2];
	if (deb == 0) return false;
	if (pipe(pipes) != 0) return false;

	printf(_("preconfiguring %s\n"), deb);
	SetCloseExec(pipes[0], true);
	SetCloseExec(pipes[1], true);

	pid_t child = ExecFork();
	if (child == 0)
	{
		dup2(pipes[0], STDIN_FILENO);
		SetCloseExec(STDOUT_FILENO, false);
		SetCloseExec(STDIN_FILENO, false);
		SetCloseExec(STDERR_FILENO, false);

		const char *args[] = { "/bin/sh", "-c", PRECONFIGURESCRIPT, 
			FRONTENDPARAMS, 0 };
		execv(args[0], (char **)args);
		_exit(0);
	}
	close(pipes[0]);

	if (write(pipes[1], deb, strlen(deb)) < 0 ||
	    write(pipes[1], "\n", 1) < 0)
	{
	    	// can't talk to the child.. :-(
		kill(child, SIGINT);
		close(pipes[1]);
		ExecWait(child, PRECONFIGURESCRIPT, true);
		return false;
	}
	return true;
}
#endif
