/* $Id: questions.cc,v 1.4 2001/02/25 17:01:24 tausq Exp $ */
#ifdef USE_DEBCONF
#include "questions.h"
#include "util.h"

//old #include <stdio.h>

Question::Question(const char *_tag) 
{
	tag = cppstrdup(_tag);
	type = QT_NOTE;
	defaultval = 0;
	desc = 0;
	ext_desc = 0;
	choices = 0;
	next = 0;
}

Question::~Question()
{
	delete[] tag;
	delete[] defaultval;
	delete[] desc;
	delete[] ext_desc;
	delete[] choices;
}

void Question::SetType(const char *_type)
{
	if (strcmp(_type, "boolean") == 0)
		type = QT_BOOLEAN;
	else if (strcmp(_type, "note") == 0)
		type = QT_NOTE;
	else if (strcmp(_type, "string") == 0)
		type = QT_STRING;
	else if (strcmp(_type, "password") == 0)
		type = QT_PASSWORD;
	else if (strcmp(_type, "text") == 0)
		type = QT_TEXT;
	else if (strcmp(_type, "select") == 0)
		type = QT_SELECT;
	else if (strcmp(_type, "multiselect") == 0)
		type = QT_MULTISELECT;
}

void Question::print()
{
	printf(_("\tTag: %s\n"), tag);
	printf(_("\tDefault: %s\n"), defaultval ? defaultval : _("(none)");
	printf(_("\tDescription: %s\n"), desc ? desc : _("(none)"));
	printf(_("\tExtended: %s\n"), ext_desc ? ext_desc : _("(none)"));
	printf(_("\tChoices: %s\n"), choices ? choices : _("(none)"));
}

QuestionList::QuestionList()
{
	questions = 0;
}

QuestionList::~QuestionList()
{
	Clear();
}

Question *QuestionList::Add(const char *_tag)
{
	Question *newq = new Question(_tag);
	if (questions == 0)
		questions = newq;
	else
	{
		Question *c = questions;
		while (c->next != 0) c = c->next;
		c->next = newq;
	}

	return newq;
}

Question *QuestionList::Find(const char *_tag)
{
	Question *c = questions;
	while (c)
	{
		if (strcmp(c->tag, _tag) == 0)
			return c;
		c = c->next;
	}
	return 0;
}

void QuestionList::Clear()
{
	Question *c = questions;
	while (c)
	{
		Question *tmp = c;
		c = c->next;
		delete tmp;
	}
	questions = 0;
}

#endif
