#ifdef DEBUG
#include "debug.h"
#include <init/config.h>
//old #include <stdio.h>
//old #include <stdlib.h>
//old #include <stdarg.h>
//old #include <ctype.h>
//old #include <unistd.h>
//old #include <time.h>

/*
 * Function: debug_printf
 * Input: int level - level of debug message
 *        char *fmt - format of message to write (a la printf)
 *        ... - other args for format string
 * Output: none
 * Description: prints a debug message to the screen if the level of the 
 *              message is higher than the current debug level
 * Assumptions: "remembers" the debug log level and log file
 *
 * Tausq: This function is not re-entrant.  We use threads.
 */
void debug_printf(int level, char *fmt, ...)
{
	static int loglevel = -1;
	static FILE *fp = NULL;
	va_list ap;
	time_t now = 0;

	if (loglevel < 0)
	{
		loglevel = (getenv("DEITY_DEBUG") 
			? atoi(getenv("DEITY_DEBUG")) : 0);

		if (getenv("DEITY_DEBUGFILE") == NULL ||
		    (fp = fopen(getenv("DEITY_DEBUGFILE"), "a")) == NULL)
			fp = stderr;

		now = time(NULL);
		fprintf(fp, PACKAGE " " VERSION _(" logging started (lvl = %d) at %s"), loglevel, ctime(&now));
	}

	if (level <= loglevel)
	{
		va_start(ap, fmt);
		vfprintf(fp, fmt, ap);
		va_end(ap);

		fflush(fp);
	}
}
#endif
