/* $Id: dependency.cc,v 1.10 2001/05/23 22:41:12 ltd Exp $ */
#include <std.h>
#include <func/apt.h>
#include <func/config.h>
#include <list/pkgtree.h>
#include <list/screens.h>
#include <interface/edeityui.h>
#include "dependency.h"

bool check_dependencies(bool AllowBroken)
{
	if (_error->PendingError() == true)
		return false;

	// Apply corrections for half-installed packages
	if (pkgApplyStatus(*Cache) == false)
		return false;

	// Nothing is broken
	if ((*Cache)->BrokenCount() == 0 || AllowBroken == true)
		return true;

	string s;

	// Attempt to fix broken things
	if (_Conf.GetBool("edeity.fix_broken_packages") == true)
	{
		if (pkgFixBroken(*Cache) == false || (*Cache)->BrokenCount() != 0)
		{
			capt_show_broken(*Cache);
			return _error->Error(_("Unable to correct dependencies"));
		}
		if (pkgMinimizeUpgrade(*Cache) == false)
			return _error->Error(_("Unable to minimize the upgrade set"));
	}
	else
	{
		capt_show_broken(*Cache);
		return _error->Error(_("Unmet dependencies. Unable to fix broken packages.\nTry setting 'edeity.fix_broken_packages=1' in the config."));
	}

	return true;
}

void capt_show_broken(pkgDepCache & Cache)
{
	string s = _("\n"
		" Some packages have unmet dependencies. This may mean that you have \n"
		" requested an impossible situation or if you are using the unstable \n"
		" distribution that some required pkgtree.have not yet been created \n" " or been moved out of Incoming. \n\n");

	for (pkgCache::PkgIterator I = Cache.PkgBegin(); I.end() != true; I++)
	{
		if (Cache[I].InstBroken() == false)
			continue;

		// Print out each package and the failed dependencies
		s += " $7#";
		s += I.Name();
		s += "$!:";
		int Indent = strlen(I.Name()) + 3;
		bool First = true;
		if (Cache[I].InstVerIter(Cache).end() == true)
		{
			s += "\n";
			continue;
		}

		for (pkgCache::DepIterator D = Cache[I].InstVerIter(Cache).DependsList(); D.end() == false;)
		{
			// Compute a single dependency element (glob or)
			pkgCache::DepIterator Start;
			pkgCache::DepIterator End;
			D.GlobOr(Start, End);

			if (Cache.IsImportantDep(End) == false || (Cache[End] & pkgDepCache::DepGInstall) == pkgDepCache::DepGInstall)
				continue;

			if (First == false)
				for (int J = 0; J != Indent; J++)
					s += " ";
			First = false;

			s += " ";
			s += End.DepType();
			s += ": ";
			s += End.TargetPkg().Name();

			// Show a quick summary of the version requirements
			if (End.TargetVer() != 0)
			{
				s += " ($3#";
				s += End.CompType();
				s += " $2#";
				s += End.TargetVer();
				s += "$!)";
			}

			/* Show a summary of the target package if possible. In the case
			   of virtual packages we show nothing */

			pkgCache::PkgIterator Targ = End.TargetPkg();
			if (Targ->ProvidesList == 0)
			{
				s += " but ";
				pkgCache::VerIterator Ver = Cache[Targ].InstVerIter(Cache);
				if (Ver.end() == false)
				{
					s += Ver.VerStr();
					s += " $7#is installed$! ";
				}
				else
				{
					if (Cache[Targ].CandidateVerIter(Cache).end() == true)
					{
						if (Targ->ProvidesList == 0)
							s += "it is $7#not installable$! ";
						else
							s += "it is a $7#virtual package$! ";
					}
					else
						s += "it is $7#not installed$! ";
				}
			}

			s += "\n";
		}
	}

	CaptUI->ShowDialog("Unmet Dependencies", (char *) s.c_str());
}
