/* $Id: pkgchecks.cc,v 1.10 2001/06/25 08:17:15 ltd Exp $ */
#include <std.h>
#include <func/apt.h>
#include <interface/edeityui.h>

#include "config.h"

void pkgVirtualCheck(pkgCache::PkgIterator &Pkg)
{
	if ((*Cache)[Pkg].CandidateVer == 0 && Pkg->ProvidesList != 0 && Pkg.ProvidesList()->NextProvides == 0)
	{
		pkgCache::PkgIterator Tmp = Pkg.ProvidesList().OwnerPkg();
		CaptUI->ShowDialog(_("\n Missed selection"), _("Note, selecting %s instead of %s \n"), Tmp.Name(), Pkg.Name());
		Pkg = Tmp;
	}
}

bool pkgInstallCheck(pkgCache::PkgIterator & Pkg, pkgDepCache &Cache)
{
	string s;

	// Check if there is something at all to install
	pkgDepCache::StateCache & State = Cache[Pkg];

	if (State.CandidateVer == 0)
	{
		if (Pkg->ProvidesList != 0)
		{
			s = _("\n Package ") + string(Pkg.Name()) + _(" is a virtual package provided by: \n");

			pkgCache::PrvIterator I = Pkg.ProvidesList();
			for (; I.end() == false; I++)
			{
				pkgCache::PkgIterator Pkg = I.OwnerPkg();

				if (Cache[Pkg].CandidateVerIter(Cache) == I.OwnerVer())
				{
					if (Cache[Pkg].Install() == true && Cache[Pkg].NewInstall() == false)
						s += "  * " + string(Pkg.Name()) + " " + string(I.OwnerVer().VerStr()) + _(" [Installed]\n");
					else
						s += "  * " + string(Pkg.Name()) + " " + string(I.OwnerVer().VerStr()) + "\n";
				}
			}
			s += _(" You should explicitly select one to install.\n");
		}
		else
		{
			s = _("\n Package $2#") + string(Pkg.Name()) + _("$! has no available version, but exists in the database. \n"
				" This typically means that the package was mentioned in a dependency and \n"
				" never uploaded, has been obsoleted or is not available with the contents \n"
				" of sources.list");

			pkgCache::DepIterator Dep = Pkg.RevDependsList();

			string t;

			for (; Dep.end() == false; Dep++)
			{
				if (Dep->Type != pkgCache::Dep::Replaces)
					continue;
				t += " * " + string(Dep.ParentPkg().Name()) + "\n";
			}

			if (t.length() > 0)
				s += _(", however the following package(s) replace it: \n\n") + t;
			else
				s += ".\n";
		}

		CaptUI->ShowDialog((char *)(string(Pkg.Name()) + _(" has no version")).c_str(), "%s", s.c_str());

		/* _error->Error("Package %s has no installation candidate", Pkg.Name()); */

		return false;
	}

	return true;
}

bool TryInstall(pkgCache::PkgIterator Pkg, bool ReInstall = false)
{
	pkgVirtualCheck(Pkg);

	pkgDepCache & Dep = *Cache;
	pkgDepCache::StateCache & State = Dep[Pkg];

	if (pkgInstallCheck(Pkg, Dep) == false)
		return false;

	Cache->Resolver().Clear(Pkg);
	Cache->Resolver().Protect(Pkg);

	// Install it
	Dep.MarkInstall(Pkg, false);

	if (State.Install() == false)
	{
		if (ReInstall)
		{
			if (Pkg->CurrentVer == 0 || Pkg.CurrentVer().Downloadable() == false)
				CaptUI->ShowDialog("Reinstall impossible", _("\n Sorry, %s cannot be downloaded \n"), Pkg.Name());
			else
				Dep.SetReInstall(Pkg, true);
		}
		else
		{
			CaptUI->ShowDialog("Package already upgraded",
				_("\n Sorry, package is already the newest version: \n\n '%s' [= $2#%s$!] \n"), 
				Pkg.Name(), Dep[Pkg].InstVerIter(Dep).VerStr());
		}
	}

	// Install it with autoinstalling enabled.
	if (State.InstBroken() == true && _Conf.GetBool("edeity.fix_broken_packages"))
		Dep.MarkInstall(Pkg, true);

	return true;
}

bool TryRemove(pkgCache::PkgIterator Pkg, bool Purge = false)
{
	pkgVirtualCheck(Pkg);

	if (Pkg->CurrentVer == 0)
	{
		CaptUI->ShowDialog("Not installed", _("\n Package %s is not installed \n"), 
			Pkg.Name());

		return false;
	}

	Cache->Resolver().Clear(Pkg);
	Cache->Resolver().Protect(Pkg);
	Cache->Resolver().Remove(Pkg);

	(*Cache)->MarkDelete(Pkg, Purge);

	return true;
}

bool TryKeep(pkgCache::PkgIterator Pkg)
{
	pkgVirtualCheck(Pkg);
	(*Cache)->MarkKeep(Pkg);

	return true;
}

