/* $Id: cmd.cc,v 1.3 2001/06/25 08:17:15 ltd Exp $ */
#include "std.h"
#include "cmd.h"
#include <parsers/strutl.h>
#include <func/apt.h>
#include <func/keys.h>
#include <func/pkgchecks.h>
#include <list/screens.h>
#include <interface/edeityui.h>
#include <backend/acquire.h>

cmdLineParser::cmdLineParser(int argc, char **argv)
{
	_argc = argc;
	_argv = argv;
	UICreated = false;
}

int cmdLineParser::Parse()
{
	int c;
	int pkgs = 0;
	cmdEnum cmd = cmdNone;
	bool WillExit = false;

	if (_argc == 1)
		return CLP_NOEFF;

	for (c = 1; c < _argc; c++) {
		if (*_argv[c] == '-') {
			if (pkgs == 0 && (cmd == cmdInstall || cmd == cmdRemove || cmd == cmdPurge || cmd == cmdReinstall)) {
				fprintf(stderr, _("Option %s requires an argument (Opt %d).\n"), _argv[c - 1], c - 1);
				if (UICreated)
					delete CaptUI;
				return CLP_EXIT;
			}

			// These ones exit the program

			if (strcmp(_argv[c] + 1, "version") == 0 || strcmp(_argv[c] + 1, "v") == 0) {
				DisplayVersion();
				cmd = cmdHelp;
			}
			else if (strcmp(_argv[c] + 1, "help") == 0 || strcmp(_argv[c] + 1, "h") == 0) {
				DisplayHelp(_argv[0]);
				cmd = cmdHelp;
			}

			// These don't :)

			else if (strcmp(_argv[c] + 1, "install") == 0 || strcmp(_argv[c] + 1, "i") == 0) {
				cmd = cmdInstall;
				pkgs = 0;
			}
			else if (strcmp(_argv[c] + 1, "remove") == 0 || strcmp(_argv[c] + 1, "d") == 0) {
				cmd = cmdRemove;
				pkgs = 0;
			}
			else if (strcmp(_argv[c] + 1, "purge") == 0 || strcmp(_argv[c] + 1, "p") == 0) {
				cmd = cmdPurge;
				pkgs = 0;
			}
			else if (strcmp(_argv[c] + 1, "reinstall") == 0 || strcmp(_argv[c] + 1, "e") == 0) {
				cmd = cmdReinstall;
				pkgs = 0;
			}
			else if (strcmp(_argv[c] + 1, "go") == 0 || strcmp(_argv[c] + 1, "o") == 0) {
				cmd = cmdGo;
				Run();
				pkgs = 0;
				WillExit = true;
			}
			else if (strcmp(_argv[c] + 1, "update") == 0 || strcmp(_argv[c] + 1, "u") == 0) {
				cmd = cmdUpdate;
				Update();
				WillExit = true;
			}
			else if (strcmp(_argv[c] + 1, "upgrade") == 0 || strcmp(_argv[c] + 1, "g") == 0) {
				cmd = cmdUpgrade;
				Upgrade();
			}
			else if (strcmp(_argv[c] + 1, "dist-upgrade") == 0 || strcmp(_argv[c] + 1, "d") == 0) {
				cmd = cmdDistUpgrade;
				DistUpgrade();
			}
			else
				InvalidOpt(_argv[c] + 1, c);
		}
		else {
			switch (cmd) {
			case cmdNone:
			case cmdUpdate:
			case cmdGo:
			case cmdUpgrade:
			case cmdDistUpgrade:
			case cmdHelp:
				InvalidOpt(_argv[c], c);
				break;
			default:
				pkgs += Prepare(cmd, _argv[c]) ? 1 : 0;
				break;
			}
		}
	}

	switch (cmd) {
		case cmdNone:
			return CLP_NOEFF;
		case cmdDistUpgrade:
		case cmdUpgrade:
			break;
		case cmdInstall:
		case cmdReinstall:
		case cmdRemove:
		case cmdPurge:
			if (pkgs == 0) {
				if (UICreated) {
					CaptUI->ShowDialog(_("Option %s requires an argument (Opt %d).\n"), _argv[c - 1], c - 1);
					delete CaptUI;
				} else
					fprintf(stderr, _("Option %s requires an argument (Opt %d).\n"), _argv[c - 1], c - 1);
				return CLP_EXIT;
			}
			break;
		case cmdGo:
		case cmdKeep: // Doesn't actually do anything
		case cmdUpdate:
		case cmdHelp:
			return CLP_EXIT;
	}

	Run();

	return CLP_EXIT;
}

cmdLineParser::~cmdLineParser() 
{}

void cmdLineParser::InvalidOpt(char *optname, int optnum)
{
	fprintf(stderr, _("Unknown option: %s at %d.\n"), optname, optnum);
	exit(255);
}

bool cmdLineParser::Prepare(cmdEnum Command, string pkgstr)
{
	pkgCache::PkgIterator Pkg;
	const char *P;

	CreateUI();
		
	for (P = pkgstr.c_str(); *P != 0; P++)
		if (*P == '?' || *P == '*' || *P == '(' || *P == ')' || *P == '[' || *P == ']')
			goto regexsearch;

	Pkg = (*Cache)->FindPkg(pkgstr);
	
	if (Pkg.end() == false) {
		DoCommand(Command, Pkg);
		return true;
	}
	else {
		CaptUI->ShowDialog(_("Not found"), _("\n Couldn't find package $2#%s$!.\n"), pkgstr.c_str());
		return false;
	}

regexsearch: /* You know you love it */
	regex_t Pattern;
	
	if (regcomp(&Pattern, pkgstr.c_str(), REG_EXTENDED | REG_ICASE | REG_NOSUB) != 0) {
		CaptUI->ShowDialog(_("Regex compilation failed: %s.\n"), (char*)pkgstr.c_str());
		return false;
	}

	bool Hit = false;
	for (pkgCache::PkgIterator I = (*Cache)->PkgBegin(); I.end() == false; I++) {
		if (regexec(&Pattern, I.Name(), 0, 0, 0) != 0)
			continue;
		DoCommand(Command, I);
		Hit |= true;
	}
	regfree(&Pattern);

	if (Hit == false) {
		CaptUI->ShowDialog(_("Match not found"), _("\n Couldn't match regexp '$2#%s$!' in the cache \n"), pkgstr.c_str());
		return false;
	}

	return true;
}

void cmdLineParser::DoCommand(cmdEnum Command, pkgCache::PkgIterator &I)
{
	switch (Command) {
	case cmdKeep:
		TryKeep(I);
		break;
	case cmdInstall:
		TryInstall(I);
		break;
	case cmdReinstall:
		TryInstall(I, true);
		break;
	case cmdRemove:
		TryRemove(I);
		break;
	case cmdPurge:
		TryRemove(I, true);
	default: break;
	}
}

void cmdLineParser::DisplayVersion()
{
	printf(_("EDeity - Debian Acquisition Package Manager.\n"
		"Created and maintained by:\n"
		"  Patrick Cole   (ltd@debian.org)\n"
		"  Randolph Chung (tausq@debian.org)\n"
		"Compiled on %s %s.\nInternal Project Version String: %s %s\n"), __TIME__, __DATE__, PACKAGE, VERSION);
}

void cmdLineParser::DisplayHelp(char *caller)
{
#define _A_BOLD   "[1m"
#define _A_NORM   "[0m"

	printf(_("\nUsage: %s -(op 1) [package1 .. package2 .. packageN] ... -(op N) [pkg1 .. pkgN]\n\n"), caller);
	printf(_("Note: Package commands do not take effect until -go is specified.\n"
				"      However, there is an implied -go at the end of your argument sequence.\n"
				"\n"));
	printf(_(_A_BOLD "Information about this program\n" _A_NORM
				"\n"
				" -<" _A_BOLD "h" _A_NORM ">elp                    This helpful little screen\n"
				" -<" _A_BOLD "v" _A_NORM ">ersion                 Current version of the program\n"
				"\n"
				_A_BOLD "Modify the state of a given set of packages\n" _A_NORM
				"\n"
				" -<" _A_BOLD "d" _A_NORM ">elete <pkg1..N>        Remove package(s)\n"
				" -<" _A_BOLD "p" _A_NORM ">urge <pkg1..N>         Remove package(s) and config files\n"
				" -r<" _A_BOLD "e" _A_NORM ">install <pkg1..N>     Reinstall package(s)\n"
				"\n"
				_A_BOLD "Acquire items from your defined archive sites and/or modify state\n" _A_NORM
				"\n"
				" -<" _A_BOLD "i" _A_NORM ">nstall <pkg1..N>       Install package(s)\n"
				" -<" _A_BOLD "s" _A_NORM ">ource <pkg1..N>        Acquire source tarball for package(s)\n"
				" -<" _A_BOLD "u" _A_NORM ">pdate-lists            Acquire new package listings if available\n"
				"\n"
				_A_BOLD "Auto dependency resolving upgrade features\n" _A_NORM
				"\n"
				" -up<" _A_BOLD "g" _A_NORM ">rade-only            Perform upgrade of installed packages\n"
				" -<" _A_BOLD "d" _A_NORM ">ist-upgrade            Perform intelligent upgrade of packages\n"
				"                            (may result in the removal of packages)\n"
				_A_BOLD "Complete said operations\n" _A_NORM
				"\n"
				" -g<" _A_BOLD "o" _A_NORM ">                      Complete specified commands\n\n"));
}

void cmdLineParser::CreateUI()
{
	if (UICreated)
		return;
	CaptUI = ::CreateUI(_argc, _argv);
	Cache = new edeitypkgcache;
	UICreated = true;
}

void cmdLineParser::Upgrade()
{}

void cmdLineParser::DistUpgrade()
{}

void cmdLineParser::Update()
{
	int fd;
	CreateUI();
	pkgAcquireBackend *Fetcher = new pkgAcquireBackend(CaptUI);
	fd = Fetcher->AcquireUpdate();
	CaptUI->WaitForFetch(Fetcher, fd);
	CaptUI->Passthrough(fd);
	delete Fetcher;
}

void cmdLineParser::Run()
{
	int fd;
	CreateUI();
	pkgAcquireBackend *Fetcher = new pkgAcquireBackend(CaptUI);
	fd = Fetcher->AcquireRun();
	CaptUI->WaitForFetch(Fetcher, fd);
	CaptUI->Passthrough(fd);
	delete Fetcher;
}
