/* $Id: main.cc,v 1.19 2001/06/25 08:17:15 ltd Exp $ */
#include <std.h>
//old #include <strings.h>
#include <cmd.h>
//old #include <stdio.h>

#include <func/apt.h>
#include <func/config.h>
#include <func/keys.h>
#include <func/mouse.h>
#include <func/dependency.h>
#include <func/statecache.h>
#include <list/screens.h>
#include <interface/edeityui.h>
#include <backend/acquire.h>

//old #include <getopt.h>

#ifndef DEFAULTUI
#define DEFAULTUI "gtk"
#endif
#ifndef UIPREFIX
#define UIPREFIX "edeity-"
#endif

EDeityUI *CaptUI = NULL;

static const char *progname;


void sighandler(int sig);
void edeity_clean_exit(void);

/*
 * Wrapper function to start up a ui based on various settings
 */
 
EDeityUI *CreateUI(int &argc, char **&argv)
{

	EDeityUI *ui = NULL;
	const char *name = NULL;

	name = getenv("CAPTUI");

	if (name == NULL)
	{
		name = strrchr(progname, '/');
		if (name == 0)
			name = progname;
		else
			name++;

		if (strncmp(name, UIPREFIX, sizeof UIPREFIX - 1) == 0)
			name += (sizeof UIPREFIX - 1);
		else
			name = DEFAULTUI;
	}
	
	
	string s;

	s = MODULE_PATH;
	if (s[s.length()-1] != '/')
		s += '/';
	s += name;

	INFO(INFO_DEBUG, _("Selected interface: %s\n"), s.c_str());

	ui = EDeityUI::CreateInterface(s.c_str(), argc, argv);

	if (ui == NULL)
	{
		s = MODULE_PATH;
		if (s[s.length()-1] != '/')
			s += '/';
		s += DEFAULTUI;
		ui = EDeityUI::CreateInterface(s.c_str(), argc, argv);
	}

	return ui;
}



int main (int argc, char **argv)
{
	// Set up the signal handlers
	signal(SIGSEGV, sighandler);
	signal(SIGINT, sighandler);
	signal(SIGKILL, sighandler);
	signal(SIGTERM, sighandler);

	progname = argv[0];

#ifdef ENABLE_NLS
    setlocale(LC_ALL, "");
    textdomain(PACKAGE);

	char *postfix = "/share/locale";
	char *dirname = (char *) malloc (strlen (PREFIX) + strlen (postfix) + 1);
	sprintf (dirname, "%s%s", PREFIX, postfix);
	
    bindtextdomain(PACKAGE, dirname);
	
	free (dirname);
#endif

	// Initialize the apt config structures
	if (!pkgInitConfig(*_config) || !pkgInitSystem(*_config, _system))
	{
		_error->DumpErrors();
		return 100;
	}
    
	// Initialize command line arguments
	cmdLineParser clp(argc, argv);
	if (clp.Parse() == CLP_EXIT)
		return 0;

	// Parse the configuration file
	if (_Conf.Init() == false)
		_Conf.Write();

	// Initialize the user interface

	if ((CaptUI = CreateUI(argc, argv)) == NULL)
	{
		fprintf(stderr, _("\nCould not load any UI modules. Try setting your dynamic loading path.\n\nFor example:  LD_LIBRARY_PATH=/usr/lib/edeity/modules capt\nor load capt using the edeity-<ui> symlinks.\n\n"));
		exit(-4);
	}

	// Initialise the APT component
	Cache = new edeitypkgcache;
	check_dependencies(false);

	// Check that there are no apt-generated errors at this point
	while (_error->PendingError() == true)
	{
		string s;
		_error->PopMessage(s);
		CaptUI->ShowDialog(_("Startup failure"), (char *) s.c_str());
	}

	// Add the first screen
	CaptUI->scrlst.add();
	CURRENTSCR->BoundsReset();

	// Turn on description window
	if (_Conf.GetBool("edeity.description.show") == true)
		CURRENTSCR->DescWindowShow();

	int UIret = CaptUI->Run();

	edeity_clean_exit();

	return UIret;

 return 0;
}



void sighandler(int sig)
{
	edeity_clean_exit();
	fprintf(stderr, _("Caught Signal %d\n"), sig);
	if (sig == 11)
		fprintf(stderr, _("Report this bug to szilard@uhulinux.hu\n"));
	exit(sig);
}


void edeity_clean_exit(void)
{
	if (Cache)
		delete Cache;

	if (CaptUI)
		delete CaptUI;

	_Conf.Write();
}
