/* $Id: acquire-ui.cc,v 1.10 2001/05/23 22:41:12 ltd Exp $ */
#include <std.h>
#include <func/apt.h>
#include <func/config.h>
#include <func/dependency.h>
#include <func/messages.h>
#include <interface/edeityui.h>
#include <list/screens.h>
#include <list/filters.h>
#include <apt-pkg/acquire-item.h>
#include <apt-pkg/acquire-worker.h>
#include <apt-pkg/strutl.h>
#include "colors.h"
#include "cursesui.h"
#include "wrappers.h"
#include "captris.h"
#include "acquire-ui.h"

CursesUI::AcquireUI::AcquireUI(CursesUI & _ui) : ui(_ui)
{
	windowsActive = false;
	wasAborted = false;
	Game = NULL;
	extra = NULL;
}

CursesUI::AcquireUI::~AcquireUI()
{
	if (windowsActive == true)
		DestroyWindows();

	if (Game != NULL)
		delete Game;
}

void CursesUI::AcquireUI::DispatchKey(int key)
{
	switch (key)
	{
		case 'r':
			ViewRemove();
			break;
		case 'i':
			ViewInstall();
			break;
		case 'q':
			// Do something
			break;
		case 't':
			if (extra == NULL)
				StartCaptris();
			else
				StopCaptris();
		default:
			if (extra != NULL)
				Game->handlekey(key);
			break;
	}
}

void CursesUI::AcquireUI::Refresh()
{
}

void CursesUI::AcquireUI::Startup(void)
{
	curs_set(0);
	clear();
	refresh();

	CalibrateWindows();

	output   = new UniqueWindow(output_len_y,      output_len_x,    output_pos_y,    output_pos_x);
	output->scrollable(true);
	output->ChangeTitle(_("fetcher backlog"));
	output->Erase();
	output->Refresh();

	itemized = new UniqueWindow(itemized_len_y,    itemized_len_x,  itemized_pos_y,  itemized_pos_x);
	itemized->scrollable(false);
	itemized->ChangeTitle(_("file progress"));
	itemized->Erase();
	itemized->Refresh();

	total    = new UniqueWindow(total_len_y,       total_len_x,     total_pos_y,     total_pos_x);
	total->scrollable(false);
	total->ChangeTitle(_("batch progress"));
	total->Erase();
	total->Refresh();

	if (_Conf.GetBool("edeity.acquire.captris") == true)
		StartCaptris();

	windowsActive = true;

	PrintOutputF(_("$3#To toggle playing the captris game, press the 't' key.$! \n"));
}

void CursesUI::AcquireUI::CalibrateWindows()
{
	extra_len_y = 16;
	extra_len_x = 37;
	extra_pos_y = LINES - extra_len_y;
	extra_pos_x = 0;

	output_len_y = LINES - extra_len_y - 1;
	output_len_x = COLS;
	output_pos_y = 0;
	output_pos_x = 0;

	if (_Conf.GetBool("edeity.acquire.captris") == true)
	{
		total_len_y = 5;
		total_len_x = COLS - extra_len_x - 2;
		total_pos_y = LINES - total_len_y;
		total_pos_x = extra_len_x + 2;
	
		itemized_len_y = LINES - output_len_y - total_len_y - 2;
		itemized_len_x = COLS - extra_len_x - 2;
		itemized_pos_y = extra_pos_y;
		itemized_pos_x = extra_len_x + 2;
	}
	else
	{
		total_len_y = 5;
		total_len_x = COLS;
		total_pos_y = LINES - total_len_y;
		total_pos_x = 0;

		itemized_len_y = LINES - output_len_y - total_len_y - 2;
		itemized_len_x = COLS;
		itemized_pos_y = extra_pos_y;
		itemized_pos_x = 0;
	}

}

void CursesUI::AcquireUI::StartCaptris()
{
	if (windowsActive == true)
	{
		_Conf.Set("edeity.acquire.captris", _TRUE);
		
		CalibrateWindows();

		delete itemized;
		itemized = new UniqueWindow(itemized_len_y,    itemized_len_x,  itemized_pos_y,  itemized_pos_x);
		itemized->scrollable(false);
		itemized->ChangeTitle(_("file progress"));
		itemized->Erase();
		itemized->Refresh();

		delete total;
		total = new UniqueWindow(total_len_y,       total_len_x,     total_pos_y,     total_pos_x);
		total->scrollable(false);
		total->ChangeTitle(_("batch progress"));
		total->Erase();
	}

	extra = new UniqueWindow(extra_len_y,       extra_len_x,     extra_pos_y,     extra_pos_x);

	extra->scrollable(false);
	extra->ChangeTitle(_("entertainment"));
	extra->Erase();
	keypad(extra->body(), true);

	if (Game == NULL)
	{
		Game = new Captris(this);
		Game->initiate();
	}

	clear();
	refresh();
	output->Redraw();
	extra->Redraw();
	total->Redraw();
	itemized->Redraw();
}

void CursesUI::AcquireUI::StopCaptris()
{
	_Conf.Set("edeity.acquire.captris", _FALSE);

	CalibrateWindows();

	delete itemized;
	itemized = new UniqueWindow(itemized_len_y,    itemized_len_x,  itemized_pos_y,  itemized_pos_x);
	itemized->scrollable(false);
	itemized->ChangeTitle(_("file progress"));
	itemized->Erase();

	delete total;
	total = new UniqueWindow(total_len_y,       total_len_x,     total_pos_y,     total_pos_x);
	total->scrollable(false);
	total->ChangeTitle(_("batch progress"));
	total->Erase();

	delete extra;
	extra = NULL;

	clear();
	refresh();
	output->Redraw();
	total->Redraw();
	itemized->Redraw();
}

void CursesUI::AcquireUI::ViewInstall()
{
	int pkgcount = 0, headcount = 0, prevheadcount = 0;
	string tmp = "  ";

	for (pkgCache::PkgIterator I = (*Cache)->PkgBegin(); I.end() == false; I++)
	{
		if ((*Cache)[I].Install() == true)
		{   
			if ((tmp.length() + strlen(I.Name()) - headcount) >= unsigned(COLS-16))
			{ 
				tmp += " \n  ";
				prevheadcount ++;
				headcount = prevheadcount;
			}
			tmp += I.Name();
			tmp += " ";

			prevheadcount += strlen(I.Name()) + 1;
		 	pkgcount ++;
		}
	}

	if (pkgcount > 0)
		CaptUI->ShowDialog(_("Install"), (char *)tmp.c_str());

}

void CursesUI::AcquireUI::ViewRemove()
{
	int pkgcount = 0, headcount = 0, prevheadcount = 0;
	string tmp = "  ";

	for (pkgCache::PkgIterator I = (*Cache)->PkgBegin(); I.end() == false; I++)
	{
		if ((*Cache)[I].Delete() == true)
		{   
			if ((tmp.length() + strlen(I.Name()) - headcount) >= unsigned(COLS-16))
			{   
				tmp += " \n  ";
				prevheadcount ++;
				headcount = prevheadcount;
			}
			tmp += I.Name();
			tmp += " ";

			prevheadcount += strlen(I.Name()) + 1;
			pkgcount ++;
		}
	}

	if (pkgcount > 0)
		CaptUI->ShowDialog(_("Delete"), (char *)tmp.c_str());
	
}

void CursesUI::AcquireUI::DestroyWindows(void)
{
	delete output;
	delete extra;
	delete itemized;
	delete total;

	clear();
	refresh();

	windowsActive = false;
}

/*void CursesUI::AcquireUI::Error(char *fmt,...)
{
	va_list va;
	char buf[512];

	va_start(va, fmt);
	vsnprintf(buf, 511, fmt, va);
	va_end(va);

	clear();
	refresh();
	CaptUI->ShowDialog("fatal error", "\n %s \n", buf);

	NoCacheInit = true;
}

void CursesUI::AcquireUI::Info(char *fmt,...)
{
	va_list va;
	char buf[512];

	va_start(va, fmt);
	vsnprintf(buf, 511, fmt, va);
	va_end(va);

	clear();
	refresh();
	CaptUI->ShowDialog("Info", "\n %s \n", buf);

	NoCacheInit = true;
}
*/

void CursesUI::AcquireUI::PrintOutputF(char *fmt,...)
{
	va_list va;
	char buf[200];

	va_start(va, fmt);

	vsnprintf(buf, 200, fmt, va);

	CursesUI::mv_printw_color(output->body(), output->body()->_cury, output->body()->_curx, "%s", buf);

	va_end(va);
}
/*
void CursesUI::AcquireUI::WaitContinue(void)
{
	printf("\nPress enter to continue ...");
	fflush(stdout);

	char C;

	while (1)
	{
		read(STDIN_FILENO, &C, 1);
		if (C == '\n')
			break;
	}

}
*/

void CursesUI::AcquireUI::Shutdown(void)
{
	DestroyWindows();
}

/*
void CursesUI::AcquireUI::SetFds(int &Fd,fd_set *RSet,fd_set *WSet)
{
	pkgAcquire::SetFds(Fd,RSet,WSet);

	FD_SET(STDIN_FILENO, RSet);

}

void CursesUI::AcquireUI::RunFds(fd_set *RSet,fd_set *WSet)
{
	pkgAcquire::RunFds(RSet,WSet);

	if (FD_ISSET(STDIN_FILENO, RSet))
	{
		int ch = wgetch(output->body());

		if (ch == 'r')
			ViewRemove();
		else if (ch == 'i')
			ViewInstall();
		else if (ch == 'q')
			wasAborted = true;
		else if (ch == 't')
		{
			if (extra == NULL)
				StartCaptris();
			else
				StopCaptris();
		}
		else
			if (extra != NULL)
				Game->handlekey(ch);
	}
}
*/

void CursesUI::AcquireUI::Start()
{
};

void CursesUI::AcquireUI::Stop()
{
/*	if (FetchedBytes != 0)
	{
	 	printf("\nFetched %sb in %s (%sb/s)\n\n", SizeToStr(FetchedBytes).c_str(), TimeToStr(ElapsedTime).c_str(), SizeToStr(CurrentCPS).c_str());
		fflush(stdout);
		sleep(2);
	}*/
}

void CursesUI::AcquireUI::IMSHit(const MsgItemDesc *M)
{
	PrintOutputF(_("Hit %s"), M->Description);

	if (M->FileSize != 0)
		PrintOutputF(" [%sb]", SizeToStr(M->FileSize).c_str());

	PrintOutputF("\n");

	output->Refresh();
};

void CursesUI::AcquireUI::Fetch(const MsgItemDesc *M)
{
	PrintOutputF(_("Get:%d %s"), M->ID, M->Description);

	if (M->FileSize != 0)
		PrintOutputF(" [%sb]", SizeToStr(M->FileSize).c_str());

	PrintOutputF("\n");

	output->Refresh();
};

void CursesUI::AcquireUI::Done(const MsgItemDesc *M)
{
};

void CursesUI::AcquireUI::Fail(const MsgItemFailed *M)
{
	if (M->Status == pkgAcquire::Item::StatDone)
		PrintOutputF(_("Ign %s\n"), M->Itm.Description);
	else
	{
		PrintOutputF(_("Err %s\n"), M->Itm.Description);
		PrintOutputF(_("    %s\n"), M->ErrorText);
	}

	output->Refresh();
};

void CursesUI::AcquireUI::Update(const vector<MsgDownloadItem> *ItemList, const MsgDownloadTotal *Total)
{
	if (extra != NULL)
		Game->pulse();

	itemized->Erase();
	total->Erase();

	bool active = false;
	int N = ItemList->size();

	for (int I = 0; I < N && I < 4; I++)
	{
		const MsgDownloadItem & Item = (*ItemList)[I];

		wattrset(itemized->body(), CursesUI::_clrsys->regpair(COLOR_BLACK, COLOR_WHITE));
		for (int H = 2; H < itemized->cols() - 2; H++)
			mvwaddch(itemized->body(), I+1, H, ' ');

		if (*Item.Status)
		{
			mvwprintw(itemized->body(), I+1, 2, " %s", Item.Status);
			itemized->Refresh();
		}
		else
		{
			if (Item.FileSize != 0)
			{
				double barlen = Item.Percent * double(itemized->cols()-4);
				char buf[COLS];
				sprintf(buf, " %s %s/%s %02d%%%% ",
						Item.ShortDesc,
						SizeToStr(Item.PartialSize).c_str(),
						SizeToStr(Item.FileSize).c_str(),
						int(Item.Percent * 100.0));
				wmove(itemized->body(), I+1, 2);
				if (barlen < strlen(buf))
				{
					char redbuf[COLS];
					int H;
					for (H = 0; H < barlen; H++)
						redbuf[H] = buf[H];
					redbuf[H] = 0;
					wattrset(itemized->body(), CursesUI::_clrsys->regpair(COLOR_WHITE, COLOR_RED));
					wprintw(itemized->body(), redbuf);
					wattrset(itemized->body(), CursesUI::_clrsys->regpair(COLOR_BLACK, COLOR_WHITE));
					wprintw(itemized->body(), buf+H);
				}
				else
				{
					wattrset(itemized->body(), CursesUI::_clrsys->regpair(COLOR_WHITE, COLOR_RED));
					wprintw(itemized->body(), buf);
					for (int H = 0; H < (barlen - strlen(buf)); H++)
						waddch(itemized->body(), ' ');
				}
				wattrset(itemized->body(), CursesUI::_clrsys->regpair(COLOR_BLACK, COLOR_WHITE));
				active = true;
			}
		}
	}

	if (active == false)
	{
		wattrset(total->body(), CursesUI::_clrsys->regpair(COLOR_WHITE, COLOR_BLUE));
		mvwprintw(total->body(), 1, 3, _("Working ..."));
		total->Refresh();
		return;
	}

	double barlen = Total->Percent * double(itemized->cols() - 4);
	char buf[BUFSIZ];

	if (Total->CurrentCPS != 0)
		sprintf(buf, _(" Total %sB/s %02d%%%% (%s remains)"), 
			SizeToStr(Total->CurrentCPS).c_str(), int(Total->Percent * 100.0), TimeToStr(Total->ETA).c_str());
	else
		sprintf(buf, " %02d%%%% ", int(Total->Percent * 100.0));

	wattrset(total->body(), CursesUI::_clrsys->regpair(COLOR_BLACK, COLOR_WHITE));

	for (int x=2; x < total->cols()-2; x++)
		mvwaddch(total->body(), 1, x, ' ');

	wmove(total->body(), 1, 2);

	if (barlen < strlen(buf))
	{
		char redbuf[COLS];
		int H;
			
		for (H = 0; H < barlen; H++)
			redbuf[H] = buf[H];

		redbuf[H] = 0;

		wattrset(total->body(), CursesUI::_clrsys->regpair(COLOR_WHITE, COLOR_RED));
		wprintw(total->body(), redbuf);
		wattrset(total->body(), CursesUI::_clrsys->regpair(COLOR_BLACK, COLOR_WHITE));
		wprintw(total->body(), buf+H);
	}
	else
	{
		wattrset(total->body(), CursesUI::_clrsys->regpair(COLOR_WHITE, COLOR_RED));
		wprintw(total->body(), buf);

		for (int H = 0; H < (barlen - strlen(buf)); H++)
			waddch(total->body(), ' ');
	}

	itemized->Refresh();
	total->Refresh();
}

void CursesUI::AcquireUI::MediaChange(const MsgMediaChange *M)
{
	PrintOutputF(_("Media Change: Please insert the disc labeled '%s' in the drive '%s' and press enter"),
		M->Title, M->Drive);
	
	output->Refresh();

	char C = 0;
	while (C != '\n' && C != '\r')
		read(STDIN_FILENO, &C, 1);
}

void CursesUI::AcquireUI::Message(const char *text)
{
	if (windowsActive)
		PrintOutputF("%s\n", text);
	else
	{
		addstr(text);
		addch('\n');
		refresh();
	}
}
