/* $Id: colors.cc,v 1.13 2001/05/15 10:11:07 ltd Exp $ */

//old #include <stdlib.h>

#include <init/std.h>
#include <func/config.h>

#include "colors.h"

struct colour_system::attribute attrs[] = {
	{ "bold",     A_BOLD },
	{ "reverse",  A_REVERSE },
	{ "blink",    A_BLINK },
	{ "invisible",A_INVIS },
	{ "dim",      A_DIM },
	{ "underline",A_UNDERLINE },
	{ "normal",   A_NORMAL },
	{ NULL, 0 }
};

struct colour_system::attribute colors[] = {
	{ "black",    COLOR_BLACK },
	{ "red",      COLOR_RED },
	{ "green",    COLOR_GREEN },
	{ "yellow",   COLOR_YELLOW },
	{ "blue",     COLOR_BLUE },
	{ "magenta",  COLOR_MAGENTA },
	{ "cyan",     COLOR_CYAN },
	{ "white",    COLOR_WHITE },
	{ NULL, 0 }
};

colour_system::colour_system()
{
	hash_size = ATTR_HASH_SIZE;
	AttrMap = (attr_map_node **) malloc(hash_size * sizeof(char **));
	PairCount = 0;

	memset(AttrMap, 0, hash_size * sizeof(char **));

	attr_map_node *attrp;
	int i=0;

	for (Config::Iterator C(_Conf); C.valid(); C++)
	{
		if (strncmp(C->param_name, "edeity.curses.color", 18) == 0)
		{
			INFO(INFO_VERBOSE, _("parsing: %s\n"), C->param_name);
			if ((attrp = ParseConfigAttrib(C->param_name)) != NULL)
				attr_map_add(C->param_name, attrp);		
			i++;
		}
	}
}

colour_system::~colour_system()
{
	for (register int I = 0; I < PairCount; I++)
		delete Pairs[I];
	PairCount = 0;
}

void colour_system::attr_map_add(char *name, attr_map_node *New)
{
	INFO(INFO_VERBOSE, _("adding %s = 0x%X\n"), name, (New->curs_pair ? COLOR_PAIR(New->curs_pair->pair) : 0) | New->curs_attr);
	attr_map_bucket_add(AttrMap + attr_map_hasher(name), New);
}

void colour_system::attr_map_bucket_add(attr_map_node **pos, attr_map_node *New)
{
	attr_map_node *x;

	if (*pos == NULL) {
		*pos = New;
		(*pos)->next = NULL;
		INFO(INFO_VERBOSE, _("adding first node : %s\n"), (*pos)->name);
		return;
	}

	for (x = *pos; x->next != NULL; x = x->next);

	x->next = New;
	x->next->next = NULL;

	INFO(INFO_VERBOSE, _("adding bucket node : %s\n"), x->next->name);
}

int colour_system::attr_map_hasher(char *name)
{
	int hash = 0;
	for (char *p = name; *p != 0; p++)
		hash += *p;
	return hash % hash_size;
}

colour_system::attr_map_node *colour_system::ParseConfigAttrib(const char *confitem)
{
	char *base = _Conf.Get((char *)confitem);
	char *p, *q;
	int _attr = 0;
	int _fg = 0, _bg = 0;
	bool got_attr = FALSE;
	bool got_color = FALSE;
	struct attribute *ap;
	bool willcontinue;

	if (base == NULL)
		return NULL;

	base = strdup(base);

	for (p = base; *p != 0; p++) 
	{
		if (*p == '%') 
		{
			p++;
			if (*p == 0)
				break;
			for (q = p; *q != ',' && *q != 0; q++);
			if (*q != 0)
				*q++ = 0;
			for (ap = attrs; ap->name != NULL; ap++)
			{
				if (strcmp(ap->name, p) == 0)
				{
					_attr |= ap->attr;
					got_attr |= TRUE;
					break;
				}
			}
			p = q - 1;
		} 
		else 
		{
			for (q = p; *q != '/' && *q != 0; q++);
			if (*q != 0)
				*q++ = 0;
			for (ap = colors; ap->name != NULL; ap++)
			{
				if (strcmp(ap->name, p) == 0)
				{
					_fg = ap->attr;
					p = q;
					break;
				}
			}
			willcontinue = false;
			for (q = p; *q != ',' && *q != 0; q++);
			if (*q != 0)
			{
				willcontinue = true;
				*q++ = 0;
			}
			for (ap = colors; ap->name != NULL; ap++)
			{
				if (strcmp(ap->name, p) == 0)
				{
					_bg = ap->attr;
					got_color |= TRUE;
					p = q;
					break;
				}
			}
			if (!willcontinue)
				break;
		}
	}

	if (!got_color && !got_attr)
		return NULL;

	free(base);

	//INFO(INFO_DEBUG, "parse finished..\n");

	attr_map_node *np = new attr_map_node;

	np->name = strdup(confitem);

	if (got_color)
	{
		INFO(INFO_DEBUG, _(" fg=%d bg=%d "), _fg, _bg);
		np->curs_pair = __regpair(_fg, _bg);
	}
	else
		np->curs_pair = NULL;

	if (got_attr)
	{
		INFO(INFO_DEBUG, _(" attr=0x%X"), _attr);
		np->curs_attr = _attr;
	}
	else
		np->curs_attr = 0;

	INFO(INFO_DEBUG, "\n");

	return np;
}

int colour_system::regpair(int fg, int bg)
{
	for (register int I = 0; I < PairCount; I++)
		if (Pairs[I]->fg == fg && Pairs[I]->bg == bg)
			return COLOR_PAIR(Pairs[I]->pair);

	PairCount += 1;
	int r = init_pair(PairCount, fg, bg);
	ASSERT(r != ERR);
	Pairs[PairCount - 1] = new c_pair(fg, bg, PairCount);

	return COLOR_PAIR(PairCount);
}

colour_system::c_pair *colour_system::__regpair(int fg, int bg)
{
	for (register int I = 0; I < PairCount; I++)
		if (Pairs[I]->fg == fg && Pairs[I]->bg == bg)
			return Pairs[I];

	PairCount += 1;
	int r = init_pair(PairCount, fg, bg);
	ASSERT(r != ERR);
	INFO(INFO_DEBUG, _("New color pair %d: (fg,bg) = (%s,%s)\n"),
		PairCount, colors[fg].name, colors[bg].name);
	Pairs[PairCount - 1] = new c_pair(fg, bg, PairCount);

	return Pairs[PairCount - 1];
}

int colour_system::operator [](char *attr_name)
{
	for (attr_map_node *p = *(AttrMap + attr_map_hasher(attr_name)); p != NULL; p = p->next)
	{
		if (strcmp(p->name, attr_name) == 0)
		{
			INFO(INFO_VERBOSE, _("attr for %s = 0x%X\n"), attr_name, (p->curs_pair ? COLOR_PAIR(p->curs_pair->pair) : 0) | p->curs_attr);
			if (p->curs_pair == NULL)
				return p->curs_attr;
			else
				return COLOR_PAIR(p->curs_pair->pair) | p->curs_attr;
		}
	}

	NOTREACHED(_("%s not found!"), attr_name);
}

/*
colour_system::c_pair colour_system::find_pair(short pairno)
{
	short f, b;
	if (pair_content(pairno, &f, &b) == ERR)
		return -1;
	else
		return regpair(f, b);
}
*/
