/* $Id: cursesdlg.cc,v 1.8 2001/03/24 08:08:32 ltd Exp $ */
//old #include <stdio.h>
//old #include <std.h>
#include "colors.h"
#include <parsers/strutl.h>
#include "cursesui.h"
#include "cursesdlg.h"
#include <vector>

CursesUI::Dialog::Entry::~Entry()
{
	delete handler;
}

CursesUI::Dialog::EntryHandler::EntryHandler(void (*f)(::Screen *, int), ::Screen *_scr) 
	: screen(_scr), type(KeyHandler) { fn.keyf = f; }
	
CursesUI::Dialog::EntryHandler::EntryHandler(void (*f)(::Screen *), ::Screen *_scr) 
	: screen(_scr), type(NullKeyHandler) { fn.nullkeyf = f; }
	
CursesUI::Dialog::EntryHandler::EntryHandler(void (::Screen::*f)(int), ::Screen *_scr) 
	: screen(_scr), type(ScrHandler) { fn.scrf = f; }

CursesUI::Dialog::EntryHandler::EntryHandler(void (::Screen::*f)(void), ::Screen *_scr) 
	: screen(_scr), type(NullScrHandler) { fn.nullscrf = f; }

CursesUI::Dialog::Dialog(Screen &_screen)
	: screen(_screen)
{
	ly=-1; lx=-1; 
	sy=-1; sx=-1;
	linecount=0; width=0;
	Initialized=false;
//	activeDialog = this;
}

CursesUI::Dialog::~Dialog()
{
	free_memory();
	deinitialize();
//	activeDialog = NULL;
}

void CursesUI::Dialog::geometry(int _ly, int _lx, int _sy, int _sx)
{
	ly=_ly; lx=_lx;
	sy=_sy; sx=_sx;
}

void CursesUI::Dialog::addline(string text, void (*keyhandler)(Screen *, int))
{
	Lines.resize(linecount + 1);

	Lines[linecount] = new Entry;
	Lines[linecount]->text = strdup(text.c_str());
	Lines[linecount]->handler = new EntryHandler(keyhandler, &screen);

	linecount ++;

	if ((unsigned)strlen_color((char*)text.c_str()) > width)
		width = strlen_color((char*)text.c_str());
}

void CursesUI::Dialog::addline(string text, void (::Screen::*keyhandler)(int))
{
	Lines.resize(linecount + 1);

	Lines[linecount] = new Entry;
	Lines[linecount]->text = strdup(text.c_str());
	Lines[linecount]->handler = new EntryHandler(keyhandler, &screen);

	linecount ++;

	if ((unsigned)CursesUI::strlen_color((char*)text.c_str()) > width)
		width = CursesUI::strlen_color((char*)text.c_str());
}

void CursesUI::Dialog::addline(string text, void (*keyhandler)(Screen *))
{
	Lines.resize(linecount + 1);

	Lines[linecount] = new Entry;
	Lines[linecount]->text = strdup(text.c_str());
	Lines[linecount]->handler = new EntryHandler(keyhandler, &screen);

	linecount ++;

	if ((unsigned)CursesUI::strlen_color((char*)text.c_str()) > width)
		width = CursesUI::strlen_color((char*)text.c_str());
}

void CursesUI::Dialog::addline(string text, void (::Screen::*keyhandler)(void))
{
	Lines.resize(linecount + 1);

	Lines[linecount] = new Entry;
	Lines[linecount]->text = strdup(text.c_str());
	Lines[linecount]->handler = new EntryHandler(keyhandler, &screen);

	linecount ++;

	if ((unsigned)CursesUI::strlen_color((char*)text.c_str()) > width)
		width = CursesUI::strlen_color((char*)text.c_str());
}

void CursesUI::Dialog::addline(string text)
{
	Lines.resize(linecount + 1);

	Lines[linecount] = new Entry;
	Lines[linecount]->text = strdup(text.c_str());
	Lines[linecount]->handler = NULL;

	linecount ++;

	if ((unsigned)CursesUI::strlen_color((char*)text.c_str()) > width)
		width = CursesUI::strlen_color((char*)text.c_str());
}

void CursesUI::Dialog::free_memory()
{
	for (unsigned I=0; I < linecount; I++)
	{
		free(Lines[I]->text);
		free(Lines[I]);
	}

	Lines.resize(0);

	linecount=0; width=0;
}

void CursesUI::Dialog::deinitialize()
{
	if (Initialized)
	{
		ly=-1; lx=-1; 
		sy=-1; sx=-1;
		delete Window;
		Initialized=false;
	}
}


void CursesUI::Dialog::initialize()
{

	if (lx == -1)
		lx = width;
	else
		if (lx > COLS)
			lx = COLS;

	if (ly == -1)
		if (linecount > (unsigned)LINES)
			ly = LINES;
		else
			ly = linecount + 2;
	else
		if (ly > LINES)
			ly = LINES;

	if (lx <= (COLS-2))
		lx += 2;

	if (sy == -1)
		sy = (LINES-ly)/2;

	if (sx == -1)
		sx = (COLS-lx)/2;

	Window = new UniqueWindow(ly, lx, sy, sx);
	
	curs_set(0);
	noecho();
	cbreak();

	Window->ChangeTitle(TitleText);
	Window->Refresh();
	Window->scrollable(false);

	// calibrate the cursor to the first line that has a handler function

	cursor = 0;

	for (unsigned I = 0; I < linecount; I++)
		if (Lines[I]->handler != NULL)
		{
			cursor = I;
			break;
		}

	bound_start     = 0;
	bound_end       = Window->rows();

	while (bound_end < (cursor + 1))
	{
		bound_start += 1;
		bound_end += 1;
	}
	Initialized=true;
}

void CursesUI::Dialog::drawall()
{
	Window->Erase();

	unsigned int I, K;

	for (I = bound_start, K = 0; I < bound_end; I++, K++)
		drawline(K, I);
	
	Window->Refresh();
}

void CursesUI::Dialog::drawline(int row, unsigned lkey)
{
	if (lkey == cursor)
		wattron(Window->body(), A_REVERSE);
		
	mv_printw_color(Window->body(), row, 0, Lines[lkey]->text);
	for (unsigned int J = strlen_color(Lines[lkey]->text); J < unsigned(Window->cols()); J++)
		mvwaddch(Window->body(), row, J, ' ');

	if (lkey == cursor)
		wattroff(Window->body(), A_REVERSE);
}

bool CursesUI::Dialog::external_key_handler(int key)
{
	return false;
}

void CursesUI::Dialog::complete_redraw()
{
	drawall();
	Window->Redraw();
}

void CursesUI::Dialog::select()
{
	fd_set rset;
	int highfd;
	int Z;

	while (1)
	{
		FD_ZERO(&rset);
		FD_SET(STDIN_FILENO, &rset);

		if (CaptUI->signalfd != -1)
		{
			FD_SET(CaptUI->signalfd, &rset);
			highfd = CaptUI->signalfd;
		}
		else
			highfd = STDIN_FILENO;

		if (::select(highfd + 1, &rset, 0, 0, 0) <= 0)
			continue;

		if (CaptUI->signalfd != -1 && FD_ISSET(CaptUI->signalfd, &rset))
		{
			CaptUI->ProcessMsg(CaptUI->signalfd);
			complete_redraw();
		}
		if (FD_ISSET(STDIN_FILENO, &rset))
		{
#ifdef HAVE_LIBGPM
//old		Z = Gpm_Getch();
//?			Z = Gpm_Wgetch();
#else
			Z = getch();
#endif
			switch (Z)
			{
				case '\n':
				case -1:
					if (Lines[cursor]->handler == NULL)
						break;
					else
						(*Lines[cursor]->handler)(Z);
	
				case -2:
					return;
				
				case 'q':
				case 'Q':
				case 'x':
				case 'X':
				case 27:
					return;
	
				// page up
				case KEY_PPAGE:
					if (signed(cursor - 5) < 0)
					{
						bound_start = 0;
						bound_end = Window->rows();
						cursor = 0;
					}
					else
					{
						cursor -= 5;
	
						if (cursor < bound_start)
						{
							bound_start = cursor;
							bound_end = bound_start + Window->rows();
						}
					}
					drawall();
					break;
	
				// page down
				case KEY_NPAGE:
					if (signed(cursor + 5) > signed(linecount-1))
					{
						bound_start = linecount - Window->rows();
						bound_end = linecount;
						cursor = bound_end - 1;
					}
					else
					{
						cursor += 5;
	
						if (cursor > bound_end)
						{
							bound_end = cursor + 1;
							bound_start = bound_end - Window->rows();
						}
					}
					drawall();
					break;
	
				case KEY_HOME:
					while (bound_start > 0)
					{
						bound_start-=1;
						bound_end-=1;
					}
					cursor=bound_start;
					drawall();
					break;
	
				case KEY_END:
					while (bound_end < linecount)
					{
						bound_start+=1;
						bound_end+=1;
					}
					cursor=bound_end-1;
					drawall();
					break;
	
				case 'j':
				case KEY_UP:
					if (cursor == 0)
						break;
	
					if (cursor == bound_start)
					{
						bound_start -= 1;
						bound_end -= 1;
					}
	
					cursor -= 1;
					drawall();
					break;
	
				case 'k':
				case KEY_DOWN:
					if (cursor == (linecount-1))
						break;
	
					if (cursor == (bound_end-1))
					{
						bound_start += 1;
						bound_end += 1;
					}
	
					cursor += 1;
					drawall();
					break;
	
				default:
					if (external_key_handler(Z) == false)
						return;
			}
		}
	}
}

void foo(Screen *) {};
void CursesUI::Dialog::execute()
{
	addline(_("$7#<--$! q/x will exit "), foo);

	initialize();
	drawall();
	select();
}


#ifdef HAVE_LIBGPM
int CursesUI::Dialog::gpm_handler(Gpm_Event *event)
{
	if (Gpm_StrictSingle(event->type))
	{
		for (unsigned I = bound_start; I < bound_end; I++)
		{
			unsigned K = I - bound_start;
	
			if (((sy + 2 + K) == event->y) &&
				(sx + 2 <= event->x && event->x <= sx + lx - 1))
			{
				cursor = I;
				drawall();
			}
		}
	}
	else if (Gpm_StrictDouble(event->type))
	{
		for (unsigned I = bound_start; I < bound_end; I++)
		{
			unsigned K = I - bound_start;
	
			if (((sy + 2 + K) == event->y) &&
				(sx + 2 <= event->x && event->x <= sx + lx - 1))
					return -1;
		}
		return -2;
	}

	return 0;
}
#endif
