/* $Id: cursesdlg.h,v 1.5 2001/03/10 14:55:44 ltd Exp $ */ 
#ifndef _dialogs_H
#define _dialogs_H

#include <gpm.h>
#include <string>
#include "wrappers.h"
#include "cursesui.h"
#include <interface/edeityui.h>
#include "screen.h"

#ifdef HAVE_LIBGPM
	#include <func/mouse.h>
#endif

/*
 * Provide the basic implementation of a scrolling dialog that can be
 * placed at any position on the screen, at any width/height, or get the
 * dimensions from the width of the inserted text strings and the length
 * of the list it is to operate on.
 *
 * It implements two methods to deal with user input:
 *
 *     i)  Function that handles a keypress on a certain item.
 *     ii) Return the key from the select function. (NOT IMPLEMENTED YET)
 */
class CursesUI;
class CursesUI::Screen;
class CursesUI::Dialog;
class CursesUI::MenuDrop;

class CursesUI::Dialog
{
	friend class CursesUI;
	friend class CursesUI::MenuDrop;

	typedef ::Screen Screen;

	private:
		Screen &screen;

		struct EntryHandler;
		struct Entry
		{
			char *text;
			EntryHandler *handler;
			unsigned char key;

			Entry() : text(0), handler(0), key(0) {};
			~Entry();
		};

		vector<Entry *> Lines;

		UniqueWindow *Window;
		string TitleText;
		int ly, lx, sy, sx;
		unsigned bound_start, bound_end, cursor;
		unsigned linecount, width;
		bool Initialized;

		void initialize();
		void deinitialize();
		void select();

		virtual void drawall();
		virtual void drawline(int row, unsigned lkey);

	public:
#ifdef HAVE_LIBGPM
		virtual int gpm_handler(Gpm_Event *);
#endif
		virtual bool external_key_handler(int);

		Entry *Selection() { return Lines[cursor]; };
		void free_memory();
		void addline(string, void (*)(Screen *));
		void addline(string, void (::Screen::*)(void));
		void addline(string, void (*)(Screen *, int));
		void addline(string, void (::Screen::*)(int));
		void addline(string);
		void geometry(int, int, int, int);
		void complete_redraw();
		inline void title(string text) { TitleText = text; };
		virtual void execute();

		Dialog(Screen &);
		virtual ~Dialog();
};

struct CursesUI::Dialog::EntryHandler
{
	typedef ::Screen Screen;

	EntryHandler(void (*f)(Screen *, int), Screen *);
	EntryHandler(void (*f)(Screen *), Screen *);
	EntryHandler(void (::Screen::*f)(int), Screen *);
	EntryHandler(void (::Screen::*f)(void), Screen *);

	Screen *screen;
	
	enum {	NoHandler,
		KeyHandler, 
		NullKeyHandler, 
		ScrHandler, 
		NullScrHandler } type;
	union {
		void (*keyf)(Screen *, int);
		void (*nullkeyf)(Screen *);
		void (::Screen::*scrf)(int);
		void (::Screen::*nullscrf)(void);
	} fn;

	inline void operator()(int);
};

inline void CursesUI::Dialog::EntryHandler::operator()(int key)
{
	if (fn.keyf == 0) return;
	switch (type) {
	case KeyHandler: fn.keyf(screen, key); break;
	case NullKeyHandler: fn.nullkeyf(screen); break;
	case ScrHandler: (screen->*(fn.scrf))(key); break;
	case NullScrHandler: (screen->*(fn.nullscrf))(); break;
	default: /* nothing */ ;
	}
}
#endif

