/* $Id: cursesui.cc,v 1.20 2001/06/25 08:17:15 ltd Exp $
 * cursesui.cc: Curses User Interface: all those lovely colors
 */

//old #include <std.h>
//old #include <strings.h>
#include <fcntl.h>
//old #include <stdlib.h>
#include <ncurses.h>
//old #include <sys/time.h>
//old #include <sys/socket.h>
//old #include <sys/ioctl.h>
//old #include <termios.h>

#include <apt-pkg/configuration.h>

#include <parsers/strutl.h>
#include <list/screens.h>
#include <func/config.h>
#include <interface/format_strings.h>
#include <interface/acquireui.h>

#include "acquire-ui.h"
#include "cursesui.h"
#include "colors.h"
#include "wrappers.h"
#include "screen.h"
#include "cursesdlg.h"
#include "init_status.h"
#include "pmi.h"

int CursesUI::top_lines = 2;
int CursesUI::stat_lines = 1;
int CursesUI::acqprogress_lines = 0;
int CursesUI::main_lines = LINES - stat_lines - top_lines;
int CursesUI::dtop_lines = 1;
int CursesUI::desc_lines = 9;
int CursesUI::desc_width, CursesUI::desc_bound_start, CursesUI::desc_bound_end;
vector<string> *CursesUI::curDescLines = NULL;
CursesUI::Dialog *CursesUI::activeDialog = NULL;
colour_system *CursesUI::_clrsys = NULL;

CursesUI::CursesUI(int &, char **&)
{
	wtop  = NULL;
	wmain = NULL;
	wstat = NULL;
	wdtop = NULL;
	wdesc = NULL;

	initscr();
	cbreak();
	start_color();

	curs_set(0);

	signal(SIGWINCH, Resize);

	acqprogress_lines = 0;
	stat_lines = 1;
	top_lines  = 2;
	main_lines = LINES - stat_lines - top_lines;
	dtop_lines = 1;
	desc_lines = 9;

	/* This cannot be a static non-pointer object because we cannot 
	 * initialize colour_system until initscr(), etc have been called
	 */
	_clrsys = new colour_system();

#ifdef HAVE_LIBGPM
	// Initialize the mouse
	gpm_open();
#endif
}

CursesUI::~CursesUI()
{
	delete _clrsys;

	Shutdown();
#ifdef HAVE_LIBGPM
	gpm_close();
#endif
}

/* Main CursesUI Functions */
int CursesUI::Run()
{
	INFO(INFO_DEBUG, _("CursesUI::Run()\n"));

	Startup();
	scrlst.Current()->Redraw();

	int key;

	fd_set readset;
	int highfd;

	while (1)
	{
		FD_ZERO(&readset);

		if (signalfd > 0)
		{
			FD_SET(signalfd, &readset);
			highfd = signalfd;
		}
		else
			highfd = STDIN_FILENO;

		FD_SET(STDIN_FILENO, &readset);

		if (select(highfd + 1, &readset, 0, 0, 0) > 0)
		{
			if (signalfd > 0)
				if (FD_ISSET(signalfd, &readset))
					ProcessMsg(signalfd);

			if (FD_ISSET(STDIN_FILENO, &readset))
			{	
#ifdef HAVE_LIBGPM
//old			key = Gpm_Getch();
				key = Gpm_Wgetch();
#else
				key = getch();
#endif
				if (signalfd > 0 && _Conf.GetBool("edeity.acquire.embedded") == false)
						AcqItf->DispatchKey(key);
				else
					if (scrlst.Current()->kdriver->DispatchKey(key) == false)
						break;
			}
	}
}
	return 0;
}

/* Main CursesUI Functions */
int CursesUI::Passthrough(int fd)
{
	INFO(INFO_DEBUG, _("CursesUI::Passthrough()\n"));

	int key;
	fd_set readset;
	int highfd;

	while (1)
	{
		FD_ZERO(&readset);
		FD_SET(STDIN_FILENO, &readset);
		FD_SET(signalfd, &readset);
		highfd = fd;

		if (select(highfd + 1, &readset, 0, 0, 0) > 0)
		{
			if (FD_ISSET(fd, &readset))
				if (ProcessMsg(fd) == false)
					break;

			if (FD_ISSET(STDIN_FILENO, &readset))
			{	
#ifdef HAVE_LIBGPM
//old			key = Gpm_Getch();
				key = Gpm_Wgetch();
#else
				key = getch();
#endif
				AcqItf->DispatchKey(key);
			}
		}
	}
	return 0;
}

void CursesUI::Startup()
{
	INFO(INFO_DEBUG, _("CursesUI::Startup()\n"));

	// Check that capt can actually run on this terminal.
	if (LINES < 15 || COLS < 80)
	{
		edeity_clean_exit();
		fprintf(stderr, _("Running this program is not recommanded without at least 15 lines and 80 columns.\n"));
		exit(-2);
	}

	wtop = newwin(top_lines, COLS, 0, 0);
	wstat = newwin(stat_lines, COLS, LINES - 1, 0);
	wmain = newwin(main_lines, COLS, top_lines, 0);

	keypad(stdscr, true);

	keypad(wstat, true);
	keypad(wtop, true);
	scrollok(wmain, TRUE);

	wbkgd(wmain, _clrsys->regpair(EDEITY_COLOR_DEFAULT, EDEITY_COLOR_BACK));
	wbkgd(wtop,  _clrsys->regpair(EDEITY_COLOR_TOP_FG, EDEITY_COLOR_TOP_BG));
	wbkgd(wstat, _clrsys->regpair(EDEITY_COLOR_STAT_FG, EDEITY_COLOR_STAT_BG));

	wrefresh(wmain);
	wrefresh(wtop);
	wrefresh(wstat);

	if (curs_set(0) == ERR)
		INFO(INFO_DEBUG, _("Error setting cursor to invisible\n"));
}

/*
 * MUST be called before an exit() or will result in your
 * termIOS settings being stuffed up.
 */

void CursesUI::Shutdown()
{
	INFO(INFO_DEBUG, _("CursesUI::Shutdown()\n"));

	delwin(wmain);
	delwin(wstat);
	delwin(wtop);
	reset_shell_mode();
	curs_set(1);
	endwin();
}

void CursesUI::Restart()
{
	INFO(INFO_DEBUG, _("CursesUI::Restart()\n"));

	CursesUI::Screen *scr = (CursesUI::Screen *) scrlst.Current();
	bool dv = scr->descVisible;

	if (dv) 
		scr->DescWindowHide();
		
	stat_lines = 1;
	acqprogress_lines = 0;
	top_lines  = 2;
	main_lines = LINES - stat_lines - top_lines;
	dtop_lines = 1;
	desc_lines = 9;

	delwin(wmain);
	delwin(wstat);
	delwin(wtop);

	CaptUI->Startup();

	if (dv) 
		CURRENTSCR->DescWindowShow();

	for (::Screen *S = CaptUI->scrlst.Head(); S != NULL; S = S->next)
		S->_boundend = S->_boundstart + main_lines - 1;

	CaptUI->scrlst.AlignAllBounds();

	CURRENTSCR->Redraw();

	return;
}

void CursesUI::Resize(int sig)
{
	INFO(INFO_DEBUG, _("CursesUI::Resize()\n"));

	int fd=open("/dev/tty",O_RDONLY);

	if (fd != -1)
	{
		struct winsize w;

		if (ioctl(fd, TIOCGWINSZ, &w) != -1)
			resizeterm(w.ws_row,w.ws_col); 
		else
			DEBUGEXIT(_("Can't get window size from tty!"));

		close(fd);
 	} 
	else 
		DEBUGEXIT(_("Can't open /dev/tty!"));

	CaptUI->Restart();

	return;
}

// FIXME: Defunct
void CursesUI::PrepareForInstall()
{
/*	clear();
	refresh();*/
}

void CursesUI::RecoverFromInstall()
{
	/*doupdate();*/
}

void CursesUI::TextEntry(char *dispstr, char **retstr)
{
	int i = 0, j = 0;

	**retstr = 0;

	curs_set(1);

	wattrset(wstat, _clrsys->regpair(COLOR_WHITE, COLOR_BLUE));
	mvwblankl(wstat, 0, COLS);

	wmove(wstat, 0, 0);

	wattrset(wstat, _clrsys->regpair(COLOR_CYAN, COLOR_BLUE) | A_BOLD);
	wprintw(wstat, dispstr);

	wattrset(wstat, _clrsys->regpair(COLOR_WHITE, COLOR_BLUE));

	wrefresh(wstat);

	int dispstrlen = strlen(dispstr);

	while (1)
	{
		int c = wgetch(wstat);
		
		switch (c)
		{
		case ' ':
			break;

		case 15: // Ctrl-O

			*(*retstr + j) = 0;

			break;

		case 6: // Ctrl-F
			*(*retstr + j + 1) = 0;
			break;

		case 9: // Ctrl-I
			*(*retstr + j + 1) = 0;
			break;

		case 18: // Ctrl-R
			*(*retstr + j + 1) = 0;
			break;

		case '\n':
		case '\r':
			*(*retstr + j) = 0;

			curs_set(0);

			scrlst.Current()->UpdateStatus();
			//scrlst.Current()->PaintTop();

			return;

		case KEY_BACKSPACE:
			if (j == 0)
				break;

			if (i < j)
			{
				char *x = *retstr + i;
				char *y = (char *)malloc(j-i);

				memcpy(y, x, j-i);

				memcpy(x-1, y, j-i);

				free(y);

				i--;
			}
			else
			{
				*(*retstr + i) = 0;

				i--;
			}

			mvwdelch(wstat, 0, i + dispstrlen);
			wmove(wstat, 0, i + dispstrlen);

			j--;

			wrefresh(wstat);

			break;

		case KEY_LEFT:
			if (j == 0)
				break;

			i--;

			wmove(wstat, 0, i + dispstrlen);

			break;

		case KEY_RIGHT:
			if (j == (signed)(COLS - dispstrlen - 1))
				break;

			if (i == j)
				break;

			i++;

			wmove(wstat, 0, i + dispstrlen);

			break;
			
		case 27:
			break;

		default:
			if (j == (signed)(COLS - dispstrlen - 1))
				break;

			if (i < j)
			{
				char *x = *retstr + i;
				char *y = (char *)malloc(j-i);
				
				memcpy(y, x, j-i);

				*x = c;

				memcpy(x+1, y, j-i);

				free(y);

				mvwinsch(wstat, 0, i + dispstrlen, c);

				i++;
			}
			else
			{
				mvwaddch(wstat, 0, i + dispstrlen, c);

				*(*retstr + i) = c;

				i++;
			}

			wmove(wstat, 0, i + dispstrlen);

			j++;

			wrefresh(wstat);

			break;
		}
	}
}

unsigned char CursesUI::ShowDialog(char *title, char *fmt, ...)
{
	string buffer;
	va_list va;

	va_start(va, fmt);

	for (char *p = fmt; *p != 0; p++)
	{
		if (*p == '%' && *(p+1))
		{
			p++;
			if (*p == '%')
			{
				buffer += '%';
			}
			else if (*p == 's')
			{
				char *s = va_arg(va, char *);
				buffer += s;
			}
		}
		else
			buffer += *p;
	}
	
	va_end(va);

	Dialog MyDialog(*CURRENTSCR);

	MyDialog.title(title);

	char *rep = strdup(buffer.c_str());

	for (char *P = sane_strtok(rep, "\n"); P != NULL; P = sane_strtok(NULL, "\n"))
		MyDialog.addline(P);

	free(rep);

	MyDialog.execute();

	return ' ';
}

void CursesUI::WaitForFetch(pkgAcquireBackend *, int fd)
{
	signalfd = fd;
}

OpProgress *CursesUI::CreateProgressBar()
{
	return new InitStatus(*_config);
}

void CursesUI::CreateAcquireUI()
{
	AcqItf = new CursesUI::AcquireUI(*this);
}

::Screen *CursesUI::CreateScreen(::Screen::ScrInit Op, ::Screen::ScrFunc Func)
{
	return new CursesUI::Screen(Op, Func, *this);
}

void CursesUI::ShowAcquireProgress()
{
	acqprogress_lines = 1;

	main_lines -= 1;
	wresize(wmain, main_lines, COLS);

	wacqprogress = newwin(acqprogress_lines, COLS, LINES - stat_lines - 1, 0);

	wbkgd(wacqprogress, _clrsys->regpair(EDEITY_COLOR_ACQPRG_TXT_RIGHT, EDEITY_COLOR_ACQPRG_BG_RIGHT));

	keypad(wacqprogress, false);
	scrollok(wacqprogress, false);

	wrefresh(wacqprogress);

	for (CursesUI::Screen *S = (CursesUI::Screen *) scrlst.Head(); S != 0; S = (CursesUI::Screen *) S->next)
	{
		S->_boundend -= 1;
		S->BoundsReset();
	}
	
	((CursesUI::Screen *)CURRENTSCR)->DescShuffle(-1);
	CURRENTSCR->Redraw();
}

void CursesUI::HideAcquireProgress()
{
	acqprogress_lines = 0;

	main_lines += 1;
	wresize(wmain, main_lines, COLS);

	for (CursesUI::Screen *S = (CursesUI::Screen *) scrlst.Head(); S != 0; S = (CursesUI::Screen *) S->next)
	{
		S->_boundend += 1;
		S->BoundsReset();
	}
	
	delwin(wacqprogress);

	wacqprogress = NULL;

	((CursesUI::Screen *)CURRENTSCR)->DescShuffle(1);
	CURRENTSCR->Redraw();
}

void CursesUI::UpdateAcquireProgress(const MsgDownloadTotal *Total)
{
	double barlen = Total->Percent * double(COLS);
	char buf[BUFSIZ];

	werase(wacqprogress);

	if (Total->CurrentCPS != 0)
		sprintf(buf, _(" Download: %sB/s %02d%%%% (%s remains)"), 
			SizeToStr(Total->CurrentCPS).c_str(), int(Total->Percent * 100.0), TimeToStr(Total->ETA).c_str());
	else
		sprintf(buf, _(" (No CPS) %02d%%%% complete "), int(Total->Percent * 100.0));

	wattrset(wacqprogress, _clrsys->regpair(COLOR_BLACK, COLOR_WHITE));

	for (int x=0; x < COLS; x++)
		mvwaddch(wacqprogress, 0, x, ' ');

	wmove(wacqprogress, 0, 0);

	if (barlen < strlen(buf))
	{
		char redbuf[COLS];
		int H;
			
		for (H = 0; H < barlen; H++)
			redbuf[H] = buf[H];

		redbuf[H] = 0;

		wattrset(wacqprogress, _clrsys->regpair(COLOR_WHITE, COLOR_RED));
		wprintw(wacqprogress, redbuf);
		wattrset(wacqprogress, _clrsys->regpair(COLOR_BLACK, COLOR_WHITE));
		wprintw(wacqprogress, buf+H);
	}
	else
	{
		wattrset(wacqprogress, _clrsys->regpair(COLOR_WHITE, COLOR_RED));
		wprintw(wacqprogress, buf);

		for (int H = 0; H < (barlen - strlen(buf)); H++)
			waddch(wacqprogress, ' ');
	}

	wrefresh(wacqprogress);
}
/*
//old bool CursesUI::FilterSelect(char **dest, Filter::Target *targ)
bool CursesUI::FilterSelect(char **dest, unsigned long *targ)
{
	*dest = (char *) malloc(BUFSIZ);

	TextEntry(_("Filter: "), dest);

	UniqueWindow w(13, 75, (LINES - 13) / 2, (COLS - 75) / 2);

//old	Filter::Target Base = Filter::Target(
//old		Filter::pkgname | 
//old		Filter::pkgavailver | 
//old		Filter::pkginstlver |
//old		Filter::pkgsection |
//old		Filter::pkgpriority |
//old		Filter::pkgshortdesc |
//old		Filter::pkglongdesc);
//old	Filter::Target New = Filter::Target(0);
	unsigned long Base = Pkg_Name | Pkg_AvailVer | Pkg_InstlVer |
			     Pkg_Section | Pkg_Priority | Pkg_ShortDesc |
			     Pkg_LongDesc;

	unsigned long New = 0;

	curs_set(1);

	while (1)
	{
		werase(w);
		wmove(w, 0, 0);

		wattrset(w, _clrsys->regpair(COLOR_YELLOW,COLOR_BLUE)|A_BOLD);
		mvwprintw(w,0,0, _("Available targets:"));
		wattrset(w, _clrsys->regpair(COLOR_GREEN,COLOR_BLUE)|A_BOLD);
		mvwprintw(w,0,37, _("Selected targets:"));

		wstandend(w);

//old		if (Base & Filter::pkgname)
		if (Base & Pkg_Name)
			mvwprintw(w, 2,0,_(" 1  =>  Package Name"));
//old		if (Base & Filter::pkgavailver)
		if (Base & Pkg_AvailVer)
			mvwprintw(w, 3,0,_(" 2  =>  Package Version (Available)"));
//old		if (Base & Filter::pkginstlver)
		if (Base & Pkg_InstlVer)
			mvwprintw(w, 4,0,_(" 3  =>  Package Version (Installed)"));
//old		if (Base & Filter::pkgsection)
		if (Base & Pkg_Section)
			mvwprintw(w, 5,0,_(" 4  =>  Package Section"));
//old		if (Base & Filter::pkgpriority)
		if (Base & Pkg_Priority)
			mvwprintw(w, 6,0,_(" 5  =>  Package Priority"));
//old		if (Base & Filter::pkgshortdesc)
		if (Base & Pkg_ShortDesc)
			mvwprintw(w, 7,0,_(" 6  =>  Package Short Description"));
//old		if (Base & Filter::pkglongdesc)
		if (Base & Pkg_LongDesc)
			mvwprintw(w, 8,0,_(" 7  =>  Package Long Description"));
	
		
//old		if (New & Filter::pkgname)
		if (New & Pkg_Name)
			mvwprintw(w, 2,36,_(" 1  =>  Package Name"));
//old		if (New & Filter::pkgavailver)
		if (New & Pkg_AvailVer)
			mvwprintw(w, 3,36,_(" 2  =>  Package Version (Available)"));
//old		if (New & Filter::pkginstlver)
		if (New & Pkg_InstlVer)
			mvwprintw(w, 4,36,_(" 3  =>  Package Version (Installed)"));
//old		if (New & Filter::pkgsection)
		if (New & Pkg_Section)
			mvwprintw(w, 5,36,_(" 4  =>  Package Section"));
//old		if (New & Filter::pkgpriority)
		if (New & Pkg_Priority)
			mvwprintw(w, 6,36,_(" 5  =>  Package Priority"));
//old		if (New & Filter::pkgshortdesc)
		if (New & Pkg_ShortDesc)
			mvwprintw(w, 7,36,_(" 6  =>  Package Short Description"));
//old		if (New & Filter::pkglongdesc)
		if (New & Pkg_LongDesc)
			mvwprintw(w, 8,36,_(" 7  =>  Package Long Description"));

		wattrset(w, _clrsys->regpair(COLOR_WHITE,COLOR_BLUE)|A_BOLD);
		mvwprintw(w, 10, 0, _("Select (enter = continue): "));

		switch (wgetch(w))
		{
			case '1': 
//old				if (New & Filter::pkgname) {
//old    					int(New) &= ~Filter::pkgname;
//old					int(Base) |= Filter::pkgname;
//old				} else {
//old					int(New) |= Filter::pkgname;
//old					int(Base) &= ~Filter::pkgname;
//old				}
				if (New & Pkg_Name) {
    					New &= ~Pkg_Name;
					Base |= Pkg_Name;
				} else {
					New |= Pkg_Name;
					Base &= ~Pkg_Name;
				}
				break;
			case '2': 
//old				if (New & Filter::pkgavailver) {
//old					int(New) &= ~Filter::pkgavailver;
//old					int(Base) |= Filter::pkgavailver;
//old				} else {
//old					int(New) |= Filter::pkgavailver;
//old					int(Base) &= ~Filter::pkgavailver;
//old				}
				if (New & Pkg_AvailVer) {
					New &= ~Pkg_AvailVer;
					Base |= Pkg_AvailVer;
				} else {
					New |= Pkg_AvailVer;
					Base &= ~Pkg_AvailVer;
				}
				break;
			case '3': 
//old				if (New & Filter::pkginstlver) {
//old					int(New) &= ~Filter::pkginstlver;
//old					int(Base) |= Filter::pkginstlver;
//old				} else {
//old					int(New) |= Filter::pkginstlver;
//old					int(Base) &= ~Filter::pkginstlver;
//old				}
				if (New & Pkg_InstlVer) {
					New &= ~Pkg_InstlVer;
					Base |= Pkg_InstlVer;
				} else {
					New |= Pkg_InstlVer;
					Base &= ~Pkg_InstlVer;
				}
				break;
			case '4': 
//old				if (New & Filter::pkgsection) {
//old					int(New) &= ~Filter::pkgsection;
//old					int(Base) |= Filter::pkgsection;
//old				} else {
//old					int(New) |= Filter::pkgsection;
//old					int(Base) &= ~Filter::pkgsection;
//old				}
				if (New & Pkg_Section) {
					New &= ~Pkg_Section;
					Base |= Pkg_Section;
				} else {
					New |= Pkg_Section;
					Base &= ~Pkg_Section;
				}
				break;
			case '5': 
//old				if (New & Filter::pkgpriority) {
//old					int(New) &= ~Filter::pkgpriority;
//old					int(Base) |= Filter::pkgpriority;
//old				} else {
//old					int(New) |= Filter::pkgpriority;
//old					int(Base) &= ~Filter::pkgpriority;
//old				}
				if (New & Pkg_Priority) {
					New &= ~Pkg_Priority;
					Base |= Pkg_Priority;
				} else {
					New |= Pkg_Priority;
					Base &= ~Pkg_Priority;
				}
				break;
			case '6': 
//old				if (New & Filter::pkgshortdesc) {
//old					int(New) &= ~Filter::pkgshortdesc;
//old					int(Base) |= Filter::pkgshortdesc;
//old				} else {
//old					int(New) |= Filter::pkgshortdesc;
//old					int(Base) &= ~Filter::pkgshortdesc;
//old				}
				if (New & Pkg_ShortDesc) {
					New &= ~Pkg_ShortDesc;
					Base |= Pkg_ShortDesc;
				} else {
					New |= Pkg_ShortDesc;
					Base &= ~Pkg_ShortDesc;
				}
				break;
			case '7': 
//old				if (New & Filter::pkglongdesc) {
//old					int(New) &= ~Filter::pkglongdesc;
//old					int(Base) |= Filter::pkglongdesc;
//old				} else {
//old					int(New) |= Filter::pkglongdesc;
//old					int(Base) &= ~Filter::pkglongdesc;
//old				}
				if (New & Pkg_LongDesc) {
					New &= ~Pkg_LongDesc;
					Base |= Pkg_LongDesc;
				} else {
					New |= Pkg_LongDesc;
					Base &= ~Pkg_LongDesc;
				}
				break;
			case '\n':
				if (New != 0)
					goto finished;
		}
	}

	finished:

	*targ = New;

	curs_set(0);

	return true;
}
*/
void CursesUI::DrawTrees()
{
	INFO(INFO_DEBUG, _("CursesUI::DrawTrees()\n"));
	scrlst.Current()->RedrawTree();
}

void CursesUI::DrawScreens()
{
	INFO(INFO_DEBUG, _("CursesUI::DrawScreens()\n"));
	INFO(INFO_DEBUG, "%p\n", scrlst.Current());
	scrlst.Current()->Redraw();
}

EDeityUI::pkgMgrInt *CursesUI::CreatePMI()
{
	return new CursesUI::pkgMgrInt;
}


// Creator function
extern "C" {
	EDeityUI *CreateInterface(int &argc, char **&argv) { return new CursesUI(argc, argv); }
};

