/* $Id: cursesui.h,v 1.17 2001/06/25 08:17:15 ltd Exp $ */
#ifndef _cursesui_H
#define _cursesui_H

//old #include <std.h>
//old #include <stdarg.h>
#include <ncurses.h>
#include <list/pkgtree.h>
#include <list/screens.h>
#include <list/sorting.h>

#include <interface/edeityui.h>

#include "colors.h"

#ifdef HAVE_LIBGPM
	#include <func/mouse.h>
#endif

class UniqueWindow;

class CursesUI : public EDeityUI 
{
	friend class Captris;
protected:
	class Dialog;
	class MenuDrop;
	class Screen;
	class InitStatus;
	class AcquireUI;
	class DebConfItf;
	class pkgMgrInt;

	friend class Dialog;
	friend class MenuDrop;
	friend class Screen;
	friend class ::colour_system;
	
	WINDOW *wtop;
	WINDOW *wmain;
	WINDOW *wacqprogress;
	WINDOW *wstat;
	WINDOW *wdesc;
	WINDOW *wdtop;

	static int top_lines, stat_lines, main_lines, acqprogress_lines, dtop_lines, desc_lines;
	static int desc_width, desc_bound_start, desc_bound_end;
	static vector<string> *curDescLines;
	static Dialog *activeDialog;

public:
	CursesUI(int &, char **&);
	~CursesUI();

	static colour_system *_clrsys;

	int Run();
	int Passthrough(int fd);
	void Startup();
	void Shutdown();
	void Restart();

	void ShowAcquireProgress();
	void HideAcquireProgress();
  void UpdateAcquireProgress(const MsgDownloadTotal *Total);
	static void Resize(int);

	void PrepareForInstall();
	void RecoverFromInstall();

	// Curses drawing functions
	static void mv_printw_color(WINDOW *, int, int, char *, ...);
	static int strlen_color(char *);
	void mvwblankl(WINDOW *, int, int);
	void my_werase(WINDOW *);
	void main_printf(WINDOW *, char *,...);
	void stat_printf(WINDOW *, char *,...);
	void mv_stat_printf(WINDOW *, int, int, char *, ...);

	// Window formatting
	void mv_print_line(PackageTree::Item *, int);
	void PaintPackage(PackageTree::Item *, bool, int);
	void PaintHeader(PackageTree::Item *, bool, int);
	void PaintDepend(PackageTree::Item *, bool, int);
	void pkgsetattribs(PackageTree::Item *P, bool cursor = false);

	void TextEntry(char *, char **);
	unsigned char ShowDialog(char *, char *, ...);
//old	bool FilterSelect(char **, Filter::Target *);

	void DrawTrees();
	void DrawScreens();

	::Screen *CreateScreen(::Screen::ScrInit, ::Screen::ScrFunc);
	OpProgress *CreateProgressBar();
	void CreateAcquireUI();
	EDeityUI::pkgMgrInt *CreatePMI();

	void WaitForFetch(pkgAcquireBackend *, int fd);
};

#endif
