/* $Id: cursfmt.cc,v 1.12 2001/05/15 08:32:58 ltd Exp $ */
#include <std.h>
#include <func/apt.h>
#include <interface/format_strings.h>
#include <list/screens.h>
#include <list/sections.h>
#include <parsers/strutl.h>
#include <ncurses.h>
#include "colors.h"
#include "cursesui.h"

/* CursesUI utility functions */

void CursesUI::mv_printw_color(WINDOW *win, int pos_y, int pos_x, char *fmt, ...)
{
	char *x, *y;
	short fc, bc, oldcolor;
	va_list va;
	attr_t newattrs=0, oldattrs;
	char BUF[768];
	short of, ob;
	
	wattr_get(win, &oldattrs, &oldcolor, NULL);
	pair_content(oldcolor, &of, &ob);

	va_start(va, fmt);
	vsnprintf(BUF, 767, fmt, va);
	va_end(va);

	wmove(win, pos_y, pos_x);
	
	for (x = BUF; *x != 0; x++) {
		if (*x == '$') {
			x++;

			if (*x == '$') {
				waddch(win, *x);
				continue;
			}

			if (*x == '!') {
					wattrset(win, oldattrs);
					continue;
			}

			if (*x == 'H' && *(x+1) == 'L') {
				waddch(win, ACS_HLINE);
				++x;
				continue;
			}
			else if (*x == 'L' && *(x+1) == 'T') {
				waddch(win, ACS_LTEE);
				++x;
				continue;
			}
			else if (*x == 'L' && *(x+1) == 'L') {
				waddch(win, ACS_LLCORNER);
				++x;
				continue;
			}
			else if (*x == 'R' && *(x+1) == 'A') {
				waddch(win, ACS_RARROW);
				++x;
				continue;
			}
			else if (*x == 'T' && *(x+1) == 'T') {
				waddch(win, ACS_TTEE);
				++x;
				continue;
			}
					
			switch (*x) {
				case '0': case '1': 
				case '2': case '3': 
				case '4': case '5': 
				case '6': case '7':
					y = x;
					x++;
					
					if (*x == '/') {
						x++;
						switch (*x)
						{
							case '0': bc = COLOR_BLACK; break;
							case '1': bc = COLOR_RED; break;
							case '2': bc = COLOR_GREEN; break;
							case '3': bc = COLOR_YELLOW; break;
							case '4': bc = COLOR_BLUE; break;
							case '5': bc = COLOR_MAGENTA; break;
							case '6': bc = COLOR_CYAN; break;
							case '7': bc = COLOR_WHITE; break;
							default: bc = ob; break;
						}
						x++;
					}
					else
						bc = ob;

					if (*x == '#') {
						newattrs = A_BOLD;
						x++;
					}
					else
						newattrs = 0;

					x--;
					
					switch (*y)
					{
						case '0': fc = COLOR_BLACK; break;
						case '1': fc = COLOR_RED; break;
						case '2': fc = COLOR_GREEN; break;
						case '3': fc = COLOR_YELLOW; break;
						case '4': fc = COLOR_BLUE; break;
						case '5': fc = COLOR_MAGENTA; break;
						case '6': fc = COLOR_CYAN; break;
						case '7': fc = COLOR_WHITE; break;
						default: goto InvalidColor;
					}
					wattron(win, _clrsys->regpair(fc, bc) | newattrs);
InvalidColor:
					continue;
					
			}
		}

		waddch(win, *x);
	}

	wattrset(win, COLOR_PAIR(oldcolor) | oldattrs);
}

int CursesUI::strlen_color(char *Text)
{
	char *x, *y;

	int length = 0;

	for (x = Text; *x != 0; x++)
	{
		if (*x == '$')
		{
			x++;

			if (*x == '$')
			{
				length++;
				continue;
			}

			if (*x == '!')
				continue;

			if (((*x == 'H' && *(x+1) == 'L')) ||
				((*x == 'L' && *(x+1) == 'T')) ||
				((*x == 'L' && *(x+1) == 'L')) ||
				((*x == 'T' && *(x+1) == 'T')) ||
				((*x == 'R' && *(x+1) == 'A')))
			{
				++x;
				++length;
				continue;
			}

			switch (*x)
			{
				case '0': case '1':
				case '2': case '3':
				case '4': case '5':
				case '6': case '7':

					y = x;

					x++;
					
					if (*x == '/')
					{
						x++;

						switch (*x)
						{
							case '0': case '1':
							case '2': case '3':
							case '4': case '5':
							case '6': case '7':
								break;
						}

						x++;
					}

					if (*x == '#')
						x++;

					x--;
					
					continue;
			}
		}
		length ++; 
	}

	return length;

}

void CursesUI::mvwblankl(WINDOW * win, int y, int x_len)
{
	int x;
	for (x = 0; x < x_len; x++)
		mvwaddch(win, y, x, ' ');
}

void CursesUI::my_werase(WINDOW *win)
{
	register short x, y;

	for (y = 0; y <= win->_maxy; y++)
		for (x = 0; x <= win->_maxx; x++)
			mvwaddch(win, y, x, ' ');
}

void CursesUI::mv_print_line(PackageTree::Item *p, int row)
{
	/*
	 * We must determine what TYPE of line this is and call the appropriate
	 * wrapper function for it.
	 */

	bool cursor = (p == CURRENTSCR->cs);

	if (p->IsDepend() && p->IsExpanded())
	{
		PaintPackage(p, cursor, row);
		return;
	}
	
	switch (p->Type)
	{
		case PackageTree::Item::Div_State:
		case PackageTree::Item::Div_Sect:
		case PackageTree::Item::Div_Pri:
		case PackageTree::Item::Div_Custom:
			PaintHeader(p, cursor, row);
			break;

		case PackageTree::Item::Pkg:
			PaintPackage(p, cursor, row);
			break;

		case PackageTree::Item::Dep:
			PaintDepend(p, cursor, row);
			break;

		default:
			/* in theory this can't happen... ? */
			break;
	}
}

/*
 * Properly format the sort method dividers in the package list
 */
void CursesUI::PaintHeader(PackageTree::Item *P, bool cursor, int row)
{
	int i;
	// Defaults
	int StartPoint = 0;
	int Factor = 2;

	wstandend(wmain);
	scrollok(wmain, false);
	wmove(wmain, row, 0);
	wclrtoeol(wmain);

	wattron(wmain, (*_clrsys)[EDEITY_ATTR_TREE]);

	// Fill in the tree bits
	PackageTree::Item *q = P;
	int X = P->increment * Factor;
	while (q->parent && X > 0)
	{
		X -= Factor;

		if (q->parent->next)
			mvwaddch(wmain, row, X, ACS_VLINE);

		q = q->parent;
	}

	pkgsetattribs(P, cursor);

	string s = scrlst.Current()->Packages->GetHeaderDesc(P);

	const char *exp;
	if (P->child != NULL)
	{
		if (P->IsExpanded())
			exp = P->next == 0 ? FMT_TREE_TERMNODE_EXP : FMT_TREE_INTERNODE_EXP;
		else
			exp = P->next == 0 ? FMT_TREE_TERMNODE_NOTEXP : FMT_TREE_INTERNODE_NOTEXP;
	}
	else
		exp = P->next == 0 ? FMT_TREE_TERMNODE_FLAT : FMT_TREE_INTERNODE_FLAT;


	int len_left = strlen_color((char *) s.c_str()) + strlen_color((char*)exp) - CURRENTSCR->scroll;

	mv_printw_color(wmain, row, StartPoint + (Factor*P->increment), "%s%s", exp, CURRENTSCR->scroll > s.length() ? "" : s.c_str() + CURRENTSCR->scroll);

	for (i = StartPoint + (Factor * P->increment) + len_left; i < COLS; i++)
			mvwaddch(wmain, row, i, ' ');

	scrollok(wmain, true);

	wrefresh(wmain);
}

/*
 * Display one package on one line on one screen [Capisch?]
 */
void CursesUI::PaintPackage(PackageTree::Item *P, bool cursor, int row)
{
	int Factor = 2;
	char left[256], right[256];
	int len_left=0, len_right=0;
	int X = P->increment * Factor;
	pkgCache::PkgIterator & I = P->TargetPkg;
	pkgCache::VerIterator V = P->Version(), InstV = P->InstVersion();
	PackageTree::Item *q = P;

	wstandend(wmain);
	wmove(wmain, row, 0);
	wclrtoeol(wmain);
	wattrset(wmain, (*_clrsys)[EDEITY_ATTR_TREE]);

	// Fill in the tree bits
	while (q->parent && X > 0)
	{
		X -= Factor;

		if (q->parent->next)
			mvwaddch(wmain, row, X, ACS_VLINE);

		q = q->parent;
	}

	// Set the colors for the event
	
	// wattrset(wmain, _clrsys->regpair(EDEITY_COLOR_TREE_NODE_FG, EDEITY_COLOR_TREE_NODE_BG) | A_BOLD);

	// Make sure the strings are not too big ..

	char *Priority=NULL, *Name=NULL, *CandVer=NULL, *InstVer=NULL, *Section=NULL;
	unsigned short PriorityLen=0, NameLen=0, CandVerLen=0, InstVerLen=0, SectionLen=0;

	// Lengths are based on which view mode we are in

	switch (scrlst.Current()->mode)
	{
		case ::Screen::Terse:
			PriorityLen = 9;
			NameLen     = 17;
			CandVerLen  = 18;
			InstVerLen  = 18;
			SectionLen  = 10;
			break;

		case ::Screen::Verbose:
			PriorityLen = 3;
			NameLen     = 12;
			CandVerLen  = 10;
			InstVerLen  = 10;
			SectionLen  = 8;
			break;
	}

	/*
	 * Chomp all the strings to the desired lengths
	 */

	Name = period_string(I.Name(), NameLen);

	if (V.end() == false)
	{
		CandVer = period_string(V.VerStr(), CandVerLen);

		if (V.PriorityType() != 0 && strcmp(V.PriorityType(), "") != 0)
			Priority = period_string(V.PriorityType(), PriorityLen);
		else
			Priority = period_string(FMT_PKG_PRI_NULL, PriorityLen);
	}
	else
		CandVer = FMT_PKG_CANDVER_NULL;

	if (InstV.end() == false)
		InstVer = period_string(InstV.VerStr(), InstVerLen);
	else
		InstVer = FMT_PKG_CANDVER_NULL;
	
	if (I.Section() && V.end() == false)
	{
		char *x;

		for(x = (char *)I.Section(); *x != '/' && *x != 0; x++);

		if (*x == '/')
			x++;
		else
			x = (char *)I.Section();

		Section = period_string(x, SectionLen);
	}

	char *leftptr=NULL;

	if (scrlst.Current()->mode == ::Screen::Terse)
	{
		len_left = snprintf(left, 255, FMT_PKG_VWNORM_LEFT, Priority, Name, CandVer);
		leftptr = strdup(left);
		len_right = snprintf(right, 255, FMT_PKG_VWNORM_RIGHT, InstVer,
			V->Size / 1024, V->InstalledSize / 1024, Section ? Section : "<none>");

		if (P->IsExpanded())
			len_left += strlen_color(FMT_PKG_EXPN);
		else
			len_left += strlen_color(FMT_PKG_NEXPN);
	}
	else
	{
		string s = _("No available version");

		if (V.end() == false) 
			s = Cache->GetPkgRecs().Lookup(V.FileList()).ShortDesc();

		len_left = sprintf(left, FMT_PKG_VWVERB_LEFT, 
			(*Cache)[I].Status == 1 ? 'I' : (
				(InstV.end() == false || (*Cache)[I].Delete() == true) && (*Cache)[I].NewInstall() == false ? 'U' : 'N'),
			(*Cache)[I].InstBroken() ? 'B' : (
				(*Cache)[I].NewInstall() ? 'I' : (
					(*Cache)[I].Upgrade() ? 'U' : (
						(*Cache)[I].Downgrade() ? 'D' : (
							(*Cache)[I].Install() ? 'I' : (
								(*Cache)[I].Delete() ? (((*Cache)[I].iFlags & pkgDepCache::Purge) ? 'P' : 'R') : (
									'H')))))),
			Priority, 
			Name, 
			Section ? Section : FMT_PKG_SECT_NULL, 
			CandVer, 
			InstVer, 
			V->Size / 1024, 
			V->InstalledSize / 1024, 
			s.c_str());

		if (CURRENTSCR->scroll >= (unsigned)len_left)
		{
			leftptr = strdup("");
			len_left = 0;
		}
		else
		{
			leftptr = period_string(left + CURRENTSCR->scroll, COLS - (P->increment*2) - 4);
			len_left = strlen(leftptr);
		}
	}


	delete Name;

	if (InstV.end() == false)
		delete InstVer;

	if (V.end() == false)
	{
		delete Priority;
		delete CandVer;
		delete Section;
	}

	scrollok(wmain, false);
	pkgsetattribs(P, cursor);

	const char *exp;
	if (P->child != 0 || !V.DependsList().end())
	{
		if (P->IsExpanded())
			exp = P->next == 0 ? FMT_TREE_TERMNODE_EXP : FMT_TREE_INTERNODE_EXP;
		else
			exp = P->next == 0 ? FMT_TREE_TERMNODE_NOTEXP : FMT_TREE_INTERNODE_NOTEXP;
	}
	else
	exp = P->next == 0 ? FMT_TREE_TERMNODE_FLAT : FMT_TREE_INTERNODE_FLAT;

	mv_printw_color(wmain, row, P->increment*2, "%s%s", exp, leftptr);

	free(leftptr);

	if (scrlst.Current()->mode == ::Screen::Terse)
	{
		mvwprintw(wmain, row, COLS - len_right, "%s", right);
		for(register int i = 0; i <= (COLS - len_right - len_left - 1); i++)
			mvwaddch(wmain, row, len_left + i, ' ');
	}
	else
	{
		for(register int i = COLS; i > (len_left + (P->increment*2) + 3); i--)
			mvwaddch(wmain, row, i, ' ');
	}
		
	scrollok(wmain, true);
}

/*
 * Display one dependency on one line on one screen [Capisch?]
 */
void CursesUI::PaintDepend(PackageTree::Item *P, bool cursor, int row)
{
	int Factor = 2;

	scrollok(wmain, false);
	wstandend(wmain);
	wmove(wmain, row, 0);
	wclrtoeol(wmain);
	wattrset(wmain, (*_clrsys)[EDEITY_ATTR_TREE]);

	// Fill in the tree bits
	PackageTree::Item *q = P;
	int X = P->increment * Factor;
	while (q->parent && X > 0)
	{
		X -= Factor;

		if (q->parent->next)
			mvwaddch(wmain, row, X, ACS_VLINE);

		q = q->parent;
	}

	pkgsetattribs(P, cursor);

	const char *e;

	if (P->Version().end() == false)
	{
		if (P->IsExpanded())
			e = P->next == 0 ? FMT_TREE_TERMNODE_EXP : FMT_TREE_INTERNODE_EXP;
		else
			e = P->next == 0 ? FMT_TREE_TERMNODE_NOTEXP : FMT_TREE_INTERNODE_NOTEXP;
	}
	else
		e = P->next == 0 ? FMT_TREE_TERMNODE_FLAT : FMT_TREE_INTERNODE_FLAT;
	
	char buf[256], *scrollptr;
	
	pkgCache::DepIterator *D = static_cast<pkgCache::DepIterator *>(P->Extra);

	char *chompname = period_string(D->TargetPkg().Name(), 20);
	const char *tver;
	if (D->TargetVer() == 0)
		tver = "";
	else
		tver = D->TargetVer();

	sprintf(buf, "%-10s %-15s %2s %-15s", D->DepType(), chompname, D->CompType(), tver);

	delete chompname;

	for(scrollptr=buf; *scrollptr != 0 && (scrollptr - buf) < (int)CURRENTSCR->scroll; scrollptr++);

	mv_printw_color(wmain, row, P->increment*2, "%s%s", e, scrollptr);

	for (int i = (P->increment*2) + strlen(scrollptr) + strlen_color((char *)e); i < COLS; i++)
		mvwaddch(wmain, row, i, ' ');

	scrollok(wmain, true);
}

//old void CursesUI::pkgsetattribs(PackageTree::Item *P, bool cursor = false)
void CursesUI::pkgsetattribs(PackageTree::Item *P, bool cursor)
{
	int attr = (*_clrsys)[EDEITY_ATTR_DEFAULT];

	switch (P->Type)
	{
		case PackageTree::Item::Div_State:
		case PackageTree::Item::Div_Sect:
		case PackageTree::Item::Div_Pri:
		case PackageTree::Item::Div_Custom:
			attr = (*_clrsys)[EDEITY_ATTR_HEADER];
			break;
		case PackageTree::Item::Dep:
			if (!P->IsExpanded())
			{
				attr = (*_clrsys)[EDEITY_ATTR_DEPENDS];
				break;
			}
		case PackageTree::Item::Pkg:
		{
			pkgCache::PkgIterator I = P->TargetPkg;
			pkgCache::VerIterator InstV = (*Cache)[I].InstVerIter(*Cache);

			bool MarkInstall = (*Cache)[I].Install();
			bool MarkDelete  = (*Cache)[I].Delete();

			// Normal packages colors are chosen here
				
			if ((*Cache)[I].Upgrade() == true && (*Cache)[I].NewInstall() == false)
				attr = (*_clrsys)[EDEITY_ATTR_UPGRADE_MARK];
			else if (MarkInstall == true)
				attr = (*_clrsys)[EDEITY_ATTR_INSTALL_MARK];
			else if (MarkDelete == true)
			{
				bool Purge = (*Cache)[I].iFlags & pkgDepCache::Purge;
				if (Purge == true)
					attr = (*_clrsys)[EDEITY_ATTR_PURGE_MARK];
				else
					attr = (*_clrsys)[EDEITY_ATTR_REMOVE_MARK];
			}
			else if (P->IsMatched())
				attr = (*_clrsys)[EDEITY_ATTR_SEARCH_MATCH];
			else if ((*Cache)[I].Status == 1)
				attr = (*_clrsys)[EDEITY_ATTR_AVAILABLE];
			else if (InstV.end() == false)
				attr = (*_clrsys)[EDEITY_ATTR_INSTALLED];
			break;

		}	 
		default:
			/* unknown type.... */
			break;
	}

	if (cursor == true)
		attr |= (*_clrsys)[EDEITY_ATTR_CURSOR];

	if (P->Type == PackageTree::Item::Pkg)
	{
		short f, b;
		pair_content(PAIR_NUMBER(attr), &f, &b);
		INFO(INFO_DEBUG, _("Package = %s, bold = %d, rev = %d, color = %d (%d,%d)\n"), P->TargetPkg.Name(), (attr & A_BOLD) ? 1 : 0, (attr & A_REVERSE) ? 1 : 0, PAIR_NUMBER(attr), f, b);
	}
	
	wattrset(wmain, attr);

	return;
}
