/* $Id: screen.cc,v 1.20 2001/06/22 14:59:17 ltd Exp $
 * Curses virtual screen interface
 */

//old #include <fcntl.h>
//old #include <stdlib.h>
#include <apt-pkg/configuration.h>
#include <std.h>
//old #include <strings.h>
#include <func/apt.h>
#include <func/keys.h>
#include <parsers/strutl.h>
#include <list/screens.h>
#include <list/sorting.h>
#include <list/pkgtree.h>
#include <list/sections.h>
#include <interface/format_strings.h>
#include <func/messages.h>
//old #include <sys/time.h>
//old #include <sys/socket.h>
//old #include <sys/ioctl.h>
#include "colors.h"
#include "cursesui.h"
#include "cursesdlg.h"
#include "cursesmnu.h"
#include "init_status.h"
#include "wrappers.h"
#include "sortui.h"
#include "screen.h"

using namespace CursesSort;

CursesUI::Screen::Screen(::Screen::ScrInit Op, ::Screen::ScrFunc Func, CursesUI &_ui)
	: ::Screen(Op, Func), ui(_ui)
{
	scroll = 0;
	_boundstart = 0;
	_boundend = CursesUI::main_lines - 1;
	_cursor = 0;

	descVisible = false;
}

void CursesUI::Screen::PaintDescriptionTop(void)
{
	/* Should put something here, in theory. */

	werase(ui.wdtop);

	switch (cs->Type)
	{
		case PackageTree::Item::Pkg:
			mvwprintw(ui.wdtop, 0, 0, "%-15s %s, %s; %s", 
				cs->TargetPkg.Name(), 
				(*Cache)[cs->TargetPkg].InstVerIter(*Cache).end() == false ? _("installed") : _("not installed"),
				(*Cache)[cs->TargetPkg].Status == 0 ? _("upgraded") : _("not upgraded"),
				(*Cache)[cs->TargetPkg].Install() == true ? _("install") : ((*Cache)[cs->TargetPkg].Delete() == true ? _("delete") : _("unchanged")));
			break;

		default:
			break;
	}

	wrefresh(ui.wdtop);
}

void CursesUI::Screen::UpdateDescription()
{
	if (ui.wdesc == NULL)
		return;

	werase(ui.wdesc);
	wmove(ui.wdesc, 0, 0);

	PaintDescriptionTop();

	switch (cs->Type)
	{
		case PackageTree::Item::Pkg:
			break;
		case PackageTree::Item::Div_Sect:
			mvwprintw(ui.wdesc, desc_lines/2, (COLS-strlen(FMT_DWIN_DIV_SECT))/2, "%s", FMT_DWIN_DIV_SECT);
			wrefresh(ui.wdesc);
			return;
		case PackageTree::Item::Div_Pri:
			mvwprintw(ui.wdesc, desc_lines/2, (COLS-strlen(FMT_DWIN_DIV_PRI))/2, "%s", FMT_DWIN_DIV_PRI);
			wrefresh(ui.wdesc);
			return;
		case PackageTree::Item::Div_State:
			mvwprintw(ui.wdesc, desc_lines/2, (COLS-strlen(FMT_DWIN_DIV_STATE))/2, "%s", FMT_DWIN_DIV_STATE);
			wrefresh(ui.wdesc);
			return;
		case PackageTree::Item::Div_Custom:
		{
			char buf[BUFSIZ];
			snprintf(buf, BUFSIZ, FMT_DWIN_DIV_CUST, (char *) cs->Extra);
			mvwprintw(ui.wdesc, desc_lines/2, (COLS-strlen(buf))/2, buf);
			wrefresh(ui.wdesc);
			return;
		}
		case PackageTree::Item::Dep:
		{
			char buf[BUFSIZ];
			snprintf(buf, BUFSIZ, _("This is a dependency for %s (%s)"), ((pkgCache::PkgIterator &)*cs->parent).Name(), ((pkgCache::PkgIterator &)*cs->parent).VersionList().VerStr());
			mvwprintw(ui.wdesc, desc_lines/2, (COLS-strlen(buf))/2, buf);
			wrefresh(ui.wdesc);
			return;
		}
		default: return;
	}

	delete curDescLines;
	curDescLines = new vector<string>(5);
	desc_width = 0;

	string s = Cache->GetPkgRecs().Lookup(cs->TargetPkg.VersionList().FileList()).LongDesc();

	char *scpy = (char *)malloc(s.length()+1);

	memcpy(scpy, s.c_str(), s.length()+1);

	int I = 0;

	for (char *P = strtok(scpy, "\n"); P != NULL; P = strtok(NULL, "\n"))
	{
		++I;

		if (I == 1)
			continue;

		++desc_width;
	
		if (strncmp(P, " .", 2) == 0)
			P = "";

		if (strlen(P) >= (unsigned)COLS)
			P[COLS - 1] = 0;

		if (curDescLines->size() == (unsigned)desc_width)
			curDescLines->resize(curDescLines->size() + 5);

		(*curDescLines)[desc_width - 1] = (string) P;

		if (desc_width <= desc_lines)
			if (strlen(P) >= scroll)
				mvwprintw(ui.wdesc, desc_width - 1, 0, "%s", P + scroll);
	}

	free(scpy);

	desc_bound_start = 0;

	if (desc_width < desc_lines)
		desc_bound_end = desc_width;
	else
		desc_bound_end = desc_lines;

	wrefresh(ui.wdesc);
}


void CursesUI::Screen::UpdateStatus(void)
{
	werase(ui.wstat);

#if 0
	mvwprintw(ui.wstat, 0, 0, _("cur=%p, next=%p, prev=%p, child=%p, parent=%p"), (PackageTree::Item *) cs, cs->next, cs->prev, cs->child, cs->parent);
	wrefresh(ui.wstat);
	return;
	//for debugging
	mvwprintw(ui.wstat, 0, 0, _("cursor = %d, boundstart = %d, boundend = %d"), _cursor, _boundstart, _boundend);
	wrefresh(ui.wstat);
	return;
#endif

	mv_printw_color(ui.wstat, 0, 0, FMT_STAT_LEFT,
		ID,
		long((double(_cursor)) / double(Packages->Length()-1) * 100.0));

	char buf[COLS];
	int len;

	sprintf(buf, FMT_STAT_CENTER,
		(*Cache)->InstCount(), (*Cache)->DelCount(), (*Cache)->KeepCount(), (*Cache)->BrokenCount());

	len = strlen_color(buf);

	mv_printw_color(ui.wstat, 0, (COLS - len) / 2, "%s", buf);

	// Display the version, and other info.

	if (Packages->Filters->Length() > 0)
			sprintf(buf, FMT_STAT_FILT FMT_STAT_RIGHT,
				Packages->Filters->Length(),
				PACKAGE " " VERSION);
	else
		 sprintf(buf, FMT_STAT_RIGHT, PACKAGE " " VERSION);

	len = strlen_color(buf);

	mv_printw_color(ui.wstat, 0, COLS - len, buf);
		
	wrefresh(ui.wstat);
}

void CursesUI::Screen::PaintTop(void)
{
	char left[256], right[256];
	int len_left=0, len_right=0;

	// Draw the menu

	wattrset(ui.wtop, _clrsys->regpair(COLOR_WHITE, COLOR_BLACK)|A_BOLD);

	for (register int i = 0; i < COLS; i++)
		mvwaddch(ui.wtop, 0, i, ' ');

	wattron(ui.wtop, A_UNDERLINE);

	unsigned alg = ((COLS)/6);

	mv_printw_color(ui.wtop, 0, (alg*0), _("> O$3#peration$! "));
	mv_printw_color(ui.wtop, 0, (alg*1), _("> M$3#ovement$!  "));
	mv_printw_color(ui.wtop, 0, (alg*2), _("> T$3#oggles$!   "));
	mv_printw_color(ui.wtop, 0, (alg*3), _("> A$3#lter$!     "));
	mv_printw_color(ui.wtop, 0, (alg*4), _("> F$3#ilters$!   "));
	mv_printw_color(ui.wtop, 0, (alg*5), _("> S$3#orting$!   "));

	// Draw the information for the main window

	wattrset(ui.wtop, _clrsys->regpair(COLOR_WHITE, COLOR_BLUE)|A_BOLD);

	string whitespace;

	for (int I = 0; I <= (((Packages->Sorting->DividerCount()*2) + 1) - 2); I++)
		whitespace += " ";

	for (register int i = 0; i < COLS; i++)
		mvwaddch(ui.wtop, 1, i, ' ');

	if (mode == Screen::Terse)
	{
	
		len_left = snprintf(left, 255, "%s" FMT_TOP_VWNORM_LEFT, whitespace.c_str(), _("Priority"), _("Package"), _("Target-Ver"));
		len_right = snprintf(right, 255, FMT_TOP_VWNORM_RIGHT, _("Instld-Ver"), _("Size"), _("Instsize"), _("Section"));

		mvwprintw(ui.wtop, 1, 0, left);

		mvwprintw(ui.wtop, 1, COLS - len_right, "%s", right);
	}
	else
	{
		len_left = snprintf(left, COLS, "%s" FMT_TOP_VWVERB_LEFT, whitespace.c_str(), _("Pri"), _("Package"), _("Section"), _("Target-Ver"), _("Instld-Ver"), _("PkgSZ"), _("InstSZ"), _("Description"));

		for (register int i = len_left; i < COLS; i++)
			mvwaddch(ui.wtop, 1, i, ' ');

		char *ptr;
		
		if (scroll >= (unsigned)len_left)
			ptr = "";
		else
			ptr = left + scroll;

		mvwprintw(ui.wtop, 1, 0, "%s", ptr);
	}

	wrefresh(ui.wtop);
}

void CursesUI::Screen::Update()
{
	wrefresh(ui.wmain);
}

void CursesUI::Screen::RedrawRow(int row, PackageTree::Item *p)
{
	ui.mv_print_line(p, row);
}

/*
 *  Redraws the current screen (main + statusbar)
 */
void CursesUI::Screen::Redraw()
{
	INFO(INFO_DEBUG, _("CursesUI::Screen::Redraw() - Redrawing screen\n"));
	RedrawTree();
	UpdateStatus();
	PaintTop();
	UpdateDescription();
}

/*
 *  Refreshes all the windows
 */
void CursesUI::Screen::RefreshCurrent()
{
	INFO(INFO_DEBUG, _("CursesUI::Screen::RefreshCurrent() - Refreshing all windows\n"));
	redrawwin(ui.wmain);
	wrefresh(ui.wmain);
	redrawwin(ui.wstat);
	wrefresh(ui.wstat);
	redrawwin(ui.wtop);
	wrefresh(ui.wtop);

	if (ui.wdesc != NULL)
	{
		redrawwin(ui.wdesc);
		wrefresh(ui.wdesc);
		redrawwin(ui.wdtop);
		wrefresh(ui.wdtop);
	}
}

void CursesUI::Screen::ClearMain()
{
	wstandend(ui.wmain);

	for (int y=0; y < getmaxy(ui.wmain); y++)
		for (int x=0; x < getmaxx(ui.wmain); x++)
			mvwaddch(ui.wmain, y, x, ' ');
}

/*
 *  Redraws the current screen list window only
 */
void CursesUI::Screen::RedrawTree()
{
	INFO(INFO_DEBUG, _("CursesUI::Screen::RedrawTree() - Redrawing tree on screen\n"));
	register unsigned I;
	PackageTree::Iterator T;

	for (I = 0, T = bs; !T.Invalid(); I++, T++)
	{
		if (T == be.next())
			break;
		ui.mv_print_line(T, I);
	}

	wstandend(ui.wmain);
	scrollok(ui.wmain, false);

	while (I < (unsigned int)main_lines)
	{
		for (int x=0; x < getmaxx(ui.wmain); x++)
			mvwaddch(ui.wmain, I, x, ' ');
		I++;
	}

	wrefresh(ui.wmain);
}

/*
 * Redraw a single line for speed. Ncurses is SLOW.
 */
void CursesUI::Screen::RedrawLine()
{
	register unsigned I;
	PackageTree::Iterator T;

	for (I = 0, T = cs; T != bs; I++, T--);

	ui.mv_print_line(cs, I);

	wrefresh(ui.wmain);
}

/*
 * Modify screen boundaries to set up for a scroll
 */
void CursesUI::Screen::ScrollUp()
{
	scrollok(ui.wmain, true);
	wscrl(ui.wmain, -1);
	scrollok(ui.wmain, false);

	_boundstart--; bs--;
	_boundend --; be--;
}

void CursesUI::Screen::ScrollDown()
{
	scrollok(ui.wmain, true);
	wscrl(ui.wmain, 1);
	scrollok(ui.wmain, false);

	_boundstart++; bs++;
	_boundend++; be++;
}


/*
 * Search text entry function with options
 */
void CursesUI::Screen::SearchEntry(char **retstr, unsigned long *options)
{
	int i = 0, j = 0;

	**retstr = 0;

	int dispstrlen = SearchPrompt(*options, *retstr);

	WINDOW * old_dwin = ui.wdesc;

	while (1)
	{
		int c = wgetch(ui.wstat);
		
		switch (c)
		{
		case 15: // Ctrl-O

			*(*retstr + j) = 0;

			SearchHelp();

			break;

		case 6: // Ctrl-F
//old		*options = *options ^ SEARCH_DESC;
			*options = *options ^ Pkg_ShortDesc;
			*(*retstr + j + 1) = 0;
			dispstrlen = SearchPrompt(*options, *retstr);
			break;

		case 9: // Ctrl-I
//old			*options = *options ^ cmpcase;
			*options = *options ^ Cmp_Case;
			*(*retstr + j + 1) = 0;
			dispstrlen = SearchPrompt(*options, *retstr);
			break;

		case 18: // Ctrl-R
//old			*options = *options ^ cmpregex;
			*options = *options ^ Cmp_Regex;
			*(*retstr + j + 1) = 0;
			dispstrlen = SearchPrompt(*options, *retstr);
			break;

		case '\n':
		case '\r':
			*(*retstr + j) = 0;

			curs_set(0);

			if (old_dwin != ui.wdesc)
				DescWindowHide();

			Redraw();

			return;

		case KEY_BACKSPACE:
			if (j == 0)
				break;

			if (i < j)
			{
				char *x = *retstr + i;
				char *y = (char *)malloc(j-i);

				memcpy(y, x, j-i);

				memcpy(x-1, y, j-i);

				free(y);

				i--;
			}
			else
			{
				*(*retstr + i) = 0;

				i--;
			}

			mvwdelch(ui.wstat, 0, i + dispstrlen);
			wmove(ui.wstat, 0, i + dispstrlen);

			j--;

			wrefresh(ui.wstat);

			break;

		case KEY_LEFT:
			if (j == 0)
				break;

			i--;

			wmove(ui.wstat, 0, i + dispstrlen);

			break;

		case KEY_RIGHT:
			if (j == (signed)(COLS - dispstrlen - 1))
				break;

			if (i == j)
				break;

			i++;

			wmove(ui.wstat, 0, i + dispstrlen);

			break;
			
		case 27:
			break;

		default:
			if (j == (signed)(COLS - dispstrlen - 1))
				break;

			if (i < j)
			{
				char *x = *retstr + i;
				char *y = (char *)malloc(j-i);
				
				memcpy(y, x, j-i);

				*x = c;

				memcpy(x+1, y, j-i);

				free(y);

				mvwinsch(ui.wstat, 0, i + dispstrlen, c);

				i++;
			}
			else
			{
				mvwaddch(ui.wstat, 0, i + dispstrlen, c);

				*(*retstr + i) = c;

				i++;
			}

			wmove(ui.wstat, 0, i + dispstrlen);

			j++;

			wrefresh(ui.wstat);

			break;
		}
	}
}

void CursesUI::Screen::SeekNextMatched()
{
	PackageTree::Item *moved_to;
	PackageTree::Iterator T;

//old	moved_to = Packages->LocateNext();
	moved_to = Packages->LocateNext(false);

	if (moved_to == NULL)
		return;

	int I, J;

	for (I = 0, T = Packages->Root(); T != moved_to; T++, I++);

	if (_cursor < I)
	{
		for (J = _cursor; J < I; J++)
		{
			if (I > _boundend)
			{
				++ _boundstart;
				++ _boundend;
			}
			++ _cursor;
		}
	}
	else if (_cursor > I)
	{
		for (J = _cursor; J > I; J--)
		{
			if (I < _boundstart)
			{
				-- _boundstart;
				-- _boundend;
			}
			-- _cursor;
		}
	}

	BoundsReset();
	Redraw();
}


/*
 * Turn the description windows on
 */
void CursesUI::Screen::DescWindowShow(void)
{
	descVisible = true;
	main_lines -= (desc_lines + dtop_lines);
	_boundend -= (desc_lines + dtop_lines);
	
	wresize(ui.wmain, main_lines, COLS);
	
	if (ui.wacqprogress)
	{
		ui.wdtop = newwin(dtop_lines, COLS, LINES - stat_lines - acqprogress_lines - desc_lines - dtop_lines, 0);
		ui.wdesc = newwin(desc_lines, COLS, LINES - stat_lines - acqprogress_lines - desc_lines, 0);
	}
	else
	{
		ui.wdtop = newwin(dtop_lines, COLS, LINES - stat_lines - desc_lines - dtop_lines, 0);
		ui.wdesc = newwin(desc_lines, COLS, LINES - stat_lines - desc_lines, 0);
	}

	keypad(ui.wdtop, true);
	keypad(ui.wdesc, true);

	wbkgd(ui.wdtop, _clrsys->regpair(EDEITY_COLOR_DTOP_FG, EDEITY_COLOR_DTOP_BG));
	wbkgd(ui.wdesc, _clrsys->regpair(EDEITY_COLOR_DESC_FG, EDEITY_COLOR_DESC_BG)|A_BOLD);

	scrollok(ui.wdesc, true);

	wrefresh(ui.wdtop);
	wrefresh(ui.wdesc);

	BoundsReset();
	UpdateDescription();
}

/*
 * Turn the description windows off
 */
void CursesUI::Screen::DescWindowHide(void)
{
	descVisible = false;
	main_lines += (desc_lines + dtop_lines);
	_boundend += (desc_lines + dtop_lines);

	delwin(ui.wdtop);
	delwin(ui.wdesc);

	ui.wdtop = NULL;
	ui.wdesc = NULL;

	wresize(ui.wmain, main_lines, COLS);

	BoundsReset();
}

void CursesUI::Screen::DescShuffle(int factorial)
{
	if (!descVisible)
		return;

	delwin(ui.wdtop);
	delwin(ui.wdesc);
	ui.wdtop = newwin(dtop_lines, COLS, LINES - stat_lines - desc_lines - dtop_lines + factorial, 0);
	ui.wdesc = newwin(desc_lines, COLS, LINES - stat_lines - desc_lines + factorial, 0);

	keypad(ui.wdtop, true);
	keypad(ui.wdesc, true);

	wbkgd(ui.wdtop, _clrsys->regpair(EDEITY_COLOR_DTOP_FG, EDEITY_COLOR_DTOP_BG));
	wbkgd(ui.wdesc, _clrsys->regpair(EDEITY_COLOR_DESC_FG, EDEITY_COLOR_DESC_BG)|A_BOLD);

	scrollok(ui.wdesc, true);

	wrefresh(ui.wdtop);
	wrefresh(ui.wdesc);

	BoundsReset();
	UpdateDescription();
}

/*
 * Open a help window for search options
 */
void CursesUI::Screen::SearchHelp()
{
	if (ui.wdesc == NULL)
		DescWindowShow();

	werase(ui.wdtop);
	mvwprintw(ui.wdtop, 0, 0, _(" Search options (all control characters) "));
	wrefresh(ui.wdtop);

	werase(ui.wdesc);
	mvwprintw(ui.wdesc, 0, 0,
		_("\n  ^F  Full-text Search of Package Descriptions  \n"
		"  ^I  Ignore Case  \n"
		"  ^R  Regular-expression Match  \n"));

	wrefresh(ui.wdesc);

	// Move the cursor back
	wrefresh(ui.wstat);
}

int CursesUI::Screen::SearchPrompt(unsigned long options, char *buf)
{
	string dispstr;

	curs_set(1);
	scrollok(ui.wstat, false);
	wattrset(ui.wstat, _clrsys->regpair(COLOR_WHITE, COLOR_BLUE));
	ui.mvwblankl(ui.wstat, 0, COLS);
	wmove(ui.wstat, 0, 0);
	wattrset(ui.wstat, _clrsys->regpair(COLOR_GREEN, COLOR_BLUE));

	dispstr = _(" locate ");

//old	if (options & cmpcase) 
	if (options & Cmp_Case) 
		dispstr += _("ignore-case ");
//old	if (options & SEARCH_DESC)
	if (options & Pkg_ShortDesc)
		dispstr += _("full-text ");
//old	if (options & cmpregex)
	if (options & Cmp_Regex)
		dispstr += _("regex ");

	dispstr +=  _("(^O for help): ");

	wprintw(ui.wstat, (char *)dispstr.c_str());

	wattrset(ui.wstat, _clrsys->regpair(COLOR_WHITE, COLOR_BLUE));

	wprintw(ui.wstat, buf);

	wclrtoeol(ui.wstat);
	wrefresh(ui.wstat);

	return dispstr.length();
}

void CursesUI::Screen::ScrollDescDown()
{
	if (ui.wdesc == NULL)
		return;

	if (cs->IsDivider())
		return;

	if (desc_bound_end == desc_width)
	{
		return;
	}

	desc_bound_end ++;
	desc_bound_start ++;

	scrollok(ui.wdesc, true);
	wscrl(ui.wdesc, 1);
	scrollok(ui.wdesc, false);
	mvwprintw(ui.wdesc, desc_lines-1, 0, (char *)(*curDescLines)[desc_bound_end - 1].c_str());

	wrefresh(ui.wdesc);
}

void CursesUI::Screen::ScrollDescUp()
{
	if (ui.wdesc == NULL)
		return;

	if (cs->IsDivider())
		return;

	if (desc_bound_start == 0)
		return;

	desc_bound_start --;
	desc_bound_end --;

	scrollok(ui.wdesc, true);
	wscrl(ui.wdesc, -1);
	scrollok(ui.wdesc, false);
	mvwprintw(ui.wdesc, 0, 0, (char *)(*curDescLines)[desc_bound_start].c_str());

	wrefresh(ui.wdesc);
}

void CursesUI::Screen::DividerPrev()
{
	PackageTree::Iterator T(cs);
	int I=_cursor, J=0;

	if (cs->IsDivider())
	{
		T--;
		I--;
	}

	while (!T.Invalid())
	{
		if (T->IsDivider())
			break;
		T--; I--;
	}

	if (T.Invalid())
		return;

	if (_cursor < I)
	{
		for (J = _cursor; J < I; J++)
		{
			if (I > _boundend)
			{
				++ _boundstart;
				++ _boundend;
			}
			++ _cursor;
		}
	}
	else if (_cursor > I)
	{
		for (J = _cursor; J > I; J--)
		{
			if (I < _boundstart)
			{
				-- _boundstart;
				-- _boundend;
			}
			-- _cursor;
		}
	}

	BoundsReset();
	Redraw();
}

void CursesUI::Screen::DividerNext()
{
	PackageTree::Iterator T(cs);
	int I=0, J=0;

	if (cs->IsDivider())
	{
		T++; I++;
	}

	while (!T.Invalid())
	{
		if (T->IsDivider())
			break;
		T++; I++;
	}

	if (T.Invalid())
		return;

	I = _cursor + I;

	if (_cursor < I)
	{
		for (J = _cursor; J < I; J++)
		{
			if (I > _boundend)
			{
				++ _boundstart;
				++ _boundend;
			}
			++ _cursor;
		}
	}
	else if (_cursor > I)
	{
		for (J = _cursor; J > I; J--)
		{
			if (I < _boundstart)
			{
				-- _boundstart;
				-- _boundend;
			}
			-- _cursor;
		}
	}

	BoundsReset();
	Redraw();
}

// Realign screen boundaries
//
void CursesUI::Screen::BoundsReset()
{
	PackageTree::Iterator T(*Packages);
	int I = 0;

/*	char buf[100];
	sprintf(buf, "boundstart = %d, boundend = %d, cursor = %d, list length = %d", _boundstart, _boundend, _cursor, Packages->Length());
	CaptUI->ShowDialog("pre-boundreset", buf);*/

	if (Packages->Length() <= CursesUI::main_lines)
	{
		_boundstart = 0;
		_boundend = Packages->Length() - 1;
	}
	else
	{
		while (_boundend >= Packages->Length())
		{
			_boundend --;
			_boundstart --;
		}

		if (_boundstart < 0)
			_boundstart = 0;

		if ((_boundend - _boundstart) < CursesUI::main_lines)
			_boundend = _boundstart + CursesUI::main_lines - 1;
	}

	if (_cursor > _boundend)
		_cursor = _boundend;

	while (I < _boundstart) 
	{
		T++;
		I++;
	}

	bs = T;

	while (I < _cursor)
	{
		T++;
		I++;
	}

	cs = T;

	while (I < _boundend)
	{
		T++;
		I++;
	}

	be = T;

	while (be->IsHidden())
		be--;

	/*sprintf(buf, "boundstart = %d, boundend = %d, cursor = %d, list length = %d", _boundstart, _boundend, _cursor, Packages->Length());
	CaptUI->ShowDialog("post-boundreset", buf);*/
	screenlength = CursesUI::main_lines;
}

void CursesUI::Screen::ActivateMenu(Menu m)
{
	MenuDrop d(m, *this);
	Redraw();
}

void CursesUI::Screen::HelpDialog()
{
	KeyHandler::List *p = NULL;
	string s;

	Dialog Help(*this);
	
	Help.geometry(25, 80, -1, -1);
	
	for (p = kdriver->ListStart(); p->key != 0; p++)
	{
		if (p->handlefunc == NULL && p->scrfunc == 0)
		{
			s = " ";
			s += p->KeyDesc;
			s += " ";
		}
		else
		{
			s = "  ";
			s += p->KeyName;
			for (int I = strlen(p->KeyName); I < 14; I++)
				s += " ";
			s += p->KeyDesc;
		}

		if (p->scrfunc != 0)
			Help.addline(s, p->scrfunc);
		else if (p->handlefunc != 0)
			Help.addline(s, p->handlefunc);
		else
			Help.addline(s);
	}

	Help.title(_("capt key binding list"));
	Help.execute();
	
	RefreshCurrent();
}

void CursesUI::Screen::SortSelectMethod()
{
	char buf[100];
	UniqueWindow *Popup = new UniqueWindow(22, 60, CenterY(21), CenterX(60));
	SortOrderList::SortIterator S;
	int I;

	Popup->ChangeTitle(_("Sort Editor"));

	while (1)
	{
		Popup->Erase();

		wattrset(Popup->body(), _clrsys->regpair(COLOR_MAGENTA, COLOR_BLUE)|A_BOLD);
		mvwprintw(Popup->body(), 1, 0, _(" Please select an option to change your sorting:"));
		wattrset(Popup->body(), _clrsys->regpair(COLOR_WHITE, COLOR_BLUE)|A_BOLD);
		mvwprintw(Popup->body(), 4, 0, _("   I  ==  Insert new before an installed method"));
		mvwprintw(Popup->body(), 3, 0, _("   A  ==  Append new at end of the list"));
		mvwprintw(Popup->body(), 5, 0, _("   D  ==  Delete installed sort method"));
		//mvwprintw(Popup->body(), 6, 0, "   M  ==  Modify installed sort method");
		wattrset(Popup->body(), _clrsys->regpair(COLOR_RED, COLOR_BLUE)|A_BOLD);
		mvwprintw(Popup->body(), 7, 0, _("   Q  ==  Quit back to main list"));

		wattrset(Popup->body(), _clrsys->regpair(COLOR_GREEN, COLOR_BLUE)|A_BOLD);

		sprintf(buf, _("  You currently have %d sorting methods installed."), Packages->Sorting->Length());
		mvwprintw(Popup->body(), 12, 0, buf);

		int list_start = 14, list_end = 21;

		wattrset(Popup->body(), _clrsys->regpair(COLOR_YELLOW, COLOR_BLUE)|A_BOLD);

		for (I = list_start, S = Packages->Sorting->ListBegin(); S.end() != true && I < list_end; S++, I++)
		{
			char *sorttext = SortToStr(S.method());

			mvwprintw(Popup->body(), I, 0, _("  ` Method %d (Packages by %s)"), I-list_start+1, sorttext);
			
			free(sorttext);
		}
		
		wattrset(Popup->body(), _clrsys->regpair(COLOR_WHITE, COLOR_BLUE)|A_BOLD);

		bool listchanged = false;

		switch (wgetch(Popup->body()))
		{
			case 'i':
			case 'I':
				listchanged = SortInsert(this, Popup, false);
				break;

			case 'a':
			case 'A':
				listchanged = SortInsert(this, Popup, true);
				break;

			case 'd':
			case 'D':
				listchanged = SortDelete(this, Popup);
				break;
				
			case 'm':
			case 'M':
				listchanged = SortModify(this, Popup);
				break;

			case 'q':
			case 'Q':
				delete Popup;
				Redraw();
				return;
		}

		if (listchanged == true)
		{
			Packages->Sorting->commit_to_config();

			delete Packages;
//old		Packages = new PackageTree(this);
			Packages = new PackageTree(this, PackageTree::AllPkgs, PackageTree::NormDep);
			BoundsReset();
			Redraw();
			Popup->Redraw();
			Popup->Refresh();
		}
	}
}

//old bool CursesUI::FilterSelect(char **dest, Filter::Target *targ)
void CursesUI::Screen::FilterSelect(char **dest, unsigned long *targ)
{
}

