//old #include <std.h>
#include "edeityui.h"

//old #include <stdio.h>
//old #include <stdlib.h>
#include <dlfcn.h>

#ifdef USE_DEBCONF
#include <debconf/backend.h>
#include <debconf/itf.h>
#endif

#include <func/messages.h>
#include <func/config.h>
#include "acquireui.h"
//old #include "pmi.h"

extern "C" {
extern EDeityUI *CreateInterface();
};

/*
 * TODO
 * Perhaps using a config file mechanism, load in as dynamic modules
 * different UI interfaces
 */
/* 
int EDeityUI::LoadUIModules()
{
	return 0;
}
*/
EDeityUI *EDeityUI::CreateInterface(const char *name, int &argc, char **&argv)
{
#ifdef NODSO
	return ::CreateInterface(argc, argv);
#else
	char buf[256];
	void *handle;
	void *mkfunc;
	EDeityUI *ui;

	snprintf(buf, sizeof buf, "%s.so", name);
	handle = dlopen(buf, RTLD_NOW);

	if (handle == NULL)
	{
		fprintf(stderr, "%s\n", dlerror());
		return NULL;
	}
	
	mkfunc = dlsym(handle, "CreateInterface");
	if (mkfunc == NULL)
	{
		fprintf(stderr, _("%s: Cannot find UI creator function.\n"), buf);
		return NULL;
	}

	ui = ((EDeityUI * (*)(int &, char **&))mkfunc) (argc, argv);
	if (ui == NULL)
	{
		fprintf(stderr, _("%s: Cannot create UI.\n"), buf);
		return NULL;
	}
	return ui;
#endif
}

int EDeityUI::Passthrough(int fd)
{
	// Talk to the fetcher directly here
	// Until it is time to return
	return 0;
}

bool EDeityUI::ProcessMsg(int fd)
{
	char buf[1024];
	MsgId type;
	unsigned long len = sizeof(buf);
	static vector<MsgDownloadItem> ItemList;
	static MsgDownloadTotal Total;
//old	static pkgMgrInt *pmi = NULL;

	int ret = MessageReceive(fd, type, buf, len);

	if (ret < 0) return false;
	/* Acquire is modal? 
	   FIXME: I'm only for curses. */
	if (_Conf.GetBool("edeity.acquire.embedded") == true)
	{
		switch (type)
		{
			case MsgId_Update_Total:
				ASSERT(len == sizeof(struct MsgDownloadTotal));
				memcpy(&Total, buf, len);
				UpdateAcquireProgress(&Total);
				break;

			case MsgId_Acquire_Start:
				ShowAcquireProgress();
				break;

			case MsgId_Acquire_Stop:
				HideAcquireProgress();
				break;

			default:
				break;
		}
		return true;
	}

	switch (type)
	{
		case MsgId_Info: 
		case MsgId_Error: 
			break;
		case MsgId_Acquire_Fatal:
		case MsgId_Acquire_Warning:
		case MsgId_Acquire_Info:
		{
			char txtmsg[len+1];
			strncpy(txtmsg, buf, len);
			txtmsg[len] = 0;
			AcqItf->Message(txtmsg);
			break;
		}		
		case MsgId_MediaChange: 
		{
			ASSERT(len == sizeof(struct MsgMediaChange));
			MsgMediaChange msg;
			memcpy(&msg, buf, len);
			AcqItf->MediaChange(&msg);
			break;
		} 
		case MsgId_Site_Hit: 
		{
			ASSERT(len == sizeof(struct MsgItemDesc));
			MsgItemDesc msg;
			memcpy(&msg, buf, len);
			AcqItf->IMSHit(&msg);
			break;
		}
		case MsgId_Update: 
		{
			ASSERT(len == sizeof(unsigned int));
			unsigned int msg;
			memcpy(&msg, buf, len);
			AcqItf->Update(&ItemList, &Total);
			ItemList.clear();
			break;
		}
		case MsgId_Update_Item: 
		{
			ASSERT(len == sizeof(struct MsgDownloadItem));
			ItemList.resize(ItemList.size() + 1);
			memcpy(&ItemList[ItemList.size() - 1], buf, len);
			break;
		}
		case MsgId_Update_Total: 
		{
			ASSERT(len == sizeof(struct MsgDownloadTotal));
			memcpy(&Total, buf, len);
			break;
		}
		case MsgId_Download_Start: 
		case MsgId_Action_Start: 
		{
			ASSERT(len == sizeof(struct MsgItemDesc));
			MsgItemDesc msg;
			memcpy(&msg, buf, len);
			AcqItf->Fetch(&msg);
			break;
		}
		case MsgId_Download_Complete: 
		case MsgId_Action_Complete: 
		{
			ASSERT(len == sizeof(struct MsgItemDesc));
			MsgItemDesc msg;
			memcpy(&msg, buf, len);
			AcqItf->Done(&msg);
			
#ifdef USE_DEBCONF
			if (AcqItf->acqState == AcquireUI::PackageRun)
			{
				printf(_("preparing to call debconf\n"));
				char *DestFile = strrchr(msg.URI, '/');
				if (DestFile != 0) DestFile++;
				DebConfPreconfigure(DestFile);
			}
#endif
			break;
		}
		case MsgId_Download_Failed: 
		{
			ASSERT(len == sizeof(struct MsgItemFailed));
			MsgItemFailed msg;
			memcpy(&msg, buf, len);
			AcqItf->Fail(&msg);
			break;
		}
		case MsgId_Acquire_Start: 
		{
			INFO(INFO_DEBUG, _("msg Acquire Start\n"));
			AcqItf->Startup();
			AcqItf->Start();
#ifdef USE_DEBCONF
			StartDebConf();
#endif
			break;
		}
		case MsgId_Acquire_Stop: 
		{
			INFO(INFO_DEBUG, _("msg Acquire Stop\n"));
			AcqItf->Stop();
			AcqItf->Shutdown();
			break;
		}
		case MsgId_Acquire_Shutdown:
		{
			INFO(INFO_DEBUG, _("msg Acquire Shutdown\n"));
			ASSERT(len == sizeof(bool));
			bool ReinitCache;
			memcpy(&ReinitCache, buf, len);
			signalfd = -1;

			/* Let us theorise, Watson! */
			if (AcqItf != NULL) {
				delete AcqItf;
				AcqItf = NULL;
			}

			/* Also, set the current screen.  SQUISHY BUG! */
			scrlst.SetCurrent(scrlst.Head());

			if (ReinitCache)
				Cache->Reinit();

			/* Finally, redraw the screens */
			CaptUI->DrawScreens();

			return false;
//old		break;
		}
		case MsgId_PkgMgr_Start:
		{
			INFO(INFO_DEBUG, _("msg Install Start\n"));
//old		bool ready = AcqItf->ReadyToInstall();
//old		MessageSend(signalfd, MsgId_PkgMgr_Ready, &ready, sizeof(bool));
//old		pmi = CaptUI->CreatePMI();
			break;
		}	
		case MsgId_PkgMgr_Stop:
		{
			INFO(INFO_DEBUG, _("msg Install Stop\n"));
#ifdef USE_DEBCONF
			StopDebConf();
#endif
//old		delete pmi; pmi = NULL;

			/* We send back a MsgId_PkgMgr_Stop so the blocking thread can catch it */
			/* Crucial ---Pat */
//old			MessageSend(signalfd, MsgId_PkgMgr_Stop, NULL, 0);

//old		return false;
			break;
		}
		case MsgId_PkgMgr_Warning:
			INFO(INFO_DEBUG, _("msg pkgMgr Warning\n"));
//old		pmi->warning(buf, len);
			break;
//old		case MsgId_PkgMgr_Output:
//old			INFO(INFO_DEBUG, "msg pkgMgr Output\n");
//old			pmi->data(buf, len);
//old		break;
		default:
			printf(_("warning: unknown message id == %d"), type);
			// NOTREACHED("Unknown message id");
			break;
	}	
	
	return true;
}

