/* $Id: debconfgui.cc,v 1.5 2001/02/25 17:01:24 tausq Exp $ */
#ifdef USE_DEBCONF
#include <debconf/questions.h>
#include <debconf/answers.h>
#include <debconf/util.h>
#include "debconfgui.h"

//old #include <string.h>

GtkUI::DebConfGUI::DebConfGUI()
{
	displayed = false;
}

GtkUI::DebConfGUI::~DebConfGUI()
{
	g_slist_free(answerwidgets);
	gtk_widget_destroy(window);
}

void GtkUI::DebConfGUI::Startup()
{
	answerwidgets = 0;

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(window), 600, 400);
	gtk_container_set_border_width(GTK_CONTAINER(window), 10);

	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), vbox);

	GtkWidget *frame = gtk_frame_new(0);
	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(frame), 3);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);

	GtkWidget *scrollwin = gtk_scrolled_window_new(0, 0);
	gtk_container_add(GTK_CONTAINER(frame), scrollwin);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	body = gtk_vbox_new(FALSE, 5);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollwin),
		body);

	GtkWidget *frmhelp = gtk_frame_new(0);
	gtk_frame_set_label(GTK_FRAME(frmhelp), _("Help"));
	gtk_box_pack_start(GTK_BOX(vbox), frmhelp, TRUE, TRUE, 0);

	GtkWidget *scrollwin2 = gtk_scrolled_window_new(0, 0);
	gtk_container_add(GTK_CONTAINER(frmhelp), scrollwin2);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin2),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	help = gtk_label_new(0);
	gtk_label_set_line_wrap(GTK_LABEL(help), TRUE);
	gtk_misc_set_alignment(GTK_MISC(help), 0.05, 0.05);

	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollwin2),
		help);


	GtkWidget *hbox = gtk_hbox_new(TRUE, 40);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 20);

	btnOK = gtk_button_new_with_label(_("OK"));
	gtk_box_pack_start(GTK_BOX(hbox), btnOK, FALSE, TRUE, 0);

	btnGoBack = gtk_button_new_with_label(_("Go Back"));
	gtk_box_pack_start(GTK_BOX(hbox), btnGoBack, FALSE, TRUE, 0);
	gtk_widget_set_sensitive(btnGoBack, FALSE);

	/*
	btnHelp = gtk_button_new_with_label("Toggle Help");
	gtk_box_pack_start(GTK_BOX(hbox), btnHelp, FALSE, TRUE, 0);
	*/

/*old	gtk_signal_connect(GTK_OBJECT(btnOK), "clicked",
		GTK_SIGNAL_FUNC(OnBtnOKClicked), this);
*/	g_signal_connect(G_OBJECT(btnOK), "clicked",
			 G_CALLBACK(OnBtnOKClicked), gpointer(this));
/*old	gtk_signal_connect(GTK_OBJECT(btnGoBack), "clicked",
		GTK_SIGNAL_FUNC(OnBtnGoBackClicked), this);
*/	g_signal_connect(G_OBJECT(btnGoBack), "clicked",
			 G_CALLBACK(OnBtnGoBackClicked), gpointer(this));
	/*
	g_signal_connect(G_OBJECT(btnHelp), "clicked",
			 G_CALLBACK(OnBtnHelpClicked), gpointer(this));
	*/

	gtk_widget_show_all(window);
}

void GtkUI::DebConfGUI::SetCapbBackup(bool backup)
{
	if (backup)
		gtk_widget_set_sensitive(btnGoBack, TRUE);
	else
		gtk_widget_set_sensitive(btnGoBack, FALSE);
}

void GtkUI::DebConfGUI::SetTitle(char *title)
{
	gtk_window_set_title(GTK_WINDOW(window), title);
}

void GtkUI::DebConfGUI::OnBtnOKClicked(GtkButton *btn, gpointer data)
{
	GtkUI::DebConfGUI *This = (GtkUI::DebConfGUI *)data;
	GSList *elt = This->answerwidgets;

	while (elt != 0)
	{
		GtkWidget *widget = (GtkWidget *)elt->data;
		Question *q = (Question *)*g_object_get_data(G_OBJECT(widget)/*gtk_object_get_data(GTK_OBJECT(widget)*/, "question");

		switch (q->type)
		{
			case Question::QT_BOOLEAN:
			{
				gboolean b = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
				if (b)
					This->answers.Add(new Answer(q->tag, "true"));
				else
					This->answers.Add(new Answer(q->tag, "false"));
				break;
			}
			case Question::QT_NOTE:
			{
				/* shouldn't happen */
				break;
			}
			case Question::QT_STRING: /* fallthrough */
			case Question::QT_PASSWORD: /* fallthrough */
			case Question::QT_TEXT:
			{
				char *txt = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
				This->answers.Add(new Answer(q->tag, txt));
				g_free(txt);
				break;
			}
			case Question::QT_SELECT:
			{
				GtkWidget *menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(widget));
				GtkWidget *item = gtk_menu_get_active(GTK_MENU(menu));
				char *value = (char *)g_object_get_data(G_OBJECT(item), "value")/*gtk_object_get_data(GTK_OBJECT(item), "value")*/;

				This->answers.Add(new Answer(q->tag, value));

				break;
			}
			case Question::QT_MULTISELECT:
			{
				break;
			}
		}
		
		elt = elt->next;
	}
	g_slist_free(This->answerwidgets);
	This->answerwidgets = 0;
	gtk_main_quit();
}

void GtkUI::DebConfGUI::OnBtnGoBackClicked(GtkButton *btn, gpointer data)
{
	GtkUI::DebConfGUI *This = (GtkUI::DebConfGUI *)data;
	This->goback = true;
	gtk_main_quit();
}

void GtkUI::DebConfGUI::OnBtnHelpClicked(GtkButton *btn, gpointer data)
{
	//GtkUI::DebConfGUI *This = (GtkUI::DebConfGUI *)data;
}

void GtkUI::DebConfGUI::ClearArea()
{
//old	GList *child = gtk_container_children(GTK_CONTAINER(body));
	GList *child = gtk_container_get_children(GTK_CONTAINER(body));
	while (child != 0)
	{
		GtkWidget *w = (GtkWidget *)child->data;
		gtk_container_remove(GTK_CONTAINER(body), w);
		//gtk_widget_destroy(w);
		child = child->next;
	}	
}

gboolean GtkUI::DebConfGUI::PaintHelp(GtkWidget *w, GdkEventCrossing *event, gboolean data)
{
	GtkUI::DebConfGUI *This = (GtkUI::DebConfGUI *)data;
	Question *q = (Question *)*g_object_get_data(G_OBJECT(w), "question")/*gtk_object_get_data(GTK_OBJECT(w), "question")*/;
	if (q != 0 && q->ext_desc != 0)
	{
		gtk_label_set_text(GTK_LABEL(This->help), q->ext_desc);
	}	
	return TRUE;
}

void GtkUI::DebConfGUI::PaintBoolean(Question *question)
{
	if (question == 0) return;

	GtkWidget *checkbox = gtk_check_button_new_with_label(question->desc);
	char *d = question->defaultval;
	if (d != 0)
		if (strcmp(d, "true") == 0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), TRUE);
		else
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), FALSE);


	gtk_box_pack_start(GTK_BOX(body), checkbox, FALSE, FALSE, 10);

//old	gtk_object_set_data(GTK_OBJECT(checkbox), "question", question);
	g_object_set_data(G_OBJECT(checkbox), "question", question);
/*old	gtk_signal_connect(GTK_OBJECT(checkbox), "enter_notify_event",
		GTK_SIGNAL_FUNC(PaintHelp), this);
*/	g_signal_connect(G_OBJECT(checkbox), "enter_notify_event",
			 G_CALLBACK(PaintHelp), this);
	answerwidgets = g_slist_append(answerwidgets, checkbox);
}

void GtkUI::DebConfGUI::PaintNote(Question *question)
{
	if (question == 0) return;
	GtkWidget *label = gtk_label_new(question->desc);

//old	gtk_object_set_data(GTK_OBJECT(label), "question", question);
	g_object_set_data(G_OBJECT(label), "question", question);
/*old	gtk_signal_connect(GTK_OBJECT(label), "enter_notify_event",
		GTK_SIGNAL_FUNC(PaintHelp), this);
*/	g_signal_connect(G_OBJECT(label), "enter_notify_event",
			 G_CALLBACK(PaintHelp), this);

	gtk_box_pack_start(GTK_BOX(body), label, FALSE, FALSE, 10);
}

void GtkUI::DebConfGUI::PaintString(Question *question)
{
	if (question == 0) return;

	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);

	GtkWidget *label = gtk_label_new(question->desc);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

	GtkWidget *text = gtk_text_new(0, 0);//it must be changed
	if (question->defaultval != 0)
		gtk_text_insert(GTK_TEXT(text), 0, 0, 0, question->defaultval, -1);
	gtk_box_pack_start(GTK_BOX(vbox), text, FALSE, FALSE, 0);

	gtk_box_pack_start(GTK_BOX(body), vbox, FALSE, FALSE, 10);

//old	gtk_object_set_data(GTK_OBJECT(text), "question", question);
	g_object_set_data(G_OBJECT(text), "question", question);
/*old	gtk_signal_connect(GTK_OBJECT(text), "enter_notify_event",
		GTK_SIGNAL_FUNC(PaintHelp), this);
*/	g_signal_connect(G_OBJECT(text), "enter_notify_event",
			 G_CALLBACK(PaintHelp), this);

	answerwidgets = g_slist_append(answerwidgets, text);
}

void GtkUI::DebConfGUI::PaintPassword(Question *question)
{
	if (question == 0) return;
	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);

	GtkWidget *label = gtk_label_new(question->desc);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 3);

	GtkWidget *entry = gtk_entry_new();
	if (question->defaultval != 0)
		gtk_entry_set_text(GTK_ENTRY(entry), question->defaultval);
	gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 3);

	gtk_box_pack_start(GTK_BOX(body), hbox, FALSE, FALSE, 10);

//old	gtk_object_set_data(GTK_OBJECT(entry), "question", question);
	g_object_set_data(G_OBJECT(entry), "question", question);
/*old	gtk_signal_connect(GTK_OBJECT(entry), "enter_notify_event",
		GTK_SIGNAL_FUNC(PaintHelp), this);
*/	g_signal_connect(G_OBJECT(entry), "enter_notify_event",
			 G_CALLBACK(PaintHelp), this);

	answerwidgets = g_slist_append(answerwidgets, entry);
}

void GtkUI::DebConfGUI::PaintText(Question *question)
{
	if (question == 0) return;
	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);

	GtkWidget *label = gtk_label_new(question->desc);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 3);

	GtkWidget *entry = gtk_entry_new();
	if (question->defaultval != 0)
		gtk_entry_set_text(GTK_ENTRY(entry), question->defaultval);
	gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 3);

	gtk_box_pack_start(GTK_BOX(body), hbox, FALSE, FALSE, 10);

//old	gtk_object_set_data(GTK_OBJECT(entry), "question", question);
	g_object_set_data(G_OBJECT(entry), "question", question);
/*old	gtk_signal_connect(GTK_OBJECT(entry), "enter_notify_event",
		GTK_SIGNAL_FUNC(PaintHelp), this);
*/	g_signal_connect(G_OBJECT(entry), "enter_notify_event",
			 G_CALLBACK(PaintHelp), this);

	answerwidgets = g_slist_append(answerwidgets, entry);
}

void GtkUI::DebConfGUI::PaintSelect(Question *question)
{
	if (question == 0) return;
	if (question->choices == 0) return;

//old	GtkWidget *mnuchoices = gtk_menu_new();
	GtkItemFactory itemfact;
	char *choice, *choices, buff[128];
	
	GtkItemFactoryEntry item = {buff, NULL,	NULL, 0, "<Item>" };

	choices = cppstrdup(question->choices);
	while ((choice = strsep(&choices, ",")) != 0)
	{
/*old		GtkWidget *item = gtk_menu_item_new_with_label(choice);
		gtk_menu_append(GTK_MENU(mnuchoices), item);
		gtk_widget_show(item);
		gtk_object_set_data(GTK_OBJECT(item), "value", choice);
		
*/		sprintf(item->path, "/%s", choise);
		gtk_item_factory_create_item(itemfact, item, NULL, 1);
		item = gtk_item_factory_get_widget_by_path(itemfact, item->path);
		g_object_set_data(G_OBJECT(item), "value", choice);
	}
	if (choices != 0)
	{
/*old		GtkWidget *item = gtk_menu_item_new_with_label(choices);
		gtk_menu_append(GTK_MENU(mnuchoices), item);
		gtk_widget_show(item);
		gtk_object_set_data(GTK_OBJECT(item), "value", choices);
*/		sprintf(item->path, "/%s", choises);
		gtk_item_factory_create_item(itemfact, item, NULL, 1);
		item = gtk_item_factory_get_widget_by_path(itemfact, item->path);
		g_object_set_data(G_OBJECT(item), "value", choices);
	}

	mnuchoises = gtk_item_factory_get_widget (itemfact, "<main>");
	GtkWidget *optmenu = gtk_option_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), mnuchoices);

	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);

	GtkWidget *label = gtk_label_new(question->desc);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 3);
	gtk_box_pack_start(GTK_BOX(hbox), optmenu, FALSE, FALSE, 3);

	gtk_box_pack_start(GTK_BOX(body), hbox, FALSE, FALSE, 10);

//old	gtk_object_set_data(GTK_OBJECT(optmenu), "question", question);
	g_object_set_data(G_OBJECT(optmenu), "question", question);
/*old	gtk_signal_connect(GTK_OBJECT(optmenu), "enter_notify_event",
		GTK_SIGNAL_FUNC(PaintHelp), this);
*/	g_signal_connect(G_OBJECT(optmenu), "enter_notify_event",
			 G_CALLBACK(PaintHelp), this);

	answerwidgets = g_slist_append(answerwidgets, optmenu);
}

void GtkUI::DebConfGUI::PaintMultiselect(Question *question)
{
	if (question == 0) return;
	/* Not yet implemented */
}


int GtkUI::DebConfGUI::AskQuestions()
{
	if (!displayed)
	{
		Startup();
		displayed = true;
	}
	ClearArea();
	Question *q;
	goback = false;

	g_slist_free(answerwidgets);
	answerwidgets = 0;
	answers.Clear();
	
	for (q = questions.Head(); q != 0; q = q->next)
	{
		switch (q->type)
		{
			case Question::QT_BOOLEAN: PaintBoolean(q); break;
			case Question::QT_NOTE: PaintNote(q); break;
			case Question::QT_STRING: PaintString(q); break;
			case Question::QT_PASSWORD: PaintPassword(q); break;
			case Question::QT_TEXT: PaintText(q); break;
			case Question::QT_SELECT: PaintSelect(q); break;
			case Question::QT_MULTISELECT: PaintMultiselect(q); break;
		}
	}
	if (questions.Head() != 0)
	{
		gtk_widget_show_all(window);
		gtk_main();
	}
	
	if (goback) return DEBCONF_GOBACK;

	return 0;
}

void GtkUI::DebConfGUI::ClientConnected(int fd)
{
}

void GtkUI::DebConfGUI::Shutdown()
{
	g_print(_("shutdown\n"));
	gtk_widget_hide(window);
	gtk_main_quit();
}
#endif
