/* $Id: screen.cc,v 1.23 2001/06/22 14:59:17 ltd Exp $ */
#include <std.h>
//old #include <stdio.h>
//old #include <gdk/gdkkeysyms.h>
#include <func/apt.h>
#include <func/config.h>
#include <func/dependency.h>
#include <func/pkgchecks.h>
#include <list/screens.h>
#include <parsers/strutl.h>
#include "about.h"
#include "screen.h"
#include "menu.h"
#include "pkgtreewidget.h"
#include "util.h"
#include "sortui.h"
#include "sortcol.h"
#include "srcedit.h"
#include "filteredit.h"
#include "pixmaps/installsm.xpm"
#include "pixmaps/removesm.xpm"
#include "pixmaps/purgesm.xpm"
#include "pixmaps/upgradesm.xpm"
#include "pixmaps/distupgradesm.xpm"
#include "pixmaps/locatesm.xpm"
#include "pixmaps/filtersm.xpm"
#include "pixmaps/completesm.xpm"
#include "pixmaps/cancelsm.xpm"

using namespace GtkUtil;

const char **install_xpm = const_cast < const char **>(installsm_xpm);
const char **remove_xpm = const_cast < const char **>(removesm_xpm);
const char **purge_xpm = const_cast < const char **>(purgesm_xpm);
const char **upgrade_xpm = const_cast < const char **>(upgradesm_xpm);
const char **distupgrade_xpm = const_cast < const char **>(distupgradesm_xpm);
const char **locate_xpm = const_cast < const char **>(locatesm_xpm);
const char **filter_xpm = const_cast < const char **>(filtersm_xpm);
const char **complete_xpm = const_cast < const char **>(completesm_xpm);
const char **cancel_xpm = const_cast < const char **>(cancelsm_xpm);

// titles
gchar *GtkUI::Screen::candidateVerListTitles[] = { _("Element"), _("Value") };
gchar *GtkUI::Screen::installedVerListTitles[] = { _("Element"), _("Value") };
gchar *GtkUI::Screen::availableVersTreeTitles[] =
	{ _("Package Version"), _("Dependencies & Provides") };

GtkUI::Screen::Screen(::Screen::ScrInit mode, ::Screen::ScrFunc func, GtkUI & _ui)
	: ::Screen(mode, func), ui(_ui), window(0), modal(false)
{
	InvalidateIterators();

	// TODO: read from config
	descwinshown = true;

	// The window
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	ui.CreateKeyInterface(window);
	gtk_window_set_title(GTK_WINDOW(window), PACKAGE " " VERSION);
	gtk_window_set_default_size(GTK_WINDOW(window), _Conf.GetInt("edeity.gtk.window.main.xsize"), _Conf.GetInt("edeity.gtk.window.main.ysize"));
	gtk_widget_realize(window);

//old	gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(delete_event), this);
//old	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(destroy), this);
	g_signal_connect(G_OBJECT(window), "delete_event", G_CALLBACK(delete_event), gpointer(this));
	g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(destroy), gpointer(this));

//old	gtk_signal_connect(GTK_OBJECT(window), "key_press_event", GTK_SIGNAL_FUNC(KeyPress), this);
//old	g_signal_connect(G_OBJECT(window), "key_press_event", G_CALLBACK(KeyPress), gpointer(this));

	mwVBox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), mwVBox);

	// Draw the menu
	GtkWidget *menu = DrawMenu();
	gtk_box_pack_start(GTK_BOX(mwVBox), menu, FALSE, FALSE, 0);

	// Draw the toolbar
	GtkWidget *toolbar = DrawToolbar();
	gtk_box_pack_start(GTK_BOX(mwVBox), toolbar, FALSE, FALSE, 0);

	GtkWidget *mainscr = DrawScreen();
	gtk_box_pack_start(GTK_BOX(mwVBox), mainscr, TRUE, TRUE, 0);

	// Draw the status bar
	status_bar = gtk_statusbar_new();
	gtk_box_pack_start(GTK_BOX(mwVBox), status_bar, FALSE, FALSE, 0);
	guint context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(status_bar), _("EDeity Status Bar"));
	gtk_statusbar_push(GTK_STATUSBAR(status_bar), context_id, _("Welcome to EDeity!"));
	
	// Show everything!
	gtk_widget_show_all(window);
	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER_ALWAYS);
}

GtkUI::Screen::~Screen()
{
	gtk_widget_hide(window);
	delete pkgtree;

	if (modal)
		gtk_main_quit();
}

void GtkUI::Screen::Show(bool ismodal)
{
	modal = ismodal;
}

void GtkUI::Screen::AddToolbarIcon(GtkWidget * toolbar,
				   GtkSignalFunc callback, gpointer callback_data,
				   const char *labeltxt, const char **xpm_data,
				   const char *tooltips)
{
	GtkWidget *icon = NULL;

	if (xpm_data != NULL)
	{
//old		icon = ImageFromXPM((char **) xpm_data);
		icon = ImageFromXPM((char **) xpm_data, NULL);
	}

	
	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), labeltxt, tooltips, NULL,
				icon, callback, callback_data);
}

void GtkUI::Screen::ShowDescription(bool flag)
{

}

GtkWidget *GtkUI::Screen::DrawScreen(bool ShowDesc)
{
	GtkWidget *vpane = gtk_vpaned_new();
	gtk_paned_set_position(GTK_PANED(vpane), _Conf.GetInt("edeity.gtk.window.main.ysize") - 200);
	
	GtkWidget *mypkgtree = DrawContent();

	gtk_paned_pack1(GTK_PANED(vpane), mypkgtree, FALSE, TRUE);
	GtkWidget *hpane = gtk_hpaned_new();
	gtk_paned_pack2(GTK_PANED(vpane), hpane, TRUE, TRUE);
	gtk_paned_set_position(GTK_PANED(hpane), 550);

	GtkWidget *scrollwin = gtk_scrolled_window_new(NULL, NULL);
	gtk_paned_pack1(GTK_PANED(hpane), scrollwin, FALSE, TRUE);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	GtkWidget *viewport = gtk_viewport_new(NULL, NULL);
	gtk_container_add(GTK_CONTAINER(scrollwin), viewport);

	description = gtk_label_new("");
	gtk_container_add(GTK_CONTAINER(viewport), description);
	gtk_label_set_justify(GTK_LABEL(description), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment(GTK_MISC(description), 0, 0);
	
/*old	GtkStyle *descstyle = gtk_style_new();
	gdk_font_unref(descstyle->font);
	if ((descstyle->font = gdk_font_load(_Conf.Get("edeity.gtk.font.description"))) == NULL) {
		fprintf(stderr, "Cannot load desc style font: %s, trying fixed..\n", _Conf.Get("edeity.gtk.font.description"));
		if ((descstyle->font = gdk_font_load("fixed")) == NULL) {
			fprintf(stderr, "Unable to load font fixed, exiting..\n");
			exit(0);
		}
	}

	gtk_widget_set_style(description, descstyle);
*/
	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
	gtk_paned_pack2(GTK_PANED(hpane), vbox, TRUE, TRUE);

	GtkWidget *toolbar = gtk_toolbar_new(); //GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_TEXT);
	gtk_box_pack_start(GTK_BOX(vbox), toolbar, FALSE, FALSE, 0);
//old	gtk_toolbar_set_space_size(GTK_TOOLBAR(toolbar), 15);
//old	gtk_toolbar_set_space_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_SPACE_LINE);
//old	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);

	gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), 
		GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Candidate"),
		_("Candidate version(s) that can be installed"), NULL, NULL, 
		NULL, NULL);

	gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), 
		GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Installed"),
		_("Installed version(s) currently on this system"), NULL, NULL, 
		NULL, NULL);

	gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), 
		GTK_TOOLBAR_CHILD_BUTTON, NULL, _("Available"),
		_("Versions available in the archive"), NULL, NULL, 
		NULL, NULL);


    	GtkTreeSelection *select;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	gchar *titles[] = { _("Attribute"), _("Value") };
	
//old	detailclist = gtk_clist_new_with_titles(2, titles);
	detailliststore = gtk_list_store_new(1, G_TYPE_STRING);
	detaillist = gtk_tree_view_new_with_model (GTK_TREE_MODEL(detailliststore));
	select = gtk_tree_view_get_selection (GTK_TREE_VIEW(detaillist));
	
//old	gtk_clist_set_column_width(GTK_CLIST(detailclist), 0, 60);
//old	gtk_clist_set_column_width(GTK_CLIST(detailclist), 1, 80);
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes (titles[0], renderer, "text", 0, NULL);
	gtk_tree_view_column_set_min_width (column, 60);
	gtk_tree_view_append_column (GTK_TREE_VIEW (detaillist), column);
	
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes (titles[1], renderer, "text", 1, NULL);
	gtk_tree_view_column_set_min_width (column, 80);
	gtk_tree_view_append_column (GTK_TREE_VIEW (detaillist), column);
	
	gtk_box_pack_start(GTK_BOX(vbox), detaillist, TRUE, TRUE, 0);
	
	UpdateDescription();

	return vpane;
}

GtkWidget *GtkUI::Screen::DrawMenu()
{

	GtkItemFactory *itemfac;
	GtkItemFactoryEntry *item;
	MenuItemCallbackData *cb;
	GtkAccelGroup *accel;
	GtkWidget *menubar;
	unsigned int i;

	MenuItemEntry menu_items[] = {
		MENU_SECTIONS(_("/_File"), "<control>F"),
		MENU_ITEM(_("/_File/Complete pending install and removals"), "c", DefaultKeys::complete_pending),
		MENU_UITEM(_("/_File/Edit APT Sources"), NULL, EditSources),
		MENU_ITEM(_("/_File/Update avail. package lists"), "u", DefaultKeys::update_lists),
		MENU_ITEM(_("/_File/Reload disk package cache"), "R", DefaultKeys::reread_cache),
		{ _("/_File/sep1"),		NULL,	0,	"<Separator>", 0, 0 },
		MENU_ITEM(_("/_File/Quit"), "q", DefaultKeys::quit),

		MENU_SECTIONS(_("/_Edit"), "<control>E"),
		MENU_ITEM(_("/_Edit/Locate package"), "//", DefaultKeys::search),
		MENU_UITEM(_("/_Edit/Next match"), "n", SeekNextMatched),
		MENU_UITEM(_("/_Edit/Prev match"), "p", SeekPrevMatched),
		MENU_UITEM(_("/_Edit/First match"), NULL, SeekFirstMatched),
		MENU_UITEM(_("/_Edit/Last match"), NULL, SeekLastMatched),
		MENU_ITEM(_("/_Edit/Reset search items"), "<control>R", DefaultKeys::reset_markers),

//old		MENU_SECTIONS(_("/_Toggle"), "<control>T"),
//old		MENU_ITEM(_("/_Toggles/Description window"), NULL, DefaultKeys::descwin_toggle), */
//old		MENU_ITEM(_("/_Toggle/Verbose tree view"), NULL, DefaultKeys::viewmode_toggle),

		MENU_SECTIONS(_("/_View"), "<control>V"),
		MENU_CHK(_("/_View/Reverse dependencie view"), NULL, DefaultKeys::depview_toggle),

		MENU_SECTIONS(_("/_Command"), "<control>C"),
//old		MENU_ITEM(_("/_Command/Hold package"), NULL, DefaultKeys::pkg_hold),
		MENU_ITEM(_("/_Command/Mark pkg for install or upgrade (*)"), "\\", DefaultKeys::pkg_install),
		MENU_ITEM(_("/_Command/Mark pkg for remove (*)"), NULL, DefaultKeys::pkg_remove),
		MENU_ITEM(_("/_Command/Mark pkg for purge (*)"), NULL, DefaultKeys::pkg_purge),
		MENU_ITEM(_("/_Command/Upgrade all possible"), "u", DefaultKeys::all_upgrade),
		MENU_ITEM(_("/_Command/Upgrade distribution"), "d", DefaultKeys::dist_upgrade),
		MENU_ITEM(_("/_Command/Expand package dependencies (*)"), NULL, DefaultKeys::depend_toggle),

		MENU_SECTIONS(_("/_Filter"), "<control>F"),
//old		MENU_ITEM(_("/_Filter/Add new filter"), "f", filter_add_single),
		MENU_ITEM(_("/_Filter/Add new filter"), "f", DefaultKeys::filter_add_single),
//old		MENU_ITEM(_("/_Filter/Remove active filters"), "r", filter_revoke_all),
		MENU_ITEM(_("/_Filter/Remove active filters"), "r", DefaultKeys::filter_revoke_all),
//old		MENU_ITEM(_("/_Filter/Display active filters"), "l", filter_display_all),
		MENU_ITEM(_("/_Filter/Display active filters"), "l", DefaultKeys::filter_display_all),
//old		MENU_ITEM(_("/_Filter/Invert filter polarity"), "v", filter_change_view),
		MENU_ITEM(_("/_Filter/Invert filter polarity"), "v", DefaultKeys::filter_change_view),

		MENU_SECTIONS(_("/_Sorting"), "<control>S"),
		MENU_UITEM(_("/_Sorting/Tree sorting method(s)"), "s", SortSelectMethod),
		MENU_UITEM(_("/_Sorting/Tree column sorting"), NULL, ColumnSelectMethod),

		{ _("/_Help"),		"<Control>H",	0,	"<LastBranch>", 0, 0 },
		MENU_ITEM(_("/_Help/About"), NULL, MnuAbout),
		MENU_UITEM(N_("/_Help/Key Bindings"), "?", HelpDialog)
	};


	accel = gtk_accel_group_new();
	itemfac = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel);

	GtkWidget *menuHandler = gtk_handle_box_new();

	for (i = 0; i < sizeof(menu_items) / sizeof(menu_items[0]); i++)
	{
		item = g_new0(GtkItemFactoryEntry, 1);
		item->path = menu_items[i].path;
//		item->path = menu_item_names[i];
		item->accelerator = menu_items[i].accelerator;
		item->callback_action = menu_items[i].callback_action;
		item->item_type = menu_items[i].item_type;
		if (menu_items[i].func == 0 && menu_items[i].scr_func == 0)
		{
			gtk_item_factory_create_item(itemfac, item, NULL, 2);
		}
		else
		{
			item->callback = (GtkItemFactoryCallback) MnuDispatch;
			cb = g_new0(MenuItemCallbackData, 1);
			cb->obj = this;
			cb->func = menu_items[i].func;
			cb->scr_func = menu_items[i].scr_func;
			gtk_item_factory_create_item(itemfac, item, (gpointer) cb, 2);

		}
	}

	gtk_window_add_accel_group(GTK_WINDOW(window), accel);

	menubar = gtk_item_factory_get_widget(itemfac, "<main>");

	gtk_container_add(GTK_CONTAINER(menuHandler), menubar);

	return menuHandler;
}

GtkWidget *GtkUI::Screen::DrawToolbar()
{
	GtkWidget *toolbarHandler = gtk_handle_box_new();

	GtkWidget *toolbar = gtk_toolbar_new(); //GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
	gtk_container_add(GTK_CONTAINER(toolbarHandler), toolbar);
	gtk_container_set_border_width(GTK_CONTAINER(toolbar), 3);
//old	gtk_toolbar_set_space_size(GTK_TOOLBAR(toolbar), 0);
	//gtk_toolbar_set_space_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_SPACE_LINE);

	GTK_WIDGET_SET_FLAGS (toolbar, GTK_CAN_FOCUS);
//old	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);
	if (func == PkgList) 
	{
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoComplete), this, _("Complete"),
			complete_xpm, _("Complete pending installs"));
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoInstall), this, _("Inst/Upg"), 
			install_xpm, _("Install/Upgrade selected packages"));
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoRemove), this, _("Remove"), 
			remove_xpm, _("Remove selected packages"));
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoPurge), this, _("Purge"),
			purge_xpm, _("Purge selected packages"));
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoUpgrade), this, _("Upgrade All"), 
			upgrade_xpm, _("Upgrade all  packages"));
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoDistUpgrade), this, 	
			_("UpDist"), distupgrade_xpm, _("Upgrade to new distribution"));
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoLocate), this, _("Locate"),
			locate_xpm, _("Locate pacakges"));
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoFilter), this, _("Filter"),
			filter_xpm, _("Filter the package list"));
	} else {
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoComplete2), this, _("Complete"),
			complete_xpm, _("Complete pending installs"));
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoCancel), this, _("Cancel"), 
			cancel_xpm, _("Cancel pending installs"));
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
	}

	return toolbarHandler;
}

GtkWidget *GtkUI::Screen::DrawContent()
{
	GtkWidget *scrollwin;
//old	GtkWidget *scroller;
//old	GtkWidget *hbox;
//old	GtkWidget *frame;

//old	hbox = gtk_hbox_new(FALSE, 0);

	// Create the scrollbar
/*old	content_adj = gtk_adjustment_new(0, 0, 100, 1, 10, 10);	// some default values //

	gtk_signal_connect(GTK_OBJECT(content_adj), "value_changed",
			   GTK_SIGNAL_FUNC(ScrollEvent), this);
*/
	scrollwin = gtk_scrolled_window_new (NULL, NULL);//GTK_ADJUSTMENT (content_adj));
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
//old	scroller = gtk_vscrollbar_new(GTK_ADJUSTMENT(content_adj));

//old	gtk_box_pack_end(GTK_BOX(hbox), scroller, FALSE, FALSE, 0);

	// Create the tree widget (put it inside a frame
//old	frame = gtk_frame_new(NULL);
//old	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);

	pkgtree = new GtkUI::PkgTreeWidget(*this);
//old	pkgtree->SetVAdjustment(GTK_ADJUSTMENT(content_adj));
	
//old	gtk_container_add(GTK_CONTAINER(frame), pkgtree->GetTree());

//old	gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 3);
	gtk_container_add (GTK_CONTAINER (scrollwin), pkgtree->GetTree());

//old	return hbox;
	return scrollwin;
}

void GtkUI::Screen::UpdateDescription()
{
	if (descwinshown == false) return;
	if (cs.Invalid()) return; 

	if (cs->IsDivider())
	{
		gtk_label_set_text(GTK_LABEL(description),
			Packages->GetHeaderDesc(cs).c_str());
	} 
	else if (cs->Version().end() == true)
	{
		gtk_label_set_text(GTK_LABEL(description), _("This package has no version"));
	}
	else 
	{
		GError *error = NULL;
		gsize bytes_read, bytes_written;
		
		gtk_label_set_text(GTK_LABEL(description),
				   g_locale_to_utf8 (cs->LongDesc(), -1, &bytes_read, &bytes_written, &error));
	}
}

/* static event handlers */
/*old
void GtkUI::Screen::ScrollEvent(GtkAdjustment * adj, gpointer data)
{

	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	screen->pkgtree->ScrollEvent(adj);
}
*/
/*old
gboolean GtkUI::Screen::KeyPress(GtkWidget * w, GdkEventKey * event, gpointer data)
{
	 // This lets us override the key capturing functions in func/keys.cc for keycodes	//
	 // that are defined differently than in the "standard" UI				//
	 
	GtkUI::Screen *screen = (GtkUI::Screen *) data;
	switch (event->keyval) {
	case GDK_Up: screen->ArrowUp(); break;
	case GDK_Down: screen->ArrowDown(); break;
	case GDK_Left: screen->ArrowLeft(); break;
	case GDK_Right: screen->ArrowRight(); break;
	case GDK_Page_Up: screen->PageUp(); break;
	case GDK_Page_Down: screen->PageDown(); break;
	case GDK_Home: screen->SeekStart(); break;
	case GDK_End: screen->SeekEnd(); break;
	case GDK_Tab: screen->pkgtree->DependToggle(); break;
	}
	return TRUE;
}
*/

gint GtkUI::Screen::delete_event(GtkWidget * widget, GdkEvent * event, gpointer data)
{
	/* Change TRUE to FALSE and the main window will be destroyed with
	   * a "delete_event". */

//old	return (FALSE);
	return (TRUE);
}

void GtkUI::Screen::destroy(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen *_this = static_cast<GtkUI::Screen *>(data);
	GtkUI &gui = _this->ui;

	gui.scrlst.remove((::Screen *) _this);
	
	if (gui.scrlst.length() == 0)
		gui.Shutdown();
}

void GtkUI::Screen::DoComplete(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	DefaultKeys::complete_pending(screen);
}

void GtkUI::Screen::DoComplete2(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	RestrictedKeys::complete(screen);
	
//	DefaultKeys::reread_cache (screen);
}

void GtkUI::Screen::DoCancel(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	RestrictedKeys::abort(screen);
}

void GtkUI::Screen::DoInstall(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
//old	GtkUI::PkgTreeWidget::MarkInstall(NULL, screen->pkgtree);
	GtkUI::PkgTreeWidget::MarkInstall(screen->pkgtree, 0, NULL);
}

void GtkUI::Screen::DoRemove(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
//old	GtkUI::PkgTreeWidget::MarkRemove(NULL, screen->pkgtree);
	GtkUI::PkgTreeWidget::MarkRemove(screen->pkgtree, 0, NULL);
}

void GtkUI::Screen::DoPurge(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
//old	GtkUI::PkgTreeWidget::MarkPurge(NULL, screen->pkgtree);
	GtkUI::PkgTreeWidget::MarkPurge(screen->pkgtree, 0, NULL);
}

void GtkUI::Screen::DoUpgrade(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;

	if (pkgAllUpgrade(*Cache) == false)
		capt_show_broken(*Cache);

	screen->pkgtree->Redraw();
}

void GtkUI::Screen::DoDistUpgrade(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	if (pkgDistUpgrade(*Cache) == false)
		capt_show_broken(*Cache);

	screen->pkgtree->Redraw();
}

void GtkUI::Screen::DoLocate(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	DefaultKeys::search(screen);
}

void GtkUI::Screen::DoFilter(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	DefaultKeys::filter_add_single(screen);
}

// Menu functions
void GtkUI::Screen::MnuAbout(::Screen *)
{
	About about;
}

void GtkUI::Screen::MnuDispatch(GtkWidget * widget, gpointer closure, guint type)
{
	MenuItemCallbackData *cb = (MenuItemCallbackData *) closure;
	if (cb) {
		if (cb->func) {
			(*(cb->func))(cb->obj);
		} else if (cb->scr_func) {
			((cb->obj)->*(cb->scr_func)) ();
		}
	}
}


void GtkUI::Screen::InvalidateIterators()
{
//old	_boundstart = _boundend = _cursor = _prelight = 0;
	_boundstart = _boundend = _cursor = 0;
	_prelight = (GtkTreePath *) 0;
	bs = (::PackageTree::Item *) 0;
	be = (::PackageTree::Item *) 0;
	cs = (::PackageTree::Item *) 0;
	pl = (::PackageTree::Item *) 0;
}

void GtkUI::Screen::BoundsReset()
{
	if (pkgtree == 0)
		return;

	pkgtree->ResetPath();
	
/*old
	int I = 0;
	::PackageTree::Iterator T(*Packages);
	screenlength = pkgtree->LinesInView();

	if (Packages->Length() <= screenlength)
	{
		_boundstart = 0;
		_boundend = Packages->Length() - 1;
	}
	else
	{
		_boundend = _boundstart + screenlength - 1;

		while (_boundend >= Packages->Length())
		{
			_boundend--;
			_boundstart--;
		}

		if (_boundstart < 0)
			_boundstart = 0;
	}

	if (_cursor > _boundend)
		_cursor = _boundend;

	if (_cursor < 0)
		_cursor = 0;

	_prelight = _cursor;

	while (I < _boundstart)
	{
		T++;
		I++;
	}

	bs = T;

	while (I < _cursor)
	{
		T++;
		I++;
	}

	cs = T;
	pl = T;

	while (I < _boundend)
	{
		T++;
		I++;
	}

	be = T;

	while (be->IsHidden())
		be--;
*/
}

void GtkUI::Screen::ScrollUp() 
{
/*old
	bs--;
	_boundstart--;
	be--;
	_boundend--;

	if (!pl.Invalid())
		GetTree().DoPrelight(_prelight);
*/		
}

void GtkUI::Screen::ScrollDown()
{
/*old
	bs++;
	_boundstart++;
	be++;
	_boundend++;

	if (!pl.Invalid())
		GetTree().DoPrelight(_prelight);
*/		
}

void GtkUI::Screen::RedrawRow(int row, ::PackageTree::Item *p)
{
//old	GetTree().Drawline(p, row, false);
}

void GtkUI::Screen::Redraw()
{
	GetTree().Redraw();
}

void GtkUI::Screen::RedrawTree()
{
	GetTree().Redraw();
}

bool GtkUI::Screen::IsDescWindowShown()
{
	return descwinshown;
}

void GtkUI::Screen::DescWindowShow()
{
	ShowDescription(true);
	descwinshown = true;
}

void GtkUI::Screen::DescWindowHide()
{
	ShowDescription(false);
	descwinshown = false;
}

void GtkUI::Screen::DrawNextScreen() {}
void GtkUI::Screen::DrawPrevScreen() {}

void GtkUI::Screen::SeekFirstMatched() 
{
    ::PackageTree::Item *T = Packages->LocateNext (TRUE);
    if (T) pkgtree->SeekToItem (T, NULL);
}

void GtkUI::Screen::SeekNextMatched() 
{
    ::PackageTree::Item *T = Packages->LocateNext (FALSE);
    if (T) pkgtree->SeekToItem (T, NULL);
}

void GtkUI::Screen::SeekPrevMatched() 
{
    ::PackageTree::Item *T = Packages->LocatePrev (FALSE);
    if (T) pkgtree->SeekToItem (T, NULL);
}

void GtkUI::Screen::SeekLastMatched() 
{
    ::PackageTree::Item *T = Packages->LocatePrev (TRUE);
    if (T) pkgtree->SeekToItem (T, NULL);
}

void GtkUI::Screen::DividerNext() {}
void GtkUI::Screen::DividerPrev() {}

GtkWidget *GtkUI::Screen::DrawSearchDialog()
{
//old	static gchar *advpkgoptmnulbls[] = { "package name", "candidate version", "install version", "short description", "long description", "maintainer", "install size" };
//old	static gchar *advopoptmnulbls[] = { "is equal to", "is not equal to", "is less than", "is greater than", "contains", "does not contain" };
	
	static GtkItemFactoryEntry advpkgoptmnulbls[] = {
	    { _("/package name"),		NULL,	NULL,	0,	"<Item>"	},
	    { _("/candidate version"),	NULL,	NULL,	0,	"<Item>"	},
	    { _("/install version"),	NULL,	NULL,	0,	"<Item>"	},
	    { _("/short description"),	NULL,	NULL,	0,	"<Item>"	},
	    { _("/long description"),	NULL,	NULL,	0,	"<Item>"	},
	    { _("/maintainer"),			NULL,	NULL,	0,	"<Item>"	},
//old	    { "/install size",		NULL,	NULL,	0,	"<Item>"	},
	    { _("/file name"),			NULL,	NULL,	0,	"<Item>"	},
	};
	static guint nadvpkgoptmnulbls = sizeof(advpkgoptmnulbls) / sizeof(advpkgoptmnulbls[0]);

	static GtkItemFactoryEntry advopoptmnulbls[] = {
	    { _("/is equal to"),		NULL,	NULL,	0,	"<Item>"	},
	    { _("/is not equal to"),	NULL,	NULL,	0,	"<Item>"	},
	    { _("/is less than"),		NULL,	NULL,	0,	"<Item>"	},
	    { _("/is graeter than"),	NULL,	NULL,	0,	"<Item>"	},
	    { _("/contains"),			NULL,	NULL,	0,	"<Item>"	},
	    { _("/does not contain"),	NULL,	NULL,	0,	"<Item>"	},
	};
	static guint nadvopoptmnulbls = sizeof(advopoptmnulbls) / sizeof(advopoptmnulbls[0]);

	gchar entrykey[] = "advanced_entry_ ";
	guint nentrykey = strlen (entrykey) - 1;
	gchar radiokey[] = "advanced_radio_ ";
	guint nradiokey = strlen (radiokey) - 1;
	gchar fieldkey[] = "advanced_field_ ";
	guint nfieldkey = strlen (fieldkey) - 1;
	gchar stroptkey[] = "advanced_stropt_ ";
	guint nstroptkey = strlen (stroptkey) - 1;
	
	int i;
//old	unsigned int j;
	GtkWidget *dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog), _("Search..."));
	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
	gtk_window_set_policy(GTK_WINDOW(dialog), TRUE, TRUE, FALSE);

	GtkWidget *dlgvbox = GTK_DIALOG(dialog)->vbox;
	
	GtkWidget *notebook = gtk_notebook_new();
	
	g_object_set_data(G_OBJECT(dialog), "notebook", gpointer(notebook));
	
	gtk_box_pack_start(GTK_BOX(dlgvbox), notebook, FALSE, TRUE, 0);

	// First page -- simple search
	GtkWidget *simplepage = gtk_vbox_new(FALSE, 18);
	gtk_container_add(GTK_CONTAINER(notebook), simplepage);
	gtk_container_set_border_width(GTK_CONTAINER(simplepage), 20);

	gtk_notebook_set_tab_label(GTK_NOTEBOOK(notebook), 
		gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), 0),
		gtk_label_new(_("Simple search")));

	gtk_box_pack_start(GTK_BOX(simplepage), gtk_label_new(_("Search for:")), 
		FALSE, FALSE, 0);
	GtkWidget *simpleentry = gtk_entry_new_with_max_length (255);
	g_signal_connect(G_OBJECT(simpleentry), "activate",
		G_CALLBACK(SearchRun), (gpointer)this);
//old	gtk_object_set_data(GTK_OBJECT(dialog), "simple_entry", simpleentry);
	g_object_set_data(G_OBJECT(dialog), "simple_entry", gpointer(simpleentry));
	gtk_box_pack_start(GTK_BOX(simplepage), simpleentry,
		FALSE, FALSE, 0);
	
	GtkWidget *simpleopthbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(simplepage), simpleopthbox, TRUE, TRUE, 0);

	GtkWidget *simple_optregex = gtk_check_button_new_with_label(_("Use regex"));
//old	gtk_object_set_data(GTK_OBJECT(dialog), "simple_optregex", simple_optregex);
	g_object_set_data(G_OBJECT(dialog), "simple_optregex", gpointer(simple_optregex));
	gtk_box_pack_start(GTK_BOX(simpleopthbox), simple_optregex, FALSE, FALSE, 0);
	GtkWidget *simple_optcase = gtk_check_button_new_with_label(_("Case insensitive"));
//old	gtk_object_set_data(GTK_OBJECT(dialog), "simple_optcase", simple_optcase);
	g_object_set_data(G_OBJECT(dialog), "simple_optcase", gpointer(simple_optcase));
	gtk_box_pack_start(GTK_BOX(simpleopthbox), simple_optcase, FALSE, FALSE, 0);
	GtkWidget *simple_optfull = gtk_check_button_new_with_label(_("Search descriptions"));
//old	gtk_object_set_data(GTK_OBJECT(dialog), "simple_optfull", simple_optfull);
	g_object_set_data(G_OBJECT(dialog), "simple_optfull", gpointer(simple_optfull));
	gtk_box_pack_start(GTK_BOX(simpleopthbox), simple_optfull, FALSE, FALSE, 0);

	
	// Second page -- Advanced search
	GtkWidget *advpage = gtk_table_new(6, 5, FALSE);
	gtk_container_add(GTK_CONTAINER(notebook), advpage);

	gtk_notebook_set_tab_label(GTK_NOTEBOOK(notebook), 
		gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), 1),
		gtk_label_new(_("Advanced search")));

	GtkWidget *optmenu;
	GtkWidget *optmenu_mnu;
	GtkItemFactory *itemfact;
	
	// create the field option menus
	for (i = 0; i < 6; i++) {
		if (i != 0) {
			GSList *radiogrp = NULL;
			GtkWidget *radioand = gtk_radio_button_new_with_label(NULL, _("and"));
			
			radiokey[nradiokey] = '0' + i;
			g_object_set_data(G_OBJECT(dialog), radiokey, gpointer(radioand));

//old			radiogrp = gtk_radio_button_group(GTK_RADIO_BUTTON(radioand));
			radiogrp = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radioand));
			
			GtkWidget *radioor = gtk_radio_button_new_with_label(radiogrp, _("or"));
//old			radiogrp = gtk_radio_button_group(GTK_RADIO_BUTTON(radioor));
			radiogrp = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radioor));

			gtk_table_attach(GTK_TABLE(advpage), radioand, 
				0, 1, i, i+1, 
				(GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0);
			gtk_table_attach(GTK_TABLE(advpage), radioor, 
				1, 2, i, i+1,
				(GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0);
		}

		optmenu = gtk_option_menu_new();
		
		fieldkey[nfieldkey] = '0' + i;
		g_object_set_data(G_OBJECT(dialog), fieldkey, gpointer(optmenu));
		
/*old		optmenu_mnu = gtk_menu_new();
		for (j = 0; j < sizeof(advpkgoptmnulbls) / sizeof(advpkgoptmnulbls[0]); j++) {
			GtkWidget *menuitem = gtk_menu_item_new_with_label(advpkgoptmnulbls[j]);
			gtk_menu_append(GTK_MENU(optmenu_mnu), menuitem);
			gtk_widget_show(menuitem);
		}
*/
		itemfact = gtk_item_factory_new (GTK_TYPE_MENU, "<main>", NULL);
		gtk_item_factory_create_items (itemfact, nadvpkgoptmnulbls, advpkgoptmnulbls, NULL);

		optmenu_mnu = gtk_item_factory_get_widget (itemfact, "<main>");
		
		gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), optmenu_mnu);
		gtk_option_menu_set_history(GTK_OPTION_MENU(optmenu), 0);
		gtk_table_attach(GTK_TABLE(advpage), optmenu, 
			2, 3, i, i+1, 
			(GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0);

		optmenu = gtk_option_menu_new();
		
		stroptkey[nstroptkey] = '0' + i;
		g_object_set_data(G_OBJECT(dialog), stroptkey, gpointer(optmenu));
		
/*old		optmenu_mnu = gtk_menu_new();
		for (j = 0; j < sizeof(advopoptmnulbls) / sizeof(advopoptmnulbls[0]); j++) {
			GtkWidget *menuitem = gtk_menu_item_new_with_label(advopoptmnulbls[j]);
			gtk_menu_append(GTK_MENU(optmenu_mnu), menuitem);
			gtk_widget_show(menuitem);
		}
*/
		itemfact = gtk_item_factory_new (GTK_TYPE_MENU, "<main>", NULL);
		gtk_item_factory_create_items (itemfact, nadvopoptmnulbls, advopoptmnulbls, NULL);

		optmenu_mnu = gtk_item_factory_get_widget (itemfact, "<main>");

		gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), optmenu_mnu);
		gtk_option_menu_set_history(GTK_OPTION_MENU(optmenu), 0);
		gtk_table_attach(GTK_TABLE(advpage), optmenu, 
			3, 4, i, i+1,
			(GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0);

		GtkWidget *entry = gtk_entry_new_with_max_length (255);
		g_signal_connect(G_OBJECT(entry), "activate",
			G_CALLBACK(SearchRun), (gpointer)this);
		
		entrykey[nentrykey] = '0' + i;
		g_object_set_data(G_OBJECT(dialog), entrykey, gpointer(entry));
		
		gtk_table_attach(GTK_TABLE(advpage), entry,
			4, 5, i, i+1, 
			(GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 
			(GtkAttachOptions)0, 0, 0);
	}

	
	GtkWidget *dlgactarea = GTK_DIALOG(dialog)->action_area;

	gtk_container_set_border_width(GTK_CONTAINER(dlgactarea), 10);
	GtkWidget *btnOK = gtk_button_new_with_label(_("OK"));
	gtk_box_pack_start(GTK_BOX(dlgactarea), btnOK, FALSE, TRUE, 0);
	GtkWidget *btnCancel = gtk_button_new_with_label(_("Cancel"));
	gtk_box_pack_start(GTK_BOX(dlgactarea), btnCancel, FALSE, TRUE, 0);

/*old	gtk_signal_connect(GTK_OBJECT(btnOK), "clicked",
		GTK_SIGNAL_FUNC(SearchRun), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(btnCancel), "clicked",
		GTK_SIGNAL_FUNC(SearchCancel), this);
*/	g_signal_connect(G_OBJECT(btnOK), "clicked",
		G_CALLBACK(SearchRun), (gpointer)this);
	g_signal_connect(G_OBJECT(btnCancel), "clicked",
		G_CALLBACK(SearchCancel), (gpointer)this);

	return dialog;
}

void GtkUI::Screen::SearchEntry(char **buf, unsigned long *options) 
{
//old	**buf = 0;
//old	*options = 0;

	GtkWidget *dialog = DrawSearchDialog();

	searchproceed = false;

	gtk_widget_show_all(dialog);
	gtk_main();
	if (searchproceed) {
	    if (gtk_notebook_get_current_page (GTK_NOTEBOOK (g_object_get_data(G_OBJECT(dialog), "notebook"))))
	    {
		GtkWidget *entry;
		GtkWidget *opt_mnu;
		GtkWidget *radio;
	    
		gchar entrykey[] = "advanced_entry_ ";
		guint nentrykey = strlen (entrykey) - 1;
		gchar radiokey[] = "advanced_radio_ ";
		guint nradiokey = strlen (radiokey) - 1;
		gchar fieldkey[] = "advanced_field_ ";
		guint nfieldkey = strlen (fieldkey) - 1;
		gchar stroptkey[] = "advanced_stropt_ ";
		guint nstroptkey = strlen (stroptkey) - 1;
		
		for (int i = 0; i < 6; i++)
		{
		    options[i] = 0;
		    
		    entrykey[nentrykey] = '0' + i;
		    entry = (GtkWidget *) g_object_get_data(G_OBJECT(dialog), entrykey);
		    buf[i] = strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
		    
		    stroptkey[nstroptkey] = '0' + i;
		    opt_mnu = (GtkWidget *) g_object_get_data(G_OBJECT(dialog), stroptkey);
		    switch (gtk_option_menu_get_history (GTK_OPTION_MENU(opt_mnu)))
		    {
			case 0:	options[i] |= Cmp_Eq /*SEARCH_EQUAL*/;
			break;
			case 1: options[i] |= Cmp_NEq /*SEARCH_NOTEQUAL*/;
			break;
			case 2: options[i] |= Cmp_Less /*SEARCH_LESS*/;
			break;
			case 3: options[i] |= Cmp_Great /*SEARCH_GREATER*/;
			break;
			case 4: options[i] |= Cmp_Cont /*SEARCH_CONT*/;
			break;
			case 5: options[i] |= Cmp_NCont /*SEARCH_NOTCONT*/;
			break;
		    }

		    fieldkey[nfieldkey] = '0' + i;
		    opt_mnu = (GtkWidget *) g_object_get_data(G_OBJECT(dialog), fieldkey);
		    switch (gtk_option_menu_get_history (GTK_OPTION_MENU(opt_mnu)))
		    {
			case 0: options[i] |= Pkg_Name /*SEARCH_NAME*/;
			break;
			case 1: options[i] |= Pkg_AvailVer /*SEARCH_CVERS*/;
			break;
			case 2: options[i] |= Pkg_InstlVer /*SEARCH_IVERS*/;
			break;
			case 3: options[i] |= Pkg_ShortDesc /*SEARCH_SDESC*/;
			break;
			case 4: options[i] |= Pkg_LongDesc /*SEARCH_LDESC*/;
			break;
			case 5: options[i] |= Pkg_Maintainer /*SEARCH_MTAIN*/;
			break;
			case 6: options[i] |= Pkg_FileName /*SEARCH_FNAME*/;
			break;
		    }
		    
		    if (i)
		    {
			radiokey[nradiokey] = '0' + i;
			radio = (GtkWidget *) g_object_get_data(G_OBJECT(dialog), radiokey);

			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)))
			    options[i] |= Cmp_And /*SEARCH_AND*/;
			else
			    options[i] |= Cmp_Or /*SEARCH_OR*/;
		    }
		    else
		    {
			options[i] |= Cmp_Or /*SEARCH_OR*/;
		    }
		    
		    options[i] |= Cmp_Case /*SEARCH_CASE*/;

		}	
	    }
	    else
	    {
/*old		GtkWidget *entry = 
			(GtkWidget *)gtk_object_get_data(GTK_OBJECT(dialog),
				"simple_entry");
		GtkWidget *optregex = 
			(GtkWidget *)gtk_object_get_data(GTK_OBJECT(dialog),
				"simple_optregex");
		GtkWidget *optcase = 
			(GtkWidget *)gtk_object_get_data(GTK_OBJECT(dialog),
				"simple_optcase");
		GtkWidget *optfull = 
			(GtkWidget *)gtk_object_get_data(GTK_OBJECT(dialog),
				"simple_optfull");
*/
		GtkWidget *entry = 
			(GtkWidget *)g_object_get_data(G_OBJECT(dialog),
				"simple_entry");
		GtkWidget *optregex = 
			(GtkWidget *)g_object_get_data(G_OBJECT(dialog),
				"simple_optregex");
		GtkWidget *optcase = 
			(GtkWidget *)g_object_get_data(G_OBJECT(dialog),
				"simple_optcase");
		GtkWidget *optfull = 
			(GtkWidget *)g_object_get_data(G_OBJECT(dialog),
				"simple_optfull");

		*buf = strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(optregex)))
			*options |= Cmp_Regex /*SEARCH_REGEX*/;
			
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(optcase)))
			*options |= Cmp_Case /*SEARCH_CASE*/;
			
		*options |= Cmp_Or /*SEARCH_OR*/;
		*options |= Cmp_Cont /*SEARCH_CONT*/;
			
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(optfull)))
		{
			options[1] = options[0];
			options[1] |= Pkg_LongDesc /*SEARCH_LDESC*/;
			buf[1] = strdup (buf[0]);
		}
		
		*options |= Pkg_Name /*SEARCH_NAME*/;
		
		g_print(_("Searching for %s\n"), *buf);
	    }
	} 
	else
	{
	    g_print(_("Search canceled\n"));
	}

	gtk_widget_destroy(dialog);
}

void GtkUI::Screen::FilterSelect(char **buf, unsigned long *target)
{
	FilterEditor ed(*this);
	if (ed.Run()) ed.GetFilter(buf, target);
}


void GtkUI::Screen::SearchRun(GtkButton *button, gpointer data)
{
	GtkUI::Screen *screen = (GtkUI::Screen *)data;
	screen->searchproceed = true;
	gtk_main_quit();
}

void GtkUI::Screen::SearchCancel(GtkButton *button, gpointer data)
{
	GtkUI::Screen *screen = (GtkUI::Screen *)data;
	screen->searchproceed = false;
	gtk_main_quit();
}

void GtkUI::Screen::SearchHelp() 
{
}

void GtkUI::Screen::SortSelectMethod() 
{
	SortUI sortui(*this);
	sortui.Show();
}

void GtkUI::Screen::ColumnSelectMethod() 
{
	SortCol sortcol(*this);
	sortcol.Show();
}

void GtkUI::Screen::EditSources()
{
	SrcEditor editor(*this);
	editor.Run();
}

void GtkUI::Screen::HelpDialog(void) 
{
	GtkWidget *dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(dialog), 640, 480);
	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER_ALWAYS);

//old	gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(DeleteEvent), NULL);
//old	gtk_signal_connect(GTK_OBJECT(dialog), "destroy", GTK_SIGNAL_FUNC(Destroy), NULL);
	g_signal_connect(G_OBJECT(dialog), "delete_event", G_CALLBACK(DeleteEvent), NULL);
	g_signal_connect(G_OBJECT(dialog), "destroy", G_CALLBACK(Destroy), NULL);

	GtkWidget *vbox = gtk_vbox_new(FALSE, 1);

//old	gtk_container_border_width(GTK_CONTAINER(vbox), 0);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 0);

	gtk_container_add(GTK_CONTAINER(dialog), vbox);

	gchar *titles[] = { _("Keypress Binding"), _("Designated outcome of keypress") };

	GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

//old	GtkWidget *keylist = gtk_ctree_new_with_titles(2, 0, titles);
	GtkTreeStore *keyliststore = gtk_tree_store_new (2, G_TYPE_STRING, G_TYPE_STRING);

//old	GtkCTreeNode *Parent = NULL;
	GtkTreeSelection *select;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeIter Parent, Child;

	for (KeyHandler::List *p = kdriver->ListStart(); p->key != 0; p++)
	{
		if (p->handlefunc == NULL && p->scrfunc == 0)
		{
//old			gchar *data[] = { stripcolorcodes(p->KeyDesc), "" };

/*old			Parent = gtk_ctree_insert_node(
				GTK_CTREE(keylist),
				NULL, NULL, data, 2,
				NULL, NULL, NULL, NULL,
				FALSE, TRUE);
*/			gtk_tree_store_append (keyliststore, &Parent, NULL);
			gtk_tree_store_set (keyliststore, &Parent,
					    0, stripcolorcodes(p->KeyDesc),
					    -1);
		}
		else
		{
//old			gchar *data[] = { p->KeyName, stripcolorcodes(p->KeyDesc) };

/*old			gtk_ctree_insert_node(
				GTK_CTREE(keylist),
				Parent, NULL, data, 2,
				NULL, NULL, NULL, NULL,
				TRUE, TRUE);
*/			gtk_tree_store_append (keyliststore, &Child, &Parent);
			gtk_tree_store_set (keyliststore, &Child,
					    0, p->KeyName,
					    1, stripcolorcodes(p->KeyDesc),
					    -1);
		}
	}

	GtkWidget *keylist = gtk_tree_view_new_with_model (GTK_TREE_MODEL(keyliststore));
	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (keylist));
	gtk_tree_selection_set_mode (select, GTK_SELECTION_NONE);
	
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes (titles[0], renderer, "text", 0, NULL);
	gtk_tree_view_column_set_min_width (column, 200);
	gtk_tree_view_append_column (GTK_TREE_VIEW (keylist), column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes (titles[1], renderer, "text", 1, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (keylist), column);

//old	gtk_clist_set_column_width(GTK_CLIST(&(GTK_CTREE(keylist)->clist)), 0, 200);

	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), keylist);

	gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);

	GtkWidget *button = gtk_button_new_with_label(_("Close"));
//old	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(WidgetDestroyEvent), dialog);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(WidgetDestroyEvent), gpointer(dialog));

	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, TRUE, 0);

	gtk_widget_show_all(dialog);

	gtk_main();
} 

void GtkUI::Screen::PageUp()
{
/*	::Screen::PageUp();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
	UpdateDescription();*/
}

void GtkUI::Screen::PageDown()
{
/*	::Screen::PageDown();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
	UpdateDescription();*/
}

void GtkUI::Screen::ArrowUp()
{
/*	::Screen::ArrowUp();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
	UpdateDescription();*/
}

void GtkUI::Screen::ArrowDown()
{
/*	::Screen::ArrowDown();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
	UpdateDescription();*/
}

void GtkUI::Screen::ArrowLeft()
{
/*	::Screen::ArrowLeft();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);*/
}

void GtkUI::Screen::ArrowRight()
{
/*	::Screen::ArrowRight();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);*/
}

void GtkUI::Screen::SeekStart()
{
/*	::Screen::SeekStart();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
	UpdateDescription();*/
}

void GtkUI::Screen::SeekEnd()
{
/*	::Screen::SeekEnd();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
	UpdateDescription();*/
}

void GtkUI::Screen::UpdateStatus(void)
{
	gchar buf[256];
	snprintf(buf, 256, "%lu package(s) listed, %lu broken, %lu mark to install/upgrade, %lu mark to delete/purge, %lu mark to keep",
		Packages->TotalLength(), (*Cache)->BrokenCount(), (*Cache)->InstCount(), (*Cache)->DelCount(), (*Cache)->KeepCount());


	guint context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(status_bar), _("EDeity Status Bar"));
	gtk_statusbar_pop(GTK_STATUSBAR(status_bar), context_id);
	gtk_statusbar_push(GTK_STATUSBAR(status_bar), context_id, buf);
}
