/* $Id: screen.h,v 1.9 2001/03/10 14:55:44 ltd Exp $ */
#ifndef _gtkui_screen_H
#define _gtkui_screen_H

#include "gtkui.h"
#include "pkgtreewidget.h"

#include <list/screens.h>
#include <list/pkgtree.h>

#include <gtk/gtk.h>
#include <glib.h>

class GtkUI::Screen : public ::Screen
{
	friend class GtkUI::PkgTreeWidget;
	friend class GtkUI::FilterEditor;
	friend class GtkUI::SortCol;
	friend class GtkUI::SortUI;
		
protected:
	typedef void (GtkUI::Screen::* scrfunc_t)(void);

	struct MenuItemCallbackData {
		GtkUI::Screen *obj;
		void (*func)(::Screen *);
		scrfunc_t scr_func;
	};
	
	struct MenuItemEntry {
		gchar *path;
		gchar *accelerator;
		guint callback_action;
		gchar *item_type;
		void (*func)(::Screen *);
		scrfunc_t scr_func;
	};

	static MenuItemEntry menu_items[];
	static gchar *candidateVerListTitles[];
	static gchar *installedVerListTitles[];
	static gchar *availableVersTreeTitles[];

	GtkUI &ui;

	GtkWidget *window;
	GtkWidget *mwVBox;

	GtkObject *content_adj;

	GtkWidget *description;
	GtkWidget *detaillist;	
	GtkListStore *detailliststore;	

	GtkWidget *status_bar;

	PkgTreeWidget *pkgtree;
	
	bool advanced;
	bool searchproceed; // I don't like this one bit...
	bool descwinshown;

	/* Static event handlers */
	static gint delete_event(GtkWidget *, GdkEvent *, gpointer);
	static void destroy(GtkWidget *, gpointer);
//old	static void ScrollEvent(GtkAdjustment *, gpointer);
//old	static gboolean KeyPress(GtkWidget *, GdkEventKey *, gpointer);

	static void DoInstall(GtkWidget *, gpointer);
	static void DoRemove(GtkWidget *, gpointer);
	static void DoPurge(GtkWidget *, gpointer);
	static void DoUpgrade(GtkWidget *, gpointer);
	static void DoDistUpgrade(GtkWidget *, gpointer);
	static void DoLocate(GtkWidget *, gpointer);
	static void DoFilter(GtkWidget *, gpointer);
	static void DoComplete(GtkWidget *, gpointer);
	static void DoComplete2(GtkWidget *, gpointer);
	static void DoCancel(GtkWidget *, gpointer);

	// Menu functions
	static void MnuAbout(::Screen *);
	static void MnuDispatch(GtkWidget *, gpointer, guint);

	// Search callbacks
	static void SearchRun(GtkButton *, gpointer);
	static void SearchCancel(GtkButton *, gpointer);

	// Draw parts of the window
	GtkWidget *DrawToolbar();
	GtkWidget *DrawMenu();
	GtkWidget *DrawContent();
	GtkWidget *DrawScreen(bool ShowDesc = true);
	GtkWidget *DrawSearchDialog();
	
	void AddToolbarIcon(GtkWidget *, GtkSignalFunc, gpointer,
		const char *, const char **, const char * = 0);

	bool modal;

public:
	Screen(::Screen::ScrInit, ::Screen::ScrFunc, GtkUI &);
	~Screen();

	/* gratuitous prelight support */
//old	long _prelight;
	GtkTreePath *_prelight;
	::PackageTree::Iterator pl;

	GtkUI::PkgTreeWidget & GetTree() { return *pkgtree; };
	void ShowDescription(bool);

	virtual void BoundsReset();
	virtual void InvalidateIterators();
	void Show(bool = false);

	void Redraw();
	void RedrawTree();
	void RedrawRow(int row, ::PackageTree::Item *p);
	void UpdateDescription();

	bool IsDescWindowShown();
	void DescWindowShow();
	void DescWindowHide();
	void DrawNextScreen();
	void DrawPrevScreen();

	void PageUp();
	void PageDown();
	void ScrollUp();
	void ScrollDown();
	void ArrowUp();
	void ArrowDown();
	void ArrowLeft();
	void ArrowRight();
	void SeekStart();
	void SeekEnd();
	
	void SeekNextMatched();
	void SeekPrevMatched();
	void SeekFirstMatched();
	void SeekLastMatched();
	void DividerNext();
	void DividerPrev();

	void UpdateStatus(void);
	
	void SearchEntry(char **, unsigned long *);
	void FilterSelect(char **, unsigned long *);
	int SearchPrompt(unsigned long, char *);
	void SearchHelp();

	void HelpDialog(void);

	void SortSelectMethod();
	void ColumnSelectMethod();
	void EditSources();
};


#endif
