/* $Id: screen.cc,v 1.3 2000/11/17 14:56:21 ltd Exp $ */
#include <std.h>
#include <cstdio>
#include <func/config.h>
#include <interface/edeityui.h>
#include <list/pkgtree.h>

Screen::Screen(ScrInit Op, ScrFunc Func) : func(Func), next(0), prev(0)
{
	switch (Func) 
	{
	case PkgList: 
//old		kdriver = new KeyHandler(*this, DefaultKeys::Bindings);
		kdriver = new KeyHandler(*this, KeyHandler::DefaultKeys);
		break;
	case InstallConf:
//old		kdriver = new KeyHandler(*this, RestrictedKeys::Bindings);
		kdriver = new KeyHandler(*this, KeyHandler::RestrictedKeys);
		break;
	default:
		NOTREACHED(_("Unknown screen function type"));
		kdriver = 0;
	}

	ID = 0;
	for (Screen *S=CaptUI->scrlst.Head(); S != NULL; S=S->next, ID++);

	if (strcmp(_Conf.Get("edeity.tree.verbosity"), "high") == 0)
		mode = Verbose;
	else
		mode = Terse;

	if (Op == PkgInit)
		if (Func == PkgList)
//old			Packages = new PackageTree(this);
			Packages = new PackageTree(this, PackageTree::AllPkgs, PackageTree::NormDep);
		else
			Packages = new PackageTree(this, PackageTree::ModPkgs, PackageTree::NormDep);
}

Screen::~Screen()
{
	delete kdriver;
	delete Packages;
}

void Screen::InvalidateIterators()
{
	_boundstart = _boundend = _cursor = 0;
	cs = (PackageTree::Item *) 0;
	bs = (PackageTree::Item *) 0;
	be = (PackageTree::Item *) 0;
}

void Screen::DrawNextScreen()
{
	if (next == NULL)
		return;

	CaptUI->scrlst.MoveNext();

	Redraw();
}

void Screen::DrawPrevScreen()
{
	if (prev == NULL)
		return;
	CaptUI->scrlst.MovePrev();

	Redraw();
}

/*
 * Key wrappers:  PageUp/PageDown button.
 */

void Screen::PageUp()
{
	register int i;

	if (Packages->Length() == 0)
		return;
	if (bs.prev() == 0)
		return;

	i = 0;
	while (i < (screenlength - 1))
	{
		if (bs.prev() == 0)
			break;

		_boundstart --; bs--;
		_boundend --; be--;

		i++;
	}

	if (bs.prev() == 0)
	{
		cs = bs;
		_cursor = _boundstart;
	}
	else
	{
		int Displacement = (_boundend - _boundstart) / 4;
		_cursor = _boundend - Displacement;
		cs = be - Displacement;
	}

	Redraw();
}

void Screen::PageDown()
{
	register int i;

	if (Packages->Length() == 0)
		return;
	if (be.next() == NULL)
		return;

	i = 0;
	while (i < (screenlength - 1))
	{
		if (be.next() == NULL)
			break;

		be++;
		_boundstart ++;
		bs++;
		_boundend ++;

		i++;
	}

	if (be.next() == 0)
	{
		cs = be;
		_cursor = _boundend;
	}
	else
	{
		int Displacement = (_boundend - _boundstart) / 4;
		_cursor = _boundstart + Displacement;
		cs = bs + Displacement;
	}

	Redraw();
}

/*
 * Key wrappers: Home/End button; Go to the start/end
 */

void Screen::SeekEnd()
{
	if (Packages->Length() == 0)
		return;

	while (be.next() != NULL)
	{
		bs++; _boundstart++;
		be++; _boundend++;
	}

	cs = be;
	_cursor = _boundend;     // Are at the end!

	Redraw();
}

void Screen::SeekStart()
{
	if (Packages->Length() == 0)
		return;

	while (bs.prev() != NULL)
	{
		bs--; _boundstart--;
		be--; _boundend--;
	}

	cs = bs;
	_cursor = _boundstart;             // Are at the start!

	Redraw();
}

/*
 * Key wrappers: Up/Down button;  change the current selected package (BLUE!)
 */

void Screen::ArrowUp()
{
	// Check to see if we're at the start of the list...
	if (cs.prev() == 0)
		return;

	int rows;
	PackageTree::Iterator ti;

	// Scroll down one if we're at the the boundaries
	if ((cs == bs) && (Packages->Length() > (screenlength - 1)))
		ScrollUp();

	// How far from the top was the old cursor
	for (rows = 0, ti = cs; ti != bs; rows++, ti--);

	// Move to the next package
	cs--; _cursor--;

	RedrawRow(rows - 1, cs);
	RedrawRow(rows, cs.next());

	Update(); // Update the package list display
	UpdateStatus();
	UpdateDescription();
}

void Screen::ArrowDown()
{
	// Check to see if we're at the end of the list...
	if (cs.next() == 0)
		return;

	int rows;
	PackageTree::Iterator ti;

	// Scroll down one if we're at the the boundaries
	if ((cs == be) && (Packages->Length() > (screenlength - 1)))
		ScrollDown();

	// How far from the top was the old cursor
	for (rows = 0, ti = cs; ti != bs; rows++, ti--);

	// Move to the next package
	cs++; _cursor++;

	RedrawRow(rows + 1, cs);
	RedrawRow(rows, cs.prev());

	Update(); // Update the package list display
	UpdateStatus();
	UpdateDescription();
}

void Screen::ArrowLeft()
{
	if (mode == Screen::Verbose)
	{
		if (scroll > 0)
			scroll -= 4;

		RedrawTree();
	}

	return;
}

void Screen::ArrowRight()
{
	if (mode == Screen::Verbose)
	{
		scroll += 4;
		RedrawTree();
	}

	return;
}

