/* $Id: column.cc,v 1.13 2003/01/28 08:32:58 ltd Exp $ 
 * This list is utilised when a PackageTree is constructed.
 */
 
//old #include <std.h>
//old #include <stdio.h>
#include <list/screens.h>
#include <func/config.h>
#include <func/apt.h>
#include <interface/edeityui.h>
#include "column.h"

ColOrderList *LastColumnCreated = NULL;

ColOrderList::ColOrderList()
{
	static List columndesc[] = {
		{ _("Name"),				_("Package"),				_("Name of the package") },
		{ _("Status"),				_("Is"),					_("Status of the package") },
		{ _("Specified status"),	_("Will"),					_("Specified status of the package") },
		{ _("Avaible Version"),		_("Avl. Vers."),			_("Version of the newest in avaible package(s)") },
		{ _("Installed Version"),	_("Inst. Vers."),			_("Version of the installed package") },
		{ _("Short Description"),	_("Short Description"),		_("Short description of the package") },
		{ _("Long Description"),	_("Long Description"),		_("Long description of the package") },
		{ _("Maintainer"),			_("Maintainer"),			_("Name and e-mail address of the package's maintainer") },
		{ _("MD5Hash"),				_("MD5 string"),			_("MD5 checksum string of the package") },
		{ _("Archive size"),		_("Arch. Size"),			_("Size of the archive file") },
		{ _("Installed size"),		_("Inst. Size"),			_("How much disk space the installed package will consume") },
		{ _("Filename"),			_("Filename"),				_("Name of the archive file") },
		{ _("Section"),				_("Section"),				_("Section of the package") },
		{ _("Priority"),			_("Priority"),				_("Priority of the package") },
		{ _("Has Depend"),			_("Has Dep."),				_("Has any dependency of package") },
		{ _("Has Reverse Depend"),	_("Has Rev. Depd."),		_("Has any reverse dependency of package") }
	};


	Head = Tail = NULL;
	nodecount = 0;
	clptr = columndesc;
	copy_from_config();
	LastColumnCreated = this;
}

ColOrderList::~ColOrderList()
{
	Flush();
}

void ColOrderList::Push(column_t myMethod)
{	
	nodecount += 1;

	if (Head == NULL && Tail == NULL)
	{
		Head = Tail = new Node;
		Tail->next = NULL;
	}
	else
	{
		Tail->next = new Node;
		Tail->next->next = NULL;
		Tail = Tail->next;
	}

	Tail->seqid = nodecount;
	Tail->thiscolumn = myMethod;
	
	reorderseqids();
}

void ColOrderList::Insert(ColIterator & S, column_t myMethod)
{	
	Node *x, *insertpos = S.Pos;

	if (insertpos == Head)
	{
		x = Head;
		Head = new Node;
		Head->next = x;

		x = Head;
	}
	else
	{
		for (x = Head; x->next != insertpos; x = x->next);

		Node *y = x->next;

		x->next = new Node;
		x->next->next = y;

		x = x->next;
	}
		
	x->seqid = 0;
	x->thiscolumn = myMethod;

	nodecount += 1;

	reorderseqids();
}

void ColOrderList::Pop(void)
{
	struct Node *p;

	nodecount -= 1;

	if (Head == Tail)
	{
		delete Head;
		Head = Tail = NULL;
	}
	else
	{
		for (p = Head; p->next != Tail; p = p->next);
		p->next = p->next->next;
		delete Tail;
		Tail = p;
	}

	reorderseqids();
}

void ColOrderList::remove(ColIterator & S)
{
	Node *p = S.Pos;

	Node *x = NULL;

	if (p == Head && p == Tail)
	{
		Head = Tail = NULL;
		delete p;
		nodecount = 0;
	}
	else if (p == Head)
	{
		x = Head;
		Head = Head->next;
		delete x;
		nodecount -= 1;
	}
	else if (p == Tail)
	{
		for (x = Head; x->next != Tail; x = x->next);
		x->next = x->next->next;
		delete Tail;
		Tail = x;
		nodecount -= 1;
	}
	else
	{
		for (x = Head; x->next != p; x = x->next);
		x->next = x->next->next;
		delete p;
		nodecount -= 1;
	}

	reorderseqids();
}

void ColOrderList::Flush(void)
{
	while (nodecount > 0)
		Pop();
}


/*
 * Useful for formatting dividers ... [Order of sort columns]
 */
int ColOrderList::FindIncrement(enum ColOrderList::column_t Type)
{
	for (ColIterator S = ListBegin(); S.end() == false; S++)
	{
		if (S.column() == Type)
			return S.seq()-1;
	}

	return -1;
}

/*
 * Useful for formatting dividers ... [Order of sort columns]
 */
bool ColOrderList::Exists(enum ColOrderList::column_t Type)
{
	for (ColIterator S = ListBegin(); S.end() == false; S++)
	{
		if (S.column() == Type)
			return true;
	}

	return false;
}

void ColOrderList::reorderseqids()
{
	int I;
	ColIterator S;

	for (I = 1, S = ListBegin(); S.end() == false; S++, I++)
		S.Pos->seqid = I;
}

void ColOrderList::commit_to_config()
{
	string s;
	char buf[50];

	for (ColIterator S = ListBegin(); S.end() == false; S++)
	{
		sprintf(buf, "%d", S.column());
		s += buf;
		s += "/";
	}

	if (nodecount > 0)
		s.resize(s.size()-1);
	else
		s = "NULL";

	_Conf.Set("edeity.columns", (char *)s.c_str());
}

void ColOrderList::copy_from_config()
{
	Flush();

	char *tmpbuf = strdup(_Conf.Get("edeity.columns"));

	if (strcmp(tmpbuf, "NULL") != 0)
		for (char *p = strtok(tmpbuf, "/"); p != NULL; p = strtok(NULL, "/"))
			Push((ColOrderList::column_t)atoi(p));

	free(tmpbuf);
}

ColOrderList::ColIterator ColOrderList::ListFind(column_t ThisMethod)
{
	ColIterator S(ListBegin());

	for (; S.end() == false; S++)
		if (S.column() == ThisMethod)
			return S;

	return S;
}
