/* $Id: column.h,v 1.9 2003/01/28 14:56:21 ltd Exp $ */
#ifndef _column_h
#define _column_h

#include <func/apt.h>

class ColOrderList
{
	public:
		class ColIterator;
		friend class ColIterator;

		struct Node;

		struct List	{
			char *Name;
			char *Title;
			char *Desc;
		};
		
		enum column_t { 
			PkgName, 
			Is,
			Will,
			AvlVers,
			InsVers,
			ShortDesc,
			LongDesc,
			Maintainer, 
			MD5Hash, 
			Size, 
			Instsize,
			Filename,
			Section,
			Priority,
			Depd,
			RevDepd,
			ColumnNum};

		// length of the list
		inline int Length() { return nodecount; };
		inline int TotalLength() { return ColumnNum; };
		
		// name of the columns
		inline char * Name (column_t colnum) { return clptr[colnum].Name; }
		inline char * Title (column_t colnum) { return clptr[colnum].Title; }
		inline char * Desc (column_t colnum) { return clptr[colnum].Desc; }
		
		// List functions
		void Push(column_t);
		void Insert(ColIterator &, column_t);
		void Pop();
		void remove(ColIterator &);
		void Flush();

		// Inquisitory functions
		int FindIncrement(enum column_t);
		bool Exists(enum column_t);

		void reorderseqids();
		void commit_to_config();
		void copy_from_config();

		// Iterator thingies
		ColIterator ListBegin();
		ColIterator ListEnd();
		ColIterator ListFind(column_t);

		// con/destructers
		ColOrderList();
		~ColOrderList();

	protected:
		unsigned int nodecount;
		
		List *clptr;
		
		Node *Head;
		Node *Tail;
};

struct ColOrderList::Node
{
	unsigned int seqid;
	enum ColOrderList::column_t thiscolumn;
	bool orderable;
	
	struct ColOrderList::Node *next;
};

class ColOrderList::ColIterator
{
	public:
		friend class ColOrderList;
	
		inline void operator =(ColIterator B) { Pos = B.Pos; };
		void operator ++(int) { Pos = Pos->next; };
		inline void operator ++() {operator ++(0);};

		inline bool operator ==(ColIterator &B) { return Pos == B.Pos; };
		inline bool operator !=(ColIterator &B) { return Pos != B.Pos; };
		inline operator Node * () { return Pos; };

		inline bool end() { return Pos == 0; };

		inline enum column_t column() { return Pos->thiscolumn; };
		inline unsigned int seq() { return Pos->seqid; };
		inline bool orderable() { return ((Pos->thiscolumn == PkgName) || (Pos->thiscolumn == Is) || (Pos->thiscolumn == Will)) ? false : true; };

		inline ColIterator(ColOrderList &Owner) { Pos = Owner.Head; };
		inline ColIterator(Node *NewPos) { Pos = NewPos; };
		inline ColIterator() {};

	protected:
		Node *Pos;
};

inline ColOrderList::ColIterator ColOrderList::ListBegin() { return ColIterator(Head); }
inline ColOrderList::ColIterator ColOrderList::ListEnd() { return ColIterator(Tail); }

extern ColOrderList *LastColumnCreated;

#endif

