/* $Id: pkgtree.cc,v 1.19 2001/06/22 14:59:17 ltd Exp $ */
#include <std.h>
#include <cstdio>
//old #include <math.h>
#include <parsers/strutl.h>
#include <func/config.h>
#include <func/apt.h>
#include <func/tree.h>
#include <list/filters.h>
#include <list/screens.h>
#include <list/sections.h>
#include <interface/edeityui.h>
#include <interface/format_strings.h>

// Begin "PackageTree" class functions

//old PackageTree::PackageTree(Screen *_Owner, ListType Init = AllPkgs) : Owner(_Owner)
PackageTree::PackageTree(Screen *_Owner, ListType List, DepViewType Dep) : Owner(_Owner)
{
	Sorting = new SortOrderList;
	Filters = new FilterList(this);
	Columns = new ColOrderList;

	root = 0;

	PackageCount = 0;

//old	Type = Init;
	Type = List;
	DependType = Dep;

	if (Type == AllPkgs)
		inject_all_packages();
	else if (Type == ModPkgs)
		inject_changed_packages();
	
	RecalcLength();
}

PackageTree::~PackageTree()
{
	FreeTree(root);
}

void PackageTree::reinitialize()
{
	FreeTree(root);

	root = 0;
	PackageCount = 0;

	if (Type == AllPkgs)
		inject_all_packages();
	else if (Type == ModPkgs)
		inject_changed_packages();

	RecalcLength();
}

void PackageTree::apply_filters()
{
	reinitialize();
}

void PackageTree::sort_packages(SortOrderList::SortIterator *S, pkgCache::Package ** SortedPkgs, long base, long end)
{
	int count;
	SortOrderList::SortIterator Q = *S; Q++;
		
		SortOrderList::Comparer *grouper = S->group();
		for (count = 0; base + count < end; count++)
			if (grouper(&SortedPkgs[base + count], &SortedPkgs[base + count + 1]) != 0) break;
			
		if (count) 
		{
			qsort(&SortedPkgs[base], count + 1, sizeof(*SortedPkgs), Q.compare());
			if ((int) S->seq() < Sorting->DividerCount() - 1)
				sort_packages(&Q, SortedPkgs, base, base + count);
		}
		 		
		if (base + count < end) sort_packages(S, SortedPkgs, base + count + 1, end);
}

unsigned PackageTree::collect_packages(pkgCache::Package** SortedPkgs)
{
	pkgCache::PkgIterator I;
	register unsigned J = 0;

	if (Filters->Length() > 0)
	{
		for (I = (*Cache)->PkgBegin(); I.end() != true; I++)
		if (I.VersionList().end() == false)
			if (Filters->IsPkgFiltered(I) == false)
				SortedPkgs[J++] = I;
	}
	else
	{
		for (I = (*Cache)->PkgBegin(); I.end() != true; I++)
		if (I.VersionList().end() == false)
			SortedPkgs[J++] = I;
	}

	SortedPkgs[J] = 0;

	return J;
}

void PackageTree::inject_all_packages()
{
	INFO(INFO_DEBUG, "PackageTree::inject_all_packages()\n");

//old	pkgCache::PkgIterator I;
	pkgCache::VerIterator V;

	Item *P = NULL;

	pkgCache::Package ** SortedPkgs = new pkgCache::Package *[(*Cache)->Head().PackageCount + 1];
	Item **ParentList = new Item *[Sorting->DividerCount() + 1];
	memset(ParentList, 0, sizeof(Item *) * Sorting->DividerCount() + 1);
	Item *ThisParent = root;
	Item *LastItem = NULL;
	
	char buff[2];
//old	char *section_ptr = "/dev/null";
//old	char *priority_ptr = "/dev/null";
//old	const char *lastpkgname = "";
//old	long lastpkgsize = -1;
//old	char *new_section;
	int State = -100;
//old	const char *PA, *PB;
//old	char *a=0, *b=0, *p=0;

//old	register int J = 0;
	int ThisIncrement = 0;
	int H, ExpandedLevel = _Conf.GetInt("edeity.tree.headers.expansion");
	bool HaveParent = false;

	unsigned J = TotalPackageCount = collect_packages (SortedPkgs);

	if (Sorting->Length() > 0)
	{
	    SortOrderList::SortIterator S = Sorting->ListBegin();
	    
	    qsort(SortedPkgs, J, sizeof(*SortedPkgs), S.compare());
		
	    if (Sorting->Length() > 1) sort_packages(&S, SortedPkgs, 0, J - 1);
	}

	if (Sorting->Length() == 1) Sorting->Push(SortOrderList::None, false);
	
	bool UseHeaders = _Conf.GetBool("edeity.tree.headers.show");
	
	for (register unsigned int K = 0; K < J; K++)
	{
		pkgCache::PkgIterator curPkg(*Cache, SortedPkgs[K]);
 		V = curPkg.VersionList();

//old		if (V.end() == true)
//old			continue;

		if (UseHeaders == true)
		{
			H = 0;
			HaveParent = false;
			for (SortOrderList::SortIterator S = Sorting->ListBegin(); S.end() == false; S++, H++)
			{
				if (K) if (S.group()(&SortedPkgs[K - 1], &SortedPkgs[K]) == 0 && HaveParent == false) continue;
				
					ThisIncrement = Sorting->FindIncrement(S.method());

					if (ThisIncrement == 0 || H == 0)	
						P = add(&root, NULL, NULL, NULL);
					else
						P = add(&ParentList[H - 1]->child, ParentList[H - 1], NULL, NULL);
						
					P->TargetPkg = curPkg;
					
					switch (S.method())
					{
					case SortOrderList::Section:
						P->Type = PackageTree::Item::Div_Sect;
						break;
						
					case SortOrderList::Status:
						P->Type = PackageTree::Item::Div_State;
						break;
						
					case SortOrderList::SpecStat:
						P->Type = PackageTree::Item::Div_sState;
						break;
						
					case SortOrderList::Priority:
						P->Type = PackageTree::Item::Div_Pri;
						break;
						
					case SortOrderList::Dep:
						P->Type = PackageTree::Item::Div_Dep;
						break;
						
					case SortOrderList::RevDep:
						P->Type = PackageTree::Item::Div_rDep;
						break;
						
					case SortOrderList::Suite:
/*						a = strdup(curPkg.Section() ? curPkg.Section() : "");
						if ((p = strchr(a, '/'))) *p=0;

						if (strcasecmp(a, "non-us") != 0 &&
						    strcasecmp(a, "non-free") != 0 &&
						    strcasecmp(a, "contrib") != 0)
						{
							free(a);
							a = strdup("main");
						}
						
						P->Extra = a;
*/						P->Type = PackageTree::Item::Div_Suite;
						break;
						
					case SortOrderList::PkgName:
						*buff = *(curPkg.Name()); *(buff + 1) = 0;
						P->Extra = strdup(buff);
						
						P->Type = PackageTree::Item::Div_Custom;
						break;
						
					case SortOrderList::Filename:
						
						*buff = *(Cache->GetPkgRecs().Lookup(V.FileList()).ShortDesc().c_str()); 
						*(buff + 1) = 0;
						
						P->Extra = strdup(buff);
						
						P->Type = PackageTree::Item::Div_fName;
						break;
						
					case SortOrderList::Instsize:
						P->Type = PackageTree::Item::Div_iSize;
						break;
						
					case SortOrderList::Size:
						P->Type = PackageTree::Item::Div_Size;
						break;
						
					default: ASSERT(0); break;
					}
				
					if (H < ExpandedLevel)
						P->State = PackageTree::Item::Expanded;
					P->increment = ThisIncrement;
					
					ThisParent = ParentList[H] = P;
					HaveParent |= true;
					LastItem = NULL;
/*				switch (S.method())
				{
				case SortOrderList::Section:
					new_section = (char *) curPkg.Section();
					INFO(INFO_DEBUG, "section = %s, new_section = %s\n", section_ptr, new_section);

					if (new_section == NULL)
						new_section = "";

					for (a = section_ptr + strlen(section_ptr); a >= section_ptr; a--)
						if (*a == '/')
						{
							a++;
							break;
						}
	
					if (*a == 0)
						a = section_ptr;

					for (b = new_section + strlen(new_section); b >= new_section; b--)
						if (*b == '/')
						{
							b++;
							break;
						}
	
					if (*b == 0)
						b = new_section;

					if (strcmp(a, b) != 0 || HaveParent)
					{
						ThisIncrement = Sorting->FindIncrement(SortOrderList::Section);

						if (ThisIncrement == 0 || H == 0)
//old							P = add(&root);
							P = add(&root, NULL, NULL, NULL);
						else
//old							P = add(&ParentList[H - 1]->child, ParentList[H - 1]);
							P = add(&ParentList[H - 1]->child, ParentList[H - 1], NULL, NULL);

						P->TargetPkg = curPkg;
						P->Type = PackageTree::Item::Div_Sect;
						if (H < ExpandedLevel)
							P->State = PackageTree::Item::Expanded;
						P->increment = ThisIncrement;

						ThisParent = ParentList[H] = P;
						HaveParent |= true;
						LastItem = NULL;
					}
					break;
	
				case SortOrderList::Status:
					if ((*Cache)[curPkg].Status != State || HaveParent)
					{
						ThisIncrement = Sorting->FindIncrement(SortOrderList::Status);
	
						if (ThisIncrement == 0 || H == 0)
//old							P = add(&root);
							P = add(&root, NULL, NULL, NULL);
						else
//old							P = add(&ParentList[H - 1]->child, ParentList[H - 1]);
							P = add(&ParentList[H - 1]->child, ParentList[H - 1], NULL, NULL);

						P->TargetPkg = curPkg;
						P->Type = PackageTree::Item::Div_State;
						if (H < ExpandedLevel)
							P->State = PackageTree::Item::Expanded;
						P->increment = ThisIncrement;

						ThisParent = ParentList[H] = P;
						HaveParent |= true;
						LastItem = NULL;
					}
					break;
	
				case SortOrderList::Priority:
					PA = V.PriorityType();
					PB = priority_ptr;
					if (!PA) PA = "";
					if (!PB) PB = "";

					INFO(INFO_DEBUG, "priority A = %s, priority B = %s\n", PA, PB);
					
					if (strcmp(PA, PB) != 0)
					{
						ThisIncrement = Sorting->FindIncrement(SortOrderList::Priority);
	
						if (ThisIncrement == 0 || H == 0)
//old							P = add(&root);
							P = add(&root, NULL, NULL, NULL);
						else
//old							P = add(&ParentList[H - 1]->child, ParentList[H - 1]);
							P = add(&ParentList[H - 1]->child, ParentList[H - 1], NULL, NULL);

						P->TargetPkg = curPkg;
						P->Type = PackageTree::Item::Div_Pri;
						if (H < ExpandedLevel)
							P->State = PackageTree::Item::Expanded;
						P->increment = ThisIncrement;

						ThisParent = ParentList[H] = P;
						HaveParent |= true;
						LastItem = NULL;
					}
					break;
	
				case SortOrderList::Suite:
					a = strdup(curPkg.Section() ? curPkg.Section() : "");
					b = strdup(section_ptr ? section_ptr : "");

					if ((p = strchr(a, '/'))) *p=0;
					if ((p = strchr(b, '/'))) *p=0;

					if (strcasecmp(a, "non-us") != 0 && strcasecmp(a, "non-free") != 0 && strcasecmp(a, "contrib") != 0)
					{
						free(a);
						a = strdup("main");
					}

					if (strcasecmp(b, "non-us") != 0 && strcasecmp(b, "non-free") != 0 && strcasecmp(b, "contrib") != 0)
					{
						free(b);
						b = strdup("main");
					}

					if (strcasecmp(a, b) != 0 || HaveParent)
					{
						ThisIncrement = Sorting->FindIncrement(SortOrderList::Suite);

						if (ThisIncrement == 0 || H == 0)
//old							P = add(&root);
							P = add(&root, NULL, NULL, NULL);
						else
//old							P = add(&ParentList[H - 1]->child, ParentList[H - 1]);
							P = add(&ParentList[H - 1]->child, ParentList[H - 1], NULL, NULL);

						P->TargetPkg = curPkg;
						P->Type = PackageTree::Item::Div_Custom;
						if (H < ExpandedLevel)
							P->State = PackageTree::Item::Expanded;
						P->increment = ThisIncrement;
						P->Extra = strdup(a);

						ThisParent = ParentList[H] = P;
						HaveParent |= true;
						LastItem = NULL;
					}

					free(a); free(b);

					break;

				case SortOrderList::PkgName:
					if (*lastpkgname != *(curPkg.Name()))
					{
						ThisIncrement = Sorting->FindIncrement(SortOrderList::PkgName);
	
						if (ThisIncrement == 0 || H == 0)
//old							P = add(&root);
							P = add(&root, NULL, NULL, NULL);
						else
//old							P = add(&ParentList[H - 1]->child, ParentList[H - 1]);
							P = add(&ParentList[H - 1]->child, ParentList[H - 1], NULL, NULL);

						P->TargetPkg = curPkg;
						P->Type = PackageTree::Item::Div_Custom;
						if (H < ExpandedLevel)
							P->State = PackageTree::Item::Expanded;
						char buf[2];
						*buf = *(curPkg.Name()); *(buf + 1) = 0;
						P->Extra = strdup(buf);
						P->increment = ThisIncrement;

						ThisParent = ParentList[H] = P;
						HaveParent |= true;
						LastItem = NULL;
					}
					break;
					
				case SortOrderList::Size:
					if (lastpkgsize >= 0) 
					{
						if ((V->Size < 1000 && lastpkgsize < 1000) || (V->Size >= 1000000 && lastpkgsize >= 1000000) || lastpkgsize == 0)
							sizeratio = false;
						else
							sizeratio = ((int) log10 (V->Size) - (int) log10 (lastpkgsize)) >= 1;
					}
					else
						sizeratio = true;
					    
					if (sizeratio)
					{
						ThisIncrement = Sorting->FindIncrement(SortOrderList::Size);
	
						if (ThisIncrement == 0 || H == 0)
							P = add(&root, NULL, NULL, NULL);
						else
							P = add(&ParentList[H - 1]->child, ParentList[H - 1], NULL, NULL);

						P->TargetPkg = curPkg;
						P->Type = PackageTree::Item::Div_Size;
						if (H < ExpandedLevel)
							P->State = PackageTree::Item::Expanded;
						P->increment = ThisIncrement;
						
						if (V->Size < 1000) P->Extra = strdup ("<1kb");
						else if (V->Size < 10000) P->Extra = strdup ("1kb - 10kb");
						else if (V->Size < 100000) P->Extra = strdup ("10kb - 100kb");
						else if (V->Size < 1000000) P->Extra = strdup ("100kb - 1Mb");
						else P->Extra = strdup (">1Mb");
						
						ThisParent = ParentList[H] = P;
						HaveParent |= true;
						LastItem = NULL;
					}
					break;

				default: ASSERT(0); break;
				}*/
			}
		}

		INFO(INFO_VERBOSE, _("tree add pkg=%-30s sect=%s\n"), curPkg.Name(), curPkg.Section());

		// Standard package! Woop.
		if (!Sorting->Exists (SortOrderList::None))
		{
			if (ThisIncrement == 0)
//old			P = add(&root, NULL, LastItem);
				P = add(&root, NULL, LastItem, NULL);
			else
//old			P = add(&ThisParent->child, ThisParent, LastItem);
				P = add(&ThisParent->child, ThisParent, LastItem, NULL);
				
			P->TargetPkg = curPkg;
			P->Type      = PackageTree::Item::Pkg;
			P->increment = ThisIncrement + 1;
		}
		
		// Record previous priority
//old		priority_ptr = (char *) V.PriorityType();
		// Record previous section
//old		if (curPkg.Section() == NULL)
//old			section_ptr = "";
//old		else
//old			section_ptr = (char *) curPkg.Section();
		// Record previous inst state
		State = (*Cache)[curPkg].Status;
		// record last pkg name
//old		lastpkgname = curPkg.Name();
		// record last pkg size
//old		lastpkgsize = V->Size;
		// Record previous Item, speed up tree insert iterations
		LastItem = P;
	}
	
	if (Sorting->Exists (SortOrderList::None)) Sorting->Pop();

	delete [] SortedPkgs;
	delete [] ParentList;
}

void PackageTree::inject_changed_packages()
{
	pkgCache::PkgIterator I;

	Item *P, *Q;

//old	Q = add(&root);
	Q = add(&root, NULL, NULL, NULL);

	Q->increment = 0;
	Q->Type = PackageTree::Item::Div_Custom;
	Q->State = PackageTree::Item::Expanded;
	Q->Extra = (char *) strdup(_("Packages to Install"));

	for (I = (*Cache)->PkgBegin(); I.end() != true; I++)
	{
		if ((*Cache)[I].CandidateVerIter(*Cache).end() == true)
			continue;

		if ((*Cache)[I].NewInstall() == true)
		{
//old			P = add(&Q->child, Q);
			P = add(&Q->child, Q, NULL, NULL);

			P->increment = 1;
			P->Type = PackageTree::Item::Pkg;
			P->TargetPkg = I;
			P->State = PackageTree::Item::None;
		}
	}

//old	Q = add(&root);
	Q = add(&root, NULL, NULL, NULL);

	Q->increment = 0;
	Q->Type = PackageTree::Item::Div_Custom;
	Q->State = PackageTree::Item::Expanded;
	Q->Extra = (char *) strdup(_("Packages to Reinstall"));

	for (I = (*Cache)->PkgBegin(); I.end() != true; I++)
	{
		if ((*Cache)[I].Delete() == false && ((*Cache)[I].iFlags & pkgDepCache::ReInstall) == pkgDepCache::ReInstall)
		{
//old			P = add(&Q->child, Q);
			P = add(&Q->child, Q, NULL, NULL);

			P->increment = 1;
			P->Type = PackageTree::Item::Pkg;
			P->TargetPkg = I;
			P->State = PackageTree::Item::None;
		}
	}

//old	Q = add(&root);
	Q = add(&root, NULL, NULL, NULL);

	Q->increment = 0;
	Q->Type = PackageTree::Item::Div_Custom;
	Q->State = PackageTree::Item::Expanded;
	Q->Extra = (char *) strdup(_("Packages to Upgrade"));

	for (I = (*Cache)->PkgBegin(); I.end() != true; I++)
	{
		if ((*Cache)[I].Upgrade() == false || (*Cache)[I].NewInstall() == true)
			continue;

		if ((*Cache)[I].CandidateVerIter(*Cache).end() == true)
			continue;

//old		P = add(&Q->child, Q);
		P = add(&Q->child, Q, NULL, NULL);

		P->increment = 1;
		P->Type = PackageTree::Item::Pkg;
		P->TargetPkg = I;
		P->State = PackageTree::Item::None;
	}

//old	Q = add(&root);
	Q = add(&root, NULL, NULL, NULL);

	Q->increment = 0;
	Q->Type = PackageTree::Item::Div_Custom;
	Q->State = PackageTree::Item::Expanded;
	Q->Extra = (char *) strdup(_("Packages to Delete"));

	for (I = (*Cache)->PkgBegin(); I.end() != true; I++)
	{
		if ((*Cache)[I].CandidateVerIter(*Cache).end() == true)
			continue;

		if ((*Cache)[I].Delete() == false)
			continue;

//old		P = add(&Q->child, Q);
		P = add(&Q->child, Q, NULL, NULL);

		P->increment = 1;
		P->Type = PackageTree::Item::Pkg;
		P->TargetPkg = I;
		P->State = PackageTree::Item::None;
	}
}

void PackageTree::FreeTree(Item *&z)
{
	if (z->child)
		FreeTree(z->child);
	
	if (z->next)
		FreeTree(z->next);

	remove(z);
}

PackageTree::Item *PackageTree::append(Item * pos)
{
	Item *ptr;

	if (pos == NULL)
		return NULL;

	ptr = pos->next;

	pos->next = (Item *) malloc(sizeof(Item));

	memset(pos->next, 0, sizeof(Item));

	pos->next->prev = pos;
	pos->next->next = ptr;
	pos->next->parent = pos->parent;
	pos->child = 0;

	if (ptr != NULL)
		ptr->prev = pos->next;

	PackageCount++;

	return pos->next;
}


//oldPackageTree::Item *PackageTree::add(Item **head, Item *parent = 0, Item *last = 0, Item *carbon = 0)
PackageTree::Item *PackageTree::add(Item **head, Item *parent, Item *last , Item *carbon)
{
	Item *P = 0;

	if (*head == 0)
	{
		*head = (Item *) malloc(sizeof(Item));

		if (carbon == 0)
			memset(*head, 0, sizeof(Item));
		else
			memcpy(*head, carbon, sizeof(Item));

		(*head)->parent = parent;
		(*head)->child = 0;
		(*head)->prev = 0;
		(*head)->next = 0;

		P = *head;
	}
	else
	{
		Item *Q;

		if (last)
			Q = last;
		else
			for (Q = *head; Q->next != 0; Q = Q->next);

		Q->next = (Item *) malloc(sizeof(Item));

		if (carbon == 0)
			memset(Q->next, 0, sizeof(Item));
		else
			memcpy(Q->next, carbon, sizeof(Item));
		
		Q->next->prev = Q;
		Q->next->next = 0;
		Q->next->parent = parent;
		Q->next->child = 0;

		P = Q->next;
	}

	PackageCount++;

	return P;
}

void PackageTree::remove(Item * p)
{
	Item *x, *y;

	if (p->IsDepend())
		delete static_cast<pkgCache::DepIterator *>(p->Extra);
	else if (p->Extra != NULL)
		free(p->Extra);

	x = p->next;
	y = p->prev;

	// Take care of our parent's child link if we are the first node in the child list
	if (p->parent != 0)
		if (p == p->parent->child)
			p->parent->child = p->next;

	// If we have children, make sure they are happy as well
	for (Item *z = p->child; z != 0; z = z->next)
			z->parent = p->parent;

	free(p);

	if (x != 0)
		x->prev = y;

	if (y != 0)
		y->next = x;

	if (p == root)
		root = 0;
	
	PackageCount--;
}

//old unsigned PackageTree::Locate(char *substring, unsigned long options)
unsigned PackageTree::Locate(char **substring, unsigned long *options)
{
/*	regex_t rectx;

	if (options & SEARCH_REGEX)
	{
		int reflags = REG_EXTENDED | REG_NOSUB;

		if (options & SEARCH_CASE)
			reflags |= REG_ICASE;

		regcomp(&rectx, substring, reflags);
	}
	else if (options & SEARCH_CASE)
	{
		strlower(substring);
	}

	unsigned numpkgsmatched = 0;
	const char *desc;
	bool DoExpansion = _Conf.GetBool("edeity.search.expand_matched_item_parents");

	for (Iterator T(*this); !T.Invalid(); T.MoveNextLinear())
	{
		if (T->IsDivider())
			continue;

		if (options & SEARCH_DESC && T->Version().end() == false)
			desc = Cache->GetPkgRecs().Lookup(T->Version().FileList()).LongDesc().c_str();
		else
			desc = NULL;

		if (search_string(T->TargetPkg.Name(), substring, options, &rectx))
		{
			(int) T->State |= PackageTree::Item::Matched;

			if (DoExpansion)
				for (Item *x = T; x->parent; x = x->parent)
					(int) x->parent->State |= PackageTree::Item::Expanded;

			numpkgsmatched++;
		}
		else if (desc && search_string(desc, substring, options, &rectx))
		{
			(int) T->State |= PackageTree::Item::Matched;

			if (DoExpansion)
				for (Item *x = T; x->parent; x = x->parent)
					(int) x->parent->State |= PackageTree::Item::Expanded;

			numpkgsmatched++;
		}
	}

	if (options & SEARCH_REGEX)
		regfree(&rectx);

	return numpkgsmatched;*/
	
	int numexp;
	unsigned numpkgsmatched = 0;
	bool DoExpansion = _Conf.GetBool("edeity.search.expand_matched_item_parents");
	
	for (numexp = 0; numexp < 6 && substring[numexp]; numexp++);
	
	for (Iterator T(*this); !T.Invalid(); T.MoveNextLinear())
	{
	    if (T->IsDivider())
			continue;

	    for (int i = 0; (i < numexp) && !(T->State & PackageTree::Item::Matched); i++)
	    {
		if (options[i] & Cmp_And/*SEARCH_AND*/)
			continue;

		if (search_string(GetDesc(T, options[i]), substring[i], options[i]))
		    (int) T->State |= PackageTree::Item::Matched;
	    }
	    
	    for (int i = 0; (i < numexp) && (T->State & PackageTree::Item::Matched); i++)
	    {
		if (options[i] & Cmp_Or /*SEARCH_OR*/)
			continue;
			
		if (!search_string(GetDesc(T, options[i]), substring[i], options[i]))
		    (int) T->State &= ~PackageTree::Item::Matched;
	    }
	    
	    if (T->State & PackageTree::Item::Matched)
	    {
		if (DoExpansion)
		    for (Item *x = T; x->parent; x = x->parent)
			(int) x->parent->State |= PackageTree::Item::Expanded;
				
		numpkgsmatched++;
	    }
	}


	return numpkgsmatched;
}

//old PackageTree::Item * PackageTree::LocateNext(bool start_at_beginning = false)
PackageTree::Item * PackageTree::LocateNext(bool start_at_first)
{
	Iterator T;

	switch (start_at_first)
	{
	case true:
		T = root;
		break;
	case false:
		T = Owner->cs;
		T++;
		break;
	}

	for (; !T.Invalid(); T++)
	{
//old		if (T->IsDivider() || T == Owner->cs)
		if (T->IsDivider())
			continue;

		if (T->IsMatched())
			return T;
	}

//old	return NULL;
	return start_at_first ? NULL : LocateNext (true);
}

PackageTree::Item * PackageTree::LocatePrev(bool start_at_last)
{
	Iterator T;

	switch (start_at_last)
	{
	case true:
		for (Iterator Q = root; !Q.Invalid(); Q++) T = Q;
		break;
	case false:
		T = Owner->cs;
		T--;
		break;
	}

	for (; !T.Invalid(); T--)
	{
//old		if (T->IsDivider() || T == Owner->cs)
		if (T->IsDivider())
			continue;

		if (T->IsMatched())
			return T;
	}

	return start_at_last ? NULL : LocatePrev (true);
}

void PackageTree::RemoveState(int Z)
{
	for (Iterator I = root; !I.Invalid(); I++)
	{
		if (I->IsDivider())
			continue;

		(int) I->State &= ~Z;
	}
}

void PackageTree::CollapseDividers()
{
	for (Iterator I = root; !I.Invalid(); I.MoveNextLinear())
	{
		if (I->IsDivider())
			(int) I->State &= ~PackageTree::Item::Expanded;
	}
}

int PackageTree::CountState(int Z)
{
	int count = 0;

	for (Iterator I = root; !I.Invalid(); I++)
		if (I->State & Z)
			count++;

	return count;
}

string PackageTree::GetHeaderDesc(Item * P)
{
	string Format;
	char *ptr, *x;
	const char *Section=NULL;
	unsigned long size;
	
	if (P->IsDivider())
	{
		switch (P->Type)
		{

		case PackageTree::Item::Div_sState:
			if ((*Cache)[P->TargetPkg].InstBroken())
							Format += _("Broken");
							
			else if ((*Cache)[P->TargetPkg].NewInstall()) 
							Format +=  _("Install");
							
			else if ((*Cache)[P->TargetPkg].Upgrade())
							Format += _("Upgrade");
							
			else if ((*Cache)[P->TargetPkg].Downgrade())
							Format += _("Delete");
							
			else if ((*Cache)[P->TargetPkg].Install())
							Format +=  _("Install");
							
			else if ((*Cache)[P->TargetPkg].Delete())
					{
						if ((*Cache)[P->TargetPkg].iFlags & pkgDepCache:: Purge)
							Format += _("Purge");
						else 
							Format += _("Remove");
					}
					else
					{
						if ((*Cache)[P->TargetPkg].iFlags & pkgDepCache::ReInstall)
							Format += _("Reinstall");
						else
							Format +=  _("Keep");
					}
		break;
			
		case PackageTree::Item::Div_State:
			if ((*Cache)[P->TargetPkg].Status == 1)
//old				Format += FMT_DIV_STAT_INST;
				Format += _("Installed packages (newer version is available)");
			else if ((*Cache)[P->TargetPkg].Status == 0)
//old				Format += FMT_DIV_STAT_NOTINST;
				Format += _("Upgraded packages (can only remove or purge)");
			else
//old				Format += FMT_DIV_STAT_NOTINST;
				Format += _("Non-installed packages (available for installation)");
			break;
			
		case PackageTree::Item::Div_Dep:
			if (P->Depend().end()) 
				Format += _("Non-dependent (package has no dependency)");
			else
				Format += _("Dependent (package has any dependency)");
			break;							
						

		case PackageTree::Item::Div_rDep:
			if (P->RevDepend().end()) 
				Format += _("Non-referred (other package not depends on it)");
			else
				Format += _("Referred (package depended by other package(s))");
			break;							
			
		case PackageTree::Item::Div_Sect:
/*			if (strncmp(P->TargetPkg.Section(), "contrib", 7) == 0)
			{
				if (strncmp(P->TargetPkg.Section() + 7, "/", 1) == 0)
				{
					Section = P->TargetPkg.Section() + 8;
					Format += "(contrib) ";
				}
				else
					Format += "(contrib) Contributed software packages";
			}
			else if (strncmp(P->TargetPkg.Section(), "non-free", 8) == 0)
			{
				if (strncmp(P->TargetPkg.Section() + 8, "/", 1) == 0)
				{
					Section = P->TargetPkg.Section() + 9;
					Format += "(non-free) ";
				}
				else
					Format += "(non-free) Software without a DFSG compatible license";
			}
			else if (strncasecmp(P->TargetPkg.Section(), "non-US", 6) == 0)
			{
				if (strncmp(P->TargetPkg.Section() + 6, "/", 1) == 0)
				{
					Section = P->TargetPkg.Section() + 7;
					Format += "(non-us) ";
				}
				else
					Format += "(non-us) Software that is restricted from being exported in the United States";
			}
			else
			{
				Section = P->TargetPkg.Section();
				Format += "(main) ";
			}
*/
			if (P->TargetPkg.Section() == NULL)
//old				Format += FMT_DIV_SECT_NULL;
				Format += _("foreign: packages without section");
			else
			{
				Section = strrchr(P->TargetPkg.Section(), '/');
				if (Section == 0)
					Section = P->TargetPkg.Section();
				else
					Section += 1;
				Format += Section;
				Format += ": ";
				Format += Section_Lookup((char *)Section);
			}

			break;

		case PackageTree::Item::Div_Pri:

			ptr = (char *) (*Cache)[P->TargetPkg].CandidateVerIter(*Cache).PriorityType();

			if (ptr == 0 || *ptr == 0)
			{
				INFO(INFO_DEBUG, _("Package %s has empty priority info\n"), P->TargetPkg.Name());
//old				Format += FMT_DIV_PRI_NULL;
				Format += _("unknown priority");
			}
			else
			{
				Format += _("priority ");
				Format += ptr;
			}

			break;

		case PackageTree::Item::Div_Suite:
			ptr = strdup(P->TargetPkg.Section());
			x = strchr(ptr, '/');

			if (x == NULL)
				Format += "main";
			else
			{
				*x = 0;
				Format += ptr;
			}

			free(ptr);
			
			break;

		case PackageTree::Item::Div_Size:
		case PackageTree::Item::Div_iSize:
			size = (P->Type == PackageTree::Item::Div_Size) ?
					P->Version()->Size :
					P->Version()->InstalledSize;
		
			if (size < 1000) Format += "<1kb";
			else if (size < 10000) Format += "1kb - 10kb";
			else if (size < 100000) Format += "10kb - 100kb";
			else if (size < 1000000) Format += "100kb - 1Mb";
			else Format += ">1Mb";
			
		break;
		
		case PackageTree::Item::Div_Custom:
		case PackageTree::Item::Div_fName:

			Format += (char *) P->Extra;

			break;

		default:
			break;
		}
	}
	return Format;
}

const char* PackageTree::GetDesc(Item * T, unsigned long option)
{
	if (option & Pkg_Name/*SEARCH_NAME*/) return T->Name();
   	else if (option & Pkg_AvailVer /*SEARCH_CVERS*/) return T->AvlVer();
   	else if (option & Pkg_InstlVer /*SEARCH_IVERS*/) return T->InstVer();
	else if (option & Pkg_ShortDesc /*SEARCH_SDESC*/) return T->ShortDesc();
	else if (option & Pkg_LongDesc /*SEARCH_LDESC*/) return T->LongDesc();
	else if (option & Pkg_Maintainer /*SEARCH_MTAIN*/) return T->Maintainer();
	else if (option & Pkg_FileName /*SEARCH_FNAME*/) return T->FileName();

    return (const char *) "";
}

void PackageTree::RecalcLength()
{
	int K = 0;
	for (Iterator I = root; I.Invalid() == false; I++, K++);
	PackageCount = K;
}
