/* $Id: pkgtree.h,v 1.6 2001/03/24 08:08:32 ltd Exp $ */
#ifndef _pkgtree_H
#define _pkgtree_H

#include <std.h>

#include "filters.h"
#include "sorting.h"
#include "column.h"

class FilterList;
class Screen;
class SortOrderList;

class PackageTree
{
public:
	class Iterator;
	struct Item;
	friend struct Item;
	friend class Iterator;
	friend class FilterList;

	enum ListType { Empty = 0, AllPkgs = 1, ModPkgs = 2 };
	enum DepViewType { NoneDep = 0, NormDep = 1, RevDep = 2 };

protected:
	Screen * Owner;
	Item *root;
	ListType Type;
	DepViewType DependType;
	long PackageCount;
	long TotalPackageCount;
	
	void sort_packages(SortOrderList::SortIterator *, pkgCache::Package **, long, long);
	
	public:
	SortOrderList *Sorting;
	ColOrderList *Columns;
	FilterList *Filters;


	// Methods to inject packages into the list (ListType)
	void reinitialize();
	void inject_all_packages();
	void inject_changed_packages();
	void FreeTree(Item *&);

	// Methods to filter the package list
	void apply_filters();

	// List modification functions
//old	Item *add(Item **head, Item *parent=NULL, Item *last=NULL, Item *carbon=NULL);
	Item *add(Item **head, Item *parent, Item *last, Item *carbon);
	Item *append(Item *prev=NULL);
	void remove(Item *);

	// Locate functions
//	Item *LocateNext(bool start_at_beginning = false);
	Item *LocateNext(bool start_at_beginning);
	Item *LocatePrev(bool start_at_beginning);
	Item *LocateFirst(void) { return LocateNext(true); };
//old	unsigned Locate(char *, unsigned long);
	unsigned Locate(char **, unsigned long *);
	unsigned collect_packages(pkgCache::Package**);

	// Flag typing functions
	void CollapseDividers();
	int CountState(int);
	void RemoveState(int);

	// Wrappers so that HiddenCount gets updated like it should
	void HidePkg(Item *);
	void ShowPkg(Item *);

	// Divider information
	string GetHeaderDesc(Item *);
	const char* GetDesc(Item *, unsigned long);

	// Size operators
	inline long Length() { return PackageCount; };
	inline long TotalLength() { return TotalPackageCount; };
	
	inline void RecalcLength();

	// Dependecy type
	inline DepViewType DepType() { return DependType; }
	void ToggleDepType() { DependType = (DependType == RevDep) ? NormDep : RevDep; };

	// Pointer accessors
	inline Item *Root() { return root; };

	// Hidden package seeking wrappers

//old	PackageTree(Screen *, ListType Init = AllPkgs);
	PackageTree(Screen *, ListType Init, DepViewType Dep);
	~PackageTree();
};

#include "pkgtreeitem.h"
#include "pkgtreeiterator.h"

#endif // _pkgtree_H
