/* $Id: pkgtreeitem.h,v 1.4 2001/05/14 06:57:32 ltd Exp $ */
#ifndef _pkgtreeitem_H
#define _pkgtreeitem_H

//old #include <std.h>

class PackageTree;

struct PackageTree::Item
{
	enum pkgState { None = 0, Expanded = (1 << 0), Matched = (1 << 1), Hidden = (1 << 2) };
	enum pkgWill { Broken, Delete, Install, Keep, Purge, ReInstall, Remove, Upgrade};
	enum pkgType { Pkg, Dep, RevDep, Div_Sect, Div_Pri, Div_State, Div_sState, Div_Custom, Div_Suite, Div_Size, Div_iSize, Div_fName, Div_Dep, Div_rDep };

	int increment;		// How far this pkg increments in dependency
	void *Extra;		// A pointer for other uses ...

	pkgCache::PkgIterator TargetPkg;
	//pkgCache::VerIterator TargetVer;

	pkgState State;
	pkgType Type;

	// Behave like a pkgiterator
	inline operator pkgCache::PkgIterator & () { return TargetPkg; };

	// Access the target version for this package
	inline pkgCache::VerIterator Version() { return TargetPkg.VersionList(); };
	inline pkgCache::DepIterator Depend() { return Version().DependsList(); };
	inline pkgCache::DepIterator RevDepend() { return TargetPkg.RevDependsList(); };
	inline pkgCache::VerIterator Candidate() { return (*Cache)[TargetPkg].CandidateVerIter(*Cache); };
	inline pkgCache::VerIterator InstVersion() { return (*Cache)[TargetPkg].InstVerIter(*Cache); };
	
	inline const char * Name () { return TargetPkg.Name(); };
	inline const char * Is () { return ((*Cache)[TargetPkg].Status == 1 ? _("Installed") : ((InstVersion().end() == false || (*Cache)[TargetPkg].Delete() == true) && (*Cache)[TargetPkg].NewInstall() == false ? _("Upgraded") : _("Kept"))); };
	inline const char * Will () { return ((*Cache)[TargetPkg].InstBroken()? _("Broken") : ((*Cache)[TargetPkg].  NewInstall()? _("Install") : ((*Cache)[TargetPkg].  Upgrade()?  _("Upgrade") : ((*Cache)[TargetPkg].  Downgrade()?  _("Delete") : ((*Cache) [TargetPkg].  Install()?  _("Install") : ( (*Cache) [TargetPkg].  Delete ()? ( ( (*Cache) [TargetPkg].  iFlags & pkgDepCache:: Purge) ? _("Purge") : _("Remove")) : ((*Cache)[TargetPkg].iFlags & pkgDepCache::ReInstall) ? _("Reinstall") : _("Keep"))))))); };
	inline const char * InstVer () { return InstVersion().end() ? "" : InstVersion().VerStr(); };
	inline const char * AvlVer () { return Candidate().end() ? "" : Candidate().VerStr(); };
	inline const char * ShortDesc () { return Cache->GetPkgRecs().Lookup(Candidate().FileList()).ShortDesc().c_str(); };
	inline const char * LongDesc () { return Cache->GetPkgRecs().Lookup(Candidate().FileList()).LongDesc().c_str(); };
	inline const char * Maintainer () { return Cache->GetPkgRecs().Lookup(Candidate().FileList()).Maintainer().c_str(); };
	inline const char * MD5Hash () { return Cache->GetPkgRecs().Lookup(Candidate().FileList()).MD5Hash().c_str(); };
	inline const char * Section () { return Version().Section(); };
	inline const char * Priority () { return Version().PriorityType(); };
	inline const char * FileName () { return Cache->GetPkgRecs().Lookup(Candidate().FileList()).FileName().c_str(); };
	inline const char * SourcePkg () { return Candidate().FileList().File().Origin(); };
	inline const char * Architecture () { return Candidate().FileList().File().Architecture(); };
	
	const char * Size ();
	const char * Instsize();
	const char * FileNames ();
	const char * Information ();

	// Common flag checking functions
	inline bool IsPackage() { return Type == Pkg; };
//old	inline bool IsDivider() { return (Type == Pkg || Type == Dep) ? false : true; };
	inline bool IsDivider() { return (Type == Pkg || Type == Dep || Type == RevDep) ? false : true; };
	inline bool IsDepend() { return Type == Dep; };
	inline bool IsRevDepend() { return Type == RevDep; };
	inline bool IsDepended() { return !Depend().end(); };
	inline bool IsRevDepended() { return !RevDepend().end(); };
	inline bool IsMatched() { return State & Matched; };
	inline bool IsExpanded() { return State & Expanded; };
	inline bool IsHidden() { return State & Hidden; };

	struct PackageTree::Item *next, *prev, *child, *parent;
};

#endif // _pkgtreeitem_H
