/* $Id: sorting.cc,v 1.13 2001/05/15 08:32:58 ltd Exp $ 
 * This list is utilised when a PackageTree is constructed.
 */
 
//old #include <std.h>
#include <math.h>
//old #include <stdio.h>
#include <list/screens.h>
#include <func/config.h>
#include <func/apt.h>
#include <interface/edeityui.h>
#include "sorting.h"

SortOrderList *LastOrderCreated = NULL;

SortOrderList::SortOrderList()
{
	Head = Tail = NULL;
	nodecount = 0;
	copy_from_config();
	LastOrderCreated = this;
}

SortOrderList::~SortOrderList()
{
	Flush();
}

//old void SortOrderList::Push(method_t myMethod)
void SortOrderList::Push(method_t myMethod, bool invert)
{	
	nodecount += 1;

	if (Head == NULL && Tail == NULL)
	{
		Head = Tail = new Node;
		Tail->next = NULL;
	}
	else
	{
		Tail->next = new Node;
		Tail->next->next = NULL;
		Tail = Tail->next;
	}

	Tail->seqid = nodecount;
	Tail->thismethod = myMethod;
	Tail->single_cmp = GetComparer (myMethod, invert);
	Tail->group_cmp = GetGrouper (myMethod, invert);
	
	reorderseqids();
}

SortOrderList::Comparer* SortOrderList::GetComparer(method_t myMethod, bool invert)
{
	switch (myMethod)
	{
	    case None:
			return compare_none;
	    
	    case Section:
			return invert ? icompare_section : compare_section;
	    
	    case PkgName:
	    	return invert ? icompare_package_name : compare_package_name;
	    
	    case Priority:
			return invert ? icompare_priority : compare_priority;
	    
	    case Status:
			return invert ? icompare_status : compare_status;
	    
	    case SpecStat:
			return invert ? icompare_specstat : compare_specstat;
	    
	    case Size:
	    	return invert ? icompare_archive_size : compare_archive_size;
	    
	    case Instsize:
			return invert ? icompare_installed_size : compare_installed_size;
	    
	    case Filename:
	    	return invert ? icompare_package_fname : compare_package_fname;
	    
	    case Suite:
	    	return invert ? icompare_suite : compare_suite;
	    
	    case Dep:
	    	return invert ? icompare_dep : compare_dep;
	    
	    case RevDep:
	    	return invert ? icompare_revdep : compare_revdep;
	    
	    default:
			return NULL;
	}
}

SortOrderList::Comparer* SortOrderList::GetGrouper(method_t myMethod, bool invert)
{
	switch (myMethod)
	{
	    case None:
			return compare_none;
	    
	    case Section:
			return invert ? icompare_section : compare_section;
	    
	    case PkgName:
	    	return invert ? igroup_package_name : group_package_name;
	    
	    case Priority:
			return invert ? icompare_priority : compare_priority;
	    
	    case Status:
			return invert ? icompare_status : compare_status;
	    
	    case SpecStat:
			return invert ? icompare_specstat : compare_specstat;
	    
	    case Size:
	    	return invert ? igroup_archive_size : group_archive_size;
	    
	    case Instsize:
			return invert ? igroup_installed_size : group_installed_size;
	    
	    case Filename:
	    	return invert ? igroup_package_fname : group_package_fname;
	    
	    case Suite:
	    	return invert ? icompare_suite : compare_suite;
	    
	    case Dep:
	    	return invert ? icompare_dep : compare_dep;
	    
	    case RevDep:
	    	return invert ? icompare_revdep : compare_revdep;
	    
	    default:
			return NULL;
	}
}

void SortOrderList::Insert(SortIterator & S, method_t myMethod, bool invert)
{	
	Node *x, *insertpos = S.Pos;

	if (insertpos == Head)
	{
		x = Head;
		Head = new Node;
		Head->next = x;

		x = Head;
	}
	else
	{
		for (x = Head; x->next != insertpos; x = x->next);

		Node *y = x->next;

		x->next = new Node;
		x->next->next = y;

		x = x->next;
	}
		
	x->seqid = 0;
	x->thismethod = myMethod;

	nodecount += 1;

	reorderseqids();
}

void SortOrderList::Pop(void)
{
	struct Node *p;

	nodecount -= 1;

	if (Head == Tail)
	{
		delete Head;
		Head = Tail = NULL;
	}
	else
	{
		for (p = Head; p->next != Tail; p = p->next);
		p->next = p->next->next;
		delete Tail;
		Tail = p;
	}

	reorderseqids();
}

void SortOrderList::remove(SortIterator & S)
{
	Node *p = S.Pos;

	Node *x = NULL;

	if (p == Head && p == Tail)
	{
		Head = Tail = NULL;
		delete p;
		nodecount = 0;
	}
	else if (p == Head)
	{
		x = Head;
		Head = Head->next;
		delete x;
		nodecount -= 1;
	}
	else if (p == Tail)
	{
		for (x = Head; x->next != Tail; x = x->next);
		x->next = x->next->next;
		delete Tail;
		Tail = x;
		nodecount -= 1;
	}
	else
	{
		for (x = Head; x->next != p; x = x->next);
		x->next = x->next->next;
		delete p;
		nodecount -= 1;
	}

	reorderseqids();
}

void SortOrderList::Flush(void)
{
	while (nodecount > 0)
		Pop();
}


/*
 * Useful for formatting dividers ... [Order of sort methods]
 */
int SortOrderList::FindIncrement(enum SortOrderList::method_t Type)
{
	for (SortIterator S = ListBegin(); S.end() == false; S++)
	{
		if (S.method() == Type)
			return S.seq()-1;
	}

	return -1;
}

/*
 * Useful for formatting dividers ... [Order of sort methods]
 */
bool SortOrderList::Exists(enum SortOrderList::method_t Type)
{
	for (SortIterator S = ListBegin(); S.end() == false; S++)
	{
		if (S.method() == Type)
			return true;
	}

	return false;
}

/*
 * Simple.. count the number of sort methods that divide the package list
 */
int SortOrderList::DividerCount()
{
	int I = 0;

	for (SortIterator S = ListBegin(); S.end() == false; S++)
	{
//old		if (S.method() == Status || S.method() == Section || S.method() == Priority)
			I++;
	}

	return I;
}

void SortOrderList::reorderseqids()
{
	int I;
	SortIterator S;

	for (I = 1, S = ListBegin(); S.end() == false; S++, I++)
		S.Pos->seqid = I;
}

void SortOrderList::commit_to_config()
{
	string s;
	char buf[50];

	for (SortIterator S = ListBegin(); S.end() == false; S++)
	{
		sprintf(buf, "%d", S.method());
		s += buf;
		s += "/";
	}

	if (nodecount > 0)
		s.resize(s.size()-1);
	else
		s = "NULL";

	_Conf.Set("edeity.sorting", (char *)s.c_str());
}

void SortOrderList::copy_from_config()
{
	Flush();

	char *tmpbuf = strdup(_Conf.Get("edeity.sorting"));

	if (strcmp(tmpbuf, "NULL") != 0)
		for (char *p = strtok(tmpbuf, "/"); p != NULL; p = strtok(NULL, "/"))
//old			Push((SortOrderList::method_t)atoi(p));
			Push((SortOrderList::method_t)atoi(p), false);

	free(tmpbuf);
}

SortOrderList::SortIterator SortOrderList::ListFind(method_t ThisMethod)
{
	SortIterator S(ListBegin());

	for (; S.end() == false; S++)
		if (S.method() == ThisMethod)
			return S;

	return S;
}
/*
int SortOrderList::qsort_handler(const void *a, const void *b)
{
	const pkgCache::Package & A = **(pkgCache::Package **) a;
	const pkgCache::Package & B = **(pkgCache::Package **) b;
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) a);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) b);
	pkgCache::VerIterator VerA = IterA.VersionList();
	pkgCache::VerIterator VerB = IterB.VersionList();

	SortIterator S;
	SortIterator End = LastOrderCreated->ListEnd();

	for (S = LastOrderCreated->ListBegin(); S.end() == false; S++)
	{
		if (S == End)
		{
			switch (S.method())
			{
			case None:
				break;
			case Section:
				return compare_section(&A, &B);
			case PkgName:
				return compare_package_name(&A, &B);
			case Priority:
				return compare_priority(VerA, VerB);
			case Status:
				return compare_status(IterA, IterB);
			case Size:
				return compare_archive_size(VerA, VerB);
			case Instsize:
				return compare_installed_size(VerA, VerB);
			case Distribution:
				return compare_distribution(VerA, VerB);
			case Suite:
				return compare_suite(&A, &B);
			}
		}
		else
		{
			int comparison = 0;
			
			switch (S.method())
			{
			case None:
				break;
			case Section:
				comparison=compare_section(&A, &B);
				break;
			case PkgName:
				comparison=compare_package_name(&A, &B);
				break;
			case Priority:
				comparison=compare_priority(VerA, VerB);
				break;
			case Status:
				comparison=compare_status(IterA, IterB);
				break;
			case Size:
				comparison=compare_archive_size(VerA, VerB);
				break;
			case Instsize:
				comparison=compare_installed_size(VerA, VerB);
				break;
			case Distribution:
				comparison=compare_distribution(VerA, VerB);
				break;
			case Suite:
				comparison=compare_suite(&A, &B);
				break;
			}

			if (comparison != 0)
				return comparison;
		}
	}
	// This shouldn't happen

	return 0;
}
*/

int SortOrderList::icompare_section(const void * P1, const void * P2)
{
	return compare_section(P2, P1);
}

//old int SortOrderList::compare_section(const pkgCache::Package * A, const pkgCache::Package * B)
int SortOrderList::compare_section(const void * P1, const void * P2)
{
//old	const pkgCache::Package *A = *(pkgCache::Package **) P1;
//old	const pkgCache::Package *B = *(pkgCache::Package **) P2;
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	pkgCache::VerIterator VerA = IterA.VersionList();
	pkgCache::VerIterator VerB = IterB.VersionList();
	
/*old
	char *a, *b;
	char *strp = Cache->GetCache()->StrP;

	for (a = strp + A->Section; *a != 0; a++)
		if (*a == '/')
		{
			a++;
			break;
		}

	if (*a == 0)
		a = strp + A->Section;

	for (b = strp + B->Section; *b != 0; b++)
		if (*b == '/')
		{
			b++;
			break;
		}
	
	if (*b == 0)
		b = strp + B->Section;

	return strcmp(a, b);
*/

	return strcmp (VerA.Section(), VerB.Section());
}

int SortOrderList::icompare_package_name(const void * P1, const void * P2)
{
	return compare_package_name(P2, P1);
}

//old int SortOrderList::compare_package_name(const pkgCache::Package * A, const pkgCache::Package * B)
int SortOrderList::compare_package_name(const void * P1, const void * P2)
{
//old	const pkgCache::Package *A = *(pkgCache::Package **) P1;
//old	const pkgCache::Package *B = *(pkgCache::Package **) P2;
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	
//old	char *strp = Cache->GetCache()->StrP;
//old	return strcmp(strp + A->Name, strp + B->Name);
	return strcmp(IterA.Name(), IterB.Name());
}

int SortOrderList::igroup_package_name(const void * P1, const void * P2)
{
	return group_package_name(P2, P1);
}

int SortOrderList::group_package_name(const void * P1, const void * P2)
{
//old	const pkgCache::Package *A = *(pkgCache::Package **) P1;
//old	const pkgCache::Package *B = *(pkgCache::Package **) P2;
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	
//old	char *strp = Cache->GetCache()->StrP;
	
//old	if (*(strp + A->Name) < *(strp + B->Name))
	if (*IterA.Name() < *(IterB.Name()))
		return -1;
//old	if (*(strp + A->Name) == *(strp + B->Name))
	if (*IterA.Name() == *IterB.Name())
		return 0;
	else
		return 1;
}

int SortOrderList::icompare_package_fname(const void * P1, const void * P2)
{
	return compare_package_fname(P2, P1);
}

int SortOrderList::compare_package_fname(const void * P1, const void * P2)
{
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	pkgCache::VerIterator VerA = IterA.VersionList();
	pkgCache::VerIterator VerB = IterB.VersionList();
	
	const char * NameA = strrchr (Cache->GetPkgRecs().Lookup(VerA.FileList()).FileName().c_str(), '/');
	const char * NameB = strrchr (Cache->GetPkgRecs().Lookup(VerB.FileList()).FileName().c_str(), '/');
	
	if (NameA) NameA++; else NameA = "";
	if (NameB) NameB++; else NameB = "";

	return strcmp(NameA, NameB);
}

int SortOrderList::igroup_package_fname(const void * P1, const void * P2)
{
	return group_package_fname(P2, P1);
}

int SortOrderList::group_package_fname(const void * P1, const void * P2)
{
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	pkgCache::VerIterator VerA = IterA.VersionList();
	pkgCache::VerIterator VerB = IterB.VersionList();
	
	const char * NameA = VerA.end() ? "" : strrchr (Cache->GetPkgRecs().Lookup(VerA.FileList()).FileName().c_str(), '/');
	const char * NameB = VerB.end() ? "" : strrchr (Cache->GetPkgRecs().Lookup(VerB.FileList()).FileName().c_str(), '/');

	if (NameA) NameA++; else NameA = "";
	if (NameB) NameB++; else NameB = "";

	if (*(NameA) < *(NameB))
		return -1;
	if (*(NameA) == *(NameB))
		return 0;
	else
		return 1;
}

int SortOrderList::icompare_priority(const void * P1, const void * P2)
{
	return compare_priority(P2, P1);
}

//old int SortOrderList::compare_priority(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB)
int SortOrderList::compare_priority(const void * P1, const void * P2)
{
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	pkgCache::VerIterator VerA = IterA.VersionList();
	pkgCache::VerIterator VerB = IterB.VersionList();
	const char *PA = VerA.PriorityType(), *PB = VerB.PriorityType();

	if (PA == 0)
		PA = "";
	if (PB == 0)
		PB = "";

	return strcmp(PA, PB);

	if (strcmp(PB, "Required") == 0)
	{
		if (strcmp(PA, "Required") == 0)
			return 0;
		else if (strcmp(PA, "Important") == 0)
			return 1;
		else if (strcmp(PA, "Standard") == 0)
			return 2;
		else if (strcmp(PA, "Optional") == 0)
			return 3;
		else if (strcmp(PA, "Extra") == 0)
			return 4;
		else
			return 5;
	}
	else if (strcmp(PB, "Important") == 0)
	{
		if (strcmp(PA, "Required") == 0)
			return -1;
		else if (strcmp(PA, "Important") == 0)
			return 0;
		else if (strcmp(PA, "Standard") == 0)
			return 1;
		else if (strcmp(PA, "Optional") == 0)
			return 2;
		else if (strcmp(PA, "Extra") == 0)
			return 3;
		else
			return 4;
	}
	else if (strcmp(PB, "Standard") == 0)
	{
		if (strcmp(PA, "Required") == 0)
			return -2;
		else if (strcmp(PA, "Important") == 0)
			return -1;
		else if (strcmp(PA, "Standard") == 0)
			return 0;
		else if (strcmp(PA, "Optional") == 0)
			return 1;
		else if (strcmp(PA, "Extra") == 0)
			return 2;
		else
			return 3;
	}
	else if (strcmp(PB, "Optional") == 0)
	{
		if (strcmp(PA, "Required") == 0)
			return -3;
		else if (strcmp(PA, "Important") == 0)
			return -2;
		else if (strcmp(PA, "Standard") == 0)
			return -1;
		else if (strcmp(PA, "Optional") == 0)
			return 0;
		else if (strcmp(PA, "Extra") == 0)
			return 1;
		else
			return 2;
	}
	else if (strcmp(PB, "Extra") == 0)
	{
		if (strcmp(PA, "Required") == 0)
			return -4;
		else if (strcmp(PA, "Important") == 0)
			return -3;
		else if (strcmp(PA, "Standard") == 0)
			return -2;
		else if (strcmp(PA, "Optional") == 0)
			return -1;
		else if (strcmp(PA, "Extra") == 0)
			return 0;
		else
			return 1;
	}
	else
		return 0;
}

int SortOrderList::icompare_status(const void * P1, const void * P2)
{
	return compare_status(P2, P1);
}

//old int SortOrderList::compare_status(pkgCache::PkgIterator & IterA, pkgCache::PkgIterator & IterB)
int SortOrderList::compare_status(const void * P1, const void * P2)
{
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	
	if ((*Cache)[IterA].Status < (*Cache)[IterB].Status)
		return -1;
	else if ((*Cache)[IterA].Status > (*Cache)[IterB].Status)
		return 1;
	else
		return 0;
}

int SortOrderList::icompare_specstat(const void * P1, const void * P2)
{
	return compare_status(P2, P1);
}

//old int SortOrderList::compare_status(pkgCache::PkgIterator & IterA, pkgCache::PkgIterator & IterB)
int SortOrderList::compare_specstat(const void * P1, const void * P2)
{
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	
	int WillA = ((*Cache)[IterA].InstBroken() ? PackageTree::Item::Broken :
				((*Cache)[IterA].NewInstall() ? PackageTree::Item::Install :
				((*Cache)[IterA].Upgrade() ? PackageTree::Item::Upgrade :
				((*Cache)[IterA].Downgrade() ? PackageTree::Item::Delete :
				((*Cache)[IterA].Install() ? PackageTree::Item::Install :
				((*Cache)[IterA].Delete () ? 
				(((*Cache)[IterA].iFlags & pkgDepCache:: Purge) ? PackageTree::Item::Purge : PackageTree::Item::Remove) :
				((*Cache)[IterA].iFlags & pkgDepCache::ReInstall) ? PackageTree::Item::ReInstall : PackageTree::Item::Keep ))))));
				
	int WillB = ((*Cache)[IterB].InstBroken() ? PackageTree::Item::Broken :
				((*Cache)[IterB].NewInstall() ? PackageTree::Item::Install :
				((*Cache)[IterB].Upgrade() ? PackageTree::Item::Upgrade :
				((*Cache)[IterB].Downgrade() ? PackageTree::Item::Delete :
				((*Cache)[IterB].Install() ? PackageTree::Item::Install :
				((*Cache)[IterB].Delete () ? 
				(((*Cache)[IterB].iFlags & pkgDepCache:: Purge) ? PackageTree::Item::Purge : PackageTree::Item::Remove) :
				((*Cache)[IterB].iFlags & pkgDepCache::ReInstall) ? PackageTree::Item::ReInstall : PackageTree::Item::Keep ))))));
				
	if (WillA < WillB)
		return -1;
	else if (WillA > WillB)
		return 1;
	else
		return 0;
}

int SortOrderList::icompare_archive_size(const void * P1, const void * P2)
{
	return compare_archive_size(P2, P1);
}

//old int SortOrderList::compare_archive_size(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB)
int SortOrderList::compare_archive_size(const void * P1, const void * P2)
{
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	pkgCache::VerIterator VerA = IterA.VersionList();
	pkgCache::VerIterator VerB = IterB.VersionList();

//old	return VerA->Size > VerB->Size;
	if (VerA->Size < VerB->Size)
		return -1;
	if (VerA->Size == VerB->Size)
		return 0;
	else
		return 1;
}

int SortOrderList::igroup_archive_size(const void * P1, const void * P2)
{
	return group_archive_size(P2, P1);
}

int SortOrderList::group_archive_size(const void * P1, const void * P2)
{
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	pkgCache::VerIterator VerA = IterA.VersionList();
	pkgCache::VerIterator VerB = IterB.VersionList();

	if ((VerA->Size < 1000 && VerB->Size < 1000) || (VerA->Size >= 1000000 && VerB->Size >= 1000000))
		return 0;
	else
		return (abs((int) log10 (VerA->Size) - (int) log10 (VerB->Size)) >= 1);
}

int SortOrderList::icompare_installed_size(const void * P1, const void * P2)
{
	return compare_installed_size(P2, P1);
}

//oldint SortOrderList::compare_installed_size(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB)
int SortOrderList::compare_installed_size(const void * P1, const void * P2)
{
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	pkgCache::VerIterator VerA = IterA.VersionList();
	pkgCache::VerIterator VerB = IterB.VersionList();

//old	return VerA->InstalledSize > VerB->InstalledSize;
	if (VerA->InstalledSize < VerB->InstalledSize)
		return -1;
	if (VerA->InstalledSize == VerB->InstalledSize)
		return 0;
	else
		return 1;
}

int SortOrderList::igroup_installed_size(const void * P1, const void * P2)
{
	return group_installed_size(P2, P1);
}

int SortOrderList::group_installed_size(const void * P1, const void * P2)
{
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	pkgCache::VerIterator VerA = IterA.VersionList();
	pkgCache::VerIterator VerB = IterB.VersionList();

	if ((VerA->InstalledSize < 1000 && VerB->InstalledSize < 1000) || (VerA->InstalledSize >= 1000000 && VerB->InstalledSize >= 1000000))
		return 0;
	else
		return (abs((int) log10 (VerA->InstalledSize) - (int) log10 (VerB->InstalledSize)) >= 1);
}

//old int SortOrderList::compare_distribution(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB)
//{
//	return 0;
//}

int SortOrderList::icompare_dep(const void * P1, const void * P2)
{
	return compare_dep(P2, P1);
}

int SortOrderList::compare_dep(const void * P1, const void * P2)
{
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	pkgCache::VerIterator VerA = IterA.VersionList();
	pkgCache::VerIterator VerB = IterB.VersionList();
	
	if (VerA.DependsList().end() != VerB.DependsList().end())
		if (VerA.DependsList().end())
			return 1;
		else
			return -1;
	
	return 0;
}

int SortOrderList::icompare_revdep(const void * P1, const void * P2)
{
	return compare_revdep(P2, P1);
}

int SortOrderList::compare_revdep(const void * P1, const void * P2)
{
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	
	if (IterA.RevDependsList().end() != IterB.RevDependsList().end())
		if (IterA.RevDependsList().end())
			return 1;
		else
			return -1;
	
	return 0;
}

int SortOrderList::icompare_suite(const void * P1, const void * P2)
{
	return compare_suite(P2, P1);
}

//old int SortOrderList::compare_suite(const pkgCache::Package * A, const pkgCache::Package * B)
int SortOrderList::compare_suite(const void * P1, const void * P2)
{
//old	const pkgCache::Package *A = *(pkgCache::Package **) P1;
//old	const pkgCache::Package *B = *(pkgCache::Package **) P2;
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) P1);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) P2);
	
	char *a, *b, *p;
//old	char *strp = Cache->GetCache()->StrP;
	
//old	a = strdup(strp + A->Section);
	a = strdup(IterA.Section());
//old	b = strdup(strp + B->Section);
	b = strdup(IterB.Section());

	if ((p = strchr(a, '/')))
		*p = 0;
	if ((p = strchr(b, '/')))
		*p = 0;

#define RET(x) {free(a); free(b); return(x); }

	if (strcasecmp(b, "non-us") == 0)
	{
		if (strcasecmp(a, "non-us") == 0)
			RET(0)
		else if (strcasecmp(a, "non-free") == 0)
			RET(1)
		else if (strcasecmp(a, "contrib") == 0)
			RET(2)
		else
			RET(3)
	}
	else if (strcasecmp(b, "non-free") == 0)
	{
		if (strcasecmp(a, "non-us") == 0)
			RET(-1)
		else if (strcasecmp(a, "non-free") == 0)
			RET(0)
		else if (strcasecmp(a, "contrib") == 0)
			RET(1)
		else
			RET(2)
	}
	else if (strcasecmp(b, "contrib") == 0)
	{
		if (strcasecmp(a, "non-us") == 0)
			RET(-2)
		else if (strcasecmp(a, "non-free") == 0)
			RET(-1)
		else if (strcasecmp(a, "contrib") == 0)
			RET(0)
		else
			RET(1)
	}
	else
	{
		if (strcasecmp(a, "non-us") == 0)
			RET(-3)
		else if (strcasecmp(a, "non-free") == 0)
			RET(-2)
		else if (strcasecmp(a, "contrib") == 0)
			RET(-1)
		else
			RET(0)
	}
}

int SortOrderList::compare_none(const void * P1, const void * P2)
{
	return -1;
}
