/* $Id: sorting.h,v 1.9 2000/11/17 14:56:21 ltd Exp $ */
#ifndef _sort_h
#define _sort_h

#include <func/apt.h>

class SortOrderList
{
	public:
		class SortIterator;
		friend class SortIterator;

		struct Node;

		// Define the different sort methods that can be added to this methodlist
/*		enum method_t { 
			None=0, 
			Section=1, 
			PkgName=2, 
			Priority=3, 
			Status=4, 
			Size=5, 
			Instsize=6,
			Filename=7,
			Suite=8 };*/
			
		enum method_t { 
			None,
			Section, 
			PkgName, 
			Priority, 
			Status,
			SpecStat, 
			Size, 
			Instsize,
			Filename,
			Suite,
			Dep,
			RevDep,
			MethodNum};

		// length of the list
		inline int Length() { return nodecount; };
		inline int TotalLength() { return MethodNum; };
		
		// List functions
//old		void Push(method_t);
		void Push(method_t, bool);
//old		void Insert(SortIterator &, method_t);
		void Insert(SortIterator &, method_t, bool);
		void Pop();
		void remove(SortIterator &);
		void Flush();

		typedef int (Comparer) (const void *, const void *);
//old		static int compare_section(const pkgCache::Package * A, const pkgCache::Package * B);
		static Comparer compare_section;
		static Comparer icompare_section;
//old		static int compare_package_name(const pkgCache::Package * A, const pkgCache::Package * B);
		static Comparer compare_package_name;
		static Comparer icompare_package_name;
//old		static int compare_priority(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB);
		static Comparer compare_priority;
		static Comparer icompare_priority;
//old		static int compare_status(pkgCache::PkgIterator & IterA, pkgCache::PkgIterator & IterB);
		static Comparer compare_status;
		static Comparer icompare_status;
		
		static Comparer compare_specstat;
		static Comparer icompare_specstat;
//old		static int compare_archive_size(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB);
		static Comparer compare_archive_size;
		static Comparer icompare_archive_size;
//old		static int compare_installed_size(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB);
		static Comparer compare_installed_size;
		static Comparer icompare_installed_size;
//old		static int compare_distribution(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB);

//old		static int compare_suite(const pkgCache::Package * A, const pkgCache::Package * B);
		static Comparer compare_suite;
		static Comparer icompare_suite;
		
		static Comparer compare_dep;
		static Comparer icompare_dep;
		
		static Comparer compare_revdep;
		static Comparer icompare_revdep;
		
		static Comparer compare_package_fname;
		static Comparer icompare_package_fname;
		
		static Comparer group_package_name;
		static Comparer igroup_package_name;
		static Comparer group_package_fname;
		static Comparer igroup_package_fname;
		static Comparer group_archive_size;
		static Comparer igroup_archive_size;
		static Comparer group_installed_size;
		static Comparer igroup_installed_size;
		
		static Comparer compare_none;
		
		Comparer* GetComparer(method_t myMethod, bool invert);
		Comparer* GetGrouper(method_t myMethod, bool invert);
		
//old		static int qsort_handler(const void *, const void *);

		// Inquisitory functions
		int DividerCount();
		int FindIncrement(enum method_t);
		bool Exists(enum method_t);

		void reorderseqids();
		void commit_to_config();
		void copy_from_config();

		// Iterator thingies
		SortIterator ListBegin();
		SortIterator ListEnd();
		SortIterator ListFind(method_t);

		// con/destructers
		SortOrderList();
		~SortOrderList();

	protected:
		unsigned int nodecount;
		Node *Head;
		Node *Tail;
};

struct SortOrderList::Node
{
	unsigned int seqid;
	enum SortOrderList::method_t thismethod;
	
	SortOrderList::Comparer *single_cmp, *group_cmp;
	
	struct SortOrderList::Node *next;
};

class SortOrderList::SortIterator
{
	public:
		friend class SortOrderList;
	
		inline void operator =(SortIterator B) { Pos = B.Pos; };
		void operator ++(int) { Pos = Pos->next; };
		inline void operator ++() {operator ++(0);};

		inline bool operator ==(SortIterator &B) { return Pos == B.Pos; };
		inline bool operator !=(SortIterator &B) { return Pos != B.Pos; };

		inline bool end() { return Pos == 0; };

		inline enum method_t method() { return Pos->thismethod; };
		inline unsigned int seq() { return Pos->seqid; };
		inline SortOrderList::Comparer* compare() { return Pos->single_cmp; };
		inline  SortOrderList::Comparer* group() { return Pos->group_cmp; };

		inline SortIterator(SortOrderList &Owner) { Pos = Owner.Head; };
		inline SortIterator(Node *NewPos) { Pos = NewPos; };
		inline SortIterator() {};

	protected:
		Node *Pos;
};

inline SortOrderList::SortIterator SortOrderList::ListBegin() { return SortIterator(Head); }
inline SortOrderList::SortIterator SortOrderList::ListEnd() { return SortIterator(Tail); }

extern SortOrderList *LastOrderCreated;

#endif

