/* $Id: sources.cc,v 1.3 2000/11/17 14:56:21 ltd Exp $ */
//old #define _CPP_IOSFWD

#include <std.h>
#include <parsers/strutl.h>
#include "sources.h"
#include <apt-pkg/configuration.h>
//old #include <fstream.h>

SourcesList::~SourcesList()
{
	SourceRecord *current, *tmp;

	current = head;
	while (current != NULL) {
		tmp = current->next;
		delete current;
		current = tmp;
	}
	head = tail = 0;
}
	
void SourcesList::AddNode(SourceRecord &rec)
{
	SourceRecord *newrec = new SourceRecord;
	*newrec = rec;
	newrec->prev = newrec->next = NULL;

	if (head == NULL) {
		head = newrec;
		tail = newrec;
	} else {
		tail->next = newrec;
		newrec->prev = tail;
		tail = newrec;
	}
}

bool SourcesList::ReadSources()
{
	char buf[512];
	const char *p;

//old	ifstream ifs(_config->FindFile("Dir::Etc::sourcelist").c_str(), ios::in );
	FILE *ifs = fopen (_config->FindFile("Dir::Etc::sourcelist").c_str(), "r" );

//old	if (!ifs != 0) return false; // cannot open file
	if (!ifs) return false; // cannot open file

//old	while (ifs.eof() == false) {
	while (get_line (ifs, buf) != EOF) {
		p = buf;
		SourceRecord rec;
		string Type;
		string URI;
		string Section;

//old		ifs.getline(buf, sizeof(buf));
				
		while (isspace(*p)) p++;
		if (*p == '#') {
			rec.Type = Disabled;
			p++;
		}
		
		while (isspace(*p)) p++;
			
		if (*p == '\r' || *p == '\n' || *p == 0) {
			rec.Type = Comment;
			rec.Comment = p;
			
			AddNode(rec);
			continue;
		}

		if (ParseQuoteWord(p, Type) == false ||
		    ParseQuoteWord(p, URI) == false ||
		    ParseQuoteWord(p, rec.Dist) == false ||
		    rec.SetType(Type) == false ||
		    rec.SetURI(URI) == false) {
			if (rec.Type == Disabled) {
				// treat as a comment field
				rec.Type = Comment;
				rec.Comment = buf;
			} else {
				// syntax error on line
				return false;
			}
		}

		// check for absolute dist
		if (rec.Dist.empty() == false && rec.Dist[rec.Dist.size()-1] == '/') {
			// make sure there's no section
			if (ParseQuoteWord(p, Section) == true)
				return false;

			rec.Dist = SubstVar(rec.Dist, "$(ARCH)",
				_config->Find("APT::Architecture"));
		
			AddNode(rec);
			continue;
		}

		const char *tmp = p;
		rec.NumSections = 0;
		while (ParseQuoteWord(p, Section) == true)
			rec.NumSections++;
		if (rec.NumSections > 0) {
			p = tmp;
			rec.Sections = new string[rec.NumSections];
			rec.NumSections = 0;
			while (ParseQuoteWord(p, Section) == true)
				rec.Sections[rec.NumSections++] = Section;
		}
		AddNode(rec);
	}

//old	ifs.close();
	fclose (ifs);
	return true;
}

SourcesList::SourceRecord *SourcesList::AddSource(RecType Type, string URI, string Dist, string *Sections, unsigned short count)
{
	SourceRecord rec;
	rec.Type = Type;

	if (rec.SetURI(URI) == false) return NULL;
	rec.Dist = Dist;
	rec.NumSections = count;
	rec.Sections = new string[count];
	for (unsigned int i = 0; i < count; i++)
		rec.Sections[i] = Sections[i];

	AddNode(rec);
	return tail;
}

void SourcesList::RemoveSource(SourceRecord *&rec)
{
	if (rec == head) {
		head = head->next;
		if (head != NULL) head->prev = NULL;
	}
	else if (rec == tail) {
		tail = tail->prev;
		if (tail != NULL) tail->next = NULL;
	}
	else if (rec != head && rec != tail) {
		rec->prev->next = rec->next;
		rec->next->prev = rec->prev;
	}

	delete rec;
	rec = 0;
}

bool SourcesList::UpdateSources()
{
//old	ofstream ofs(_config->FindFile("Dir::Etc::sourcelist").c_str(), ios::out);
	FILE *ofs = fopen (_config->FindFile("Dir::Etc::sourcelist").c_str(), "w" );
//old	if (!ofs != 0) return false;
	if (!ofs) return false;

	for (SourceRecord *current = head; current != NULL; current = current->next) {
		string S;

		if ((current->Type & Comment) != 0) {
			S = current->Comment;
		} else {
			if ((current->Type & Disabled) != 0)
				S = "# ";
			if ((current->Type & Deb) != 0)
				S += "deb ";
			else if ((current->Type & DebSrc) != 0)
				S += "deb-src ";

			S += current->URI;
			S += " ";

			S += current->Dist;
			S += " ";

			for (unsigned int J = 0; J < current->NumSections; J++) {
				S += current->Sections[J];
				S += " ";
			}
		}
//old		ofs << S << endl;
		fputs (S.c_str(), ofs);
		fputc ('\n', ofs);
	}

//old	ofs.close();
	fclose (ofs);
	return true;
}

bool SourcesList::SourceRecord::SetType(string S)
{
	if (S == "deb") 
		Type |= Deb;
	else if (S == "deb-src")
		Type |= DebSrc;
	else
		return false;
	return true;
}

bool SourcesList::SourceRecord::SetURI(string S)
{
	if (S.empty() == true) return false;
	if (S.find(':') == string::npos) return false;

	S = SubstVar(S, "$(ARCH)", _config->Find("APT::Architecture"));
	URI = S;
	
	// append a / to the end if one is not already there
	if (URI[URI.size() - 1] != '/') URI += '/';

	return true;
}

SourcesList::SourceRecord &SourcesList::SourceRecord::operator=(const SourceRecord &rhs)
{
	// Needed for a proper deep copy of the record; uses the string operator= to properly copy the strings
	Type = rhs.Type;
	URI = rhs.URI;
	Dist = rhs.Dist;
	Sections = new string[rhs.NumSections];
	for (unsigned int I = 0; I < rhs.NumSections; I++)
		Sections[I] = rhs.Sections[I];
	NumSections = rhs.NumSections;
	Comment = rhs.Comment;

	prev = rhs.prev;
	next = rhs.next;

	return *this;
}
/*old
ostream &operator<< (ostream &os, const SourcesList::SourceRecord &rec)
{
	os << "Type: ";
	if ((rec.Type & SourcesList::Comment) != 0) os << "Comment ";
	if ((rec.Type & SourcesList::Disabled) != 0) os << "Disabled ";
	if ((rec.Type & SourcesList::Deb) != 0) os << "Deb";
	if ((rec.Type & SourcesList::DebSrc) != 0) os << "DebSrc";
	os << endl;
	os << "URI: " << rec.URI << endl;
	os << "Dist: " << rec.Dist << endl;
	os << "Section(s):" << endl;
	for (unsigned int J = 0; J < rec.NumSections; J++) {
		cout << "\t" <<  rec.Sections[J] << endl;
	}
	os << endl;
	return os;
}
*/
